@@ -22,32 +22,30 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: browse-dnd.c,v 1.25 2005/05/05 04:48:16 dave_malcolm Exp $ */
-
-/*  typedef unsigned long ulong; */
-
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include  <config.h>
+#endif /* HAVE_CONFIG_H */
 #ifdef BROWSER_DND
 
 #include <stdio.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "browse.h" 
-#include "dn-browse.h" 
-#include "browse-dnd.h" 
-
+#include "browse-dnd.h"
+#include "debug.h"
 #include "errorchain.h"
+#include "gq-browser-node-dn.h"
+#include "gq-server-list.h"
+#include "gq-tab-browse.h"
 #include "ldapops.h"
 #include "ldif.h"
-#include "util.h" 
-#include "i18n.h" 
-#include "debug.h" 
+#include "util.h"
 
 typedef struct _dnd_refresh {
      GQTreeWidget *tree;
-     struct ldapserver *server;
+     GqServer *server;
      int options;
      char *dn;
 } dnd_refresh;
@@ -56,7 +54,7 @@ static GList *dnd_refresh_list = NULL;
 
 
 static dnd_refresh *new_dnd_refresh_dn(GQTreeWidget *tree, 
-				       struct ldapserver *server,
+				       GqServer *server,
 				       char *dn,
 				       int options)
 {
@@ -78,10 +76,10 @@ static dnd_refresh *new_dnd_refresh_node(GQTreeWidget *tree,
 					 GQTreeWidgetNode *node,
 					 int options)
 {
-     dn_browse_entry *entry;
-     entry = (dn_browse_entry *) gq_tree_get_node_data (tree, node);
+     GqBrowserNodeDn *entry;
+     entry = GQ_BROWSER_NODE_DN(gq_tree_get_node_data (tree, node));
 
-     if (IS_DN_ENTRY(entry)) {
+     if (GQ_IS_BROWSER_NODE_DN(entry)) {
 	  return new_dnd_refresh_dn(tree, server_from_node(tree, node),
 				    entry->dn, options);
      } else {
@@ -90,7 +88,7 @@ static dnd_refresh *new_dnd_refresh_node(GQTreeWidget *tree,
      }
 }
 
-static void dnd_refresh_free(dnd_refresh *dr) 
+static void dnd_refresh_free(dnd_refresh *dr)
 {
      if (dr) {
 	  if (dr->dn) g_free(dr->dn);
@@ -98,46 +96,6 @@ static void dnd_refresh_free(dnd_refresh *dr)
      }
 }
 
-#if 0
-static gboolean drag_drop(GtkWidget *ctreeroot,
-			  GdkDragContext *drag_context,
-			  gint x,
-			  gint y,
-			  guint time,
-			  gpointer user_data)
-{
-     GNode *node;
-     GQTreeWidgetNode *ctree_node;
-     struct node_entry *target_entry;
-     int answer;
-
-#ifdef DEBUG
-     if (debug & GQ_DEBUG_BROWSE_DND) {
-	  printf("dropped %08lx at x=%d, y=%d\n", (unsigned long) drag_context, x, y);
-     }
-#endif
-
-     ctree_node = gq_tree_get_node_at (GTK_CTREE(ctreeroot), x, y);
-     if (ctree_node == NULL) return FALSE;
-
-     node = (GNode *) gq_tree_get_node_data (GTK_CTREE(ctreeroot),
-						  GTK_CTREE_NODE(ctree_node));
-     if (node == NULL) return FALSE;
-     if ((target_entry = (struct node_entry *) node->data) == NULL) 
-	  return FALSE;
-
-     if (target_entry->status & IS_SERVER) return FALSE;
-
-     answer = question_popup(_("Do you really want to move this entry?"),
-			     _("Do you really want to move this entry?"));
-     if (!answer) return FALSE;
-
-     /* dropping OK */
-
-     return TRUE;
-}
-#endif
-
 typedef struct _motion_timer_data {
      GtkAdjustment *adj;
      gfloat change;
@@ -146,9 +104,9 @@ typedef struct _motion_timer_data {
 } motion_timer_data;
 
 /* We want the CTree to scroll up or down during dragging if we get
-   close to its border. We use a time for this. 
+   close to its border. We use a time for this.
 
-   Some Notes: 
+   Some Notes:
    * The closer we are to the border the faster we scroll.
    * The timer gets started by a drag motion callback
  */
@@ -270,7 +228,7 @@ static gboolean drag_and_select_free_elem(gpointer key,
 }
 
 /* Cleanup function for the drag and selection data hash */
-static void drag_and_select_data_free(GHashTable *hash) 
+static void drag_and_select_data_free(GHashTable *hash)
 {
      g_hash_table_foreach_remove(hash, drag_and_select_free_elem, NULL);
      g_hash_table_destroy(hash);
@@ -278,21 +236,19 @@ static void drag_and_select_data_free(GHashTable *hash)
 
 static void drag_begin(GtkWidget *ctreeroot,
 		       GdkDragContext *drag_context,
-		       struct tab *tab) 
+		       GqTab *tab)
 {
      /* store data we began dragging with for drag_data_get later on */
      GQTreeWidgetNode *ctree_node;
-     struct ldapserver *server;
-     browse_entry *entry;
+     GqServer *server;
+     GqBrowserNode *entry;
      GHashTable *seldata;
      char cbuf[20];
 
-     ctree_node = BROWSETAB(tab)->tree_row_selected;
-/*       node = (GNode *) gq_tree_get_node_data (GTK_CTREE(ctreeroot), */
-/*  						  GTK_CTREE_NODE(ctree_node)); */
+     ctree_node = GQ_TAB_BROWSE(tab)->tree_row_selected;
      server = server_from_node(GTK_CTREE(ctreeroot), ctree_node);
-     entry = (browse_entry *) gq_tree_get_node_data (GTK_CTREE(ctreeroot),
-							  ctree_node);
+     entry = GQ_BROWSER_NODE(gq_tree_get_node_data (GTK_CTREE(ctreeroot),
+							  ctree_node));
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
@@ -302,18 +258,18 @@ static void drag_begin(GtkWidget *ctreeroot,
      }
 #endif
 
-     if (IS_DN_ENTRY(entry)) {
-	  dn_browse_entry *dn_entry = (dn_browse_entry *) entry;
+     if (GQ_IS_BROWSER_NODE_DN(entry)) {
+	  GqBrowserNodeDn *dn_entry = GQ_BROWSER_NODE_DN(entry);
 
 	  seldata = g_hash_table_new(g_str_hash, g_str_equal);
 	  g_hash_table_insert(seldata, g_strdup("dn"), g_strdup(dn_entry->dn));
-	  g_hash_table_insert(seldata, g_strdup("nickname"), 
+	  g_hash_table_insert(seldata, g_strdup("nickname"),
 			      g_strdup(server->name));
-	  g_hash_table_insert(seldata, g_strdup("server"), 
+	  g_hash_table_insert(seldata, g_strdup("server"),
 			      g_strdup(server->ldaphost));
-	  snprintf(cbuf, sizeof(cbuf), "%d", server->ldapport);
+	  g_snprintf(cbuf, sizeof(cbuf), "%d", server->ldapport);
 	  g_hash_table_insert(seldata, g_strdup("port"), g_strdup(cbuf));
-	  
+
 	  /* dragging is always done recusively (for now) */
 	  g_hash_table_insert(seldata, g_strdup("recursively"), g_strdup("TRUE"));
 	  
@@ -341,7 +297,7 @@ static int dnd_refresh_sort_func(const dnd_refresh *a, const dnd_refresh *b)
 }
 
 
-static void do_refresh(int error_context, struct tab *tab) {
+static void do_refresh(int error_context, GqTab *tab) {
      GList *l;
      dnd_refresh *at_end = NULL;
 
@@ -394,7 +350,7 @@ static void do_refresh(int error_context, struct tab *tab) {
 
 static void drag_end(GtkWidget *ctreeroot,
 		     GdkDragContext *drag_context,
-		     struct tab *tab) 
+		     GqTab *tab) 
 {
      int ctx;
 #ifdef DEBUG
@@ -423,11 +379,11 @@ static void drag_selection_data_prepare(gpointer key,
 					gpointer value,
 					GByteArray *buf)
 {
-     char nul = 0;
+     guchar nul = '\0';
      g_byte_array_append(buf, key, strlen(key));
-     g_byte_array_append(buf, "=", 1);
+     g_byte_array_append(buf, (guchar*)"=", 1);
      g_byte_array_append(buf, value, strlen(value));
-     g_byte_array_append(buf, &nul, 1); 
+     g_byte_array_append(buf, &nul, 1);
 }
 
 static void drag_data_get(GtkWidget *ctreeroot,
@@ -435,12 +391,12 @@ static void drag_data_get(GtkWidget *ctreeroot,
 			  GtkSelectionData *data,
 			  guint info,
 			  guint time,
-			  struct tab *tab)   /* don't really need tab here */
+			  GqTab *tab)   /* don't really need tab here */
 {
-     GHashTable *selhash = gtk_object_get_data(GTK_OBJECT(ctreeroot), 
+     GHashTable *selhash = gtk_object_get_data(GTK_OBJECT(ctreeroot),
 					       "drag-and-selection-data");
      GByteArray *buf;
-     char nul = 0;
+     guchar nul = '\0';
 
      if (!selhash) {
 	  statusbar_msg(_("Could not find data to drag - internal error"));
@@ -449,7 +405,7 @@ static void drag_data_get(GtkWidget *ctreeroot,
      buf = g_byte_array_new();
 
      g_hash_table_foreach(selhash, (GHFunc) drag_selection_data_prepare, buf);
-     g_byte_array_append(buf, &nul, 1); 
+     g_byte_array_append(buf, &nul, 1);
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
@@ -523,13 +479,13 @@ static GHashTable *drag_selection_data_unpack(const char *data, int len)
 static void do_move_after_reception(GtkWidget *ctreeroot,
 				    GHashTable *selhash,
 				    GQTreeWidgetNode *target_ctree_node,
-				    dn_browse_entry *target_entry,
-				    struct ldapserver *target_server,
+				    GqBrowserNodeDn *target_entry,
+				    GqServer *target_server,
 				    int flags)
 {
      GQTreeWidgetNode *ctree_node;
      char *dn = NULL;
-     struct ldapserver *source_server;
+     GqServer *source_server;
      char *newdn = NULL;
      int context;
 
@@ -542,7 +498,8 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
      }
 #endif
 
-     source_server = server_by_name(g_hash_table_lookup(selhash, "nickname"));
+     source_server = gq_server_list_get_by_name(gq_server_list_get(),
+						g_hash_table_lookup(selhash, "nickname"));
      if (source_server == NULL) {
 	  statusbar_msg(_("Could not find source server by its nickname ('%s')!"),
 			(const char*)g_hash_table_lookup(selhash, "nickname"));
@@ -576,7 +533,7 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
 	  goto done;
      }
      
-     if (IS_DN_ENTRY(target_entry)) {
+     if (GQ_IS_BROWSER_NODE_DN(target_entry)) {
 	  newdn = move_entry(dn, source_server, 
 			     target_entry->dn, target_server,
 			     flags, move_progress, context);
@@ -637,9 +594,9 @@ static void drag_data_received(GtkWidget *ctreeroot,
 			       gpointer user_data)
 {
      GQTreeWidgetNode *ctree_node;
-     struct ldapserver *target_server;
+     GqServer *target_server;
      GHashTable *selhash = NULL;
-     dn_browse_entry *target_entry;
+     GqBrowserNodeDn *target_entry;
      int answer;
 
      if (data == NULL) {  /* PSt: I've seen this happen */
@@ -658,19 +615,19 @@ static void drag_data_received(GtkWidget *ctreeroot,
      if (ctree_node == NULL) return;
 
      /* NOTE: type has not yet been checked !!! */
-     target_entry = (dn_browse_entry *) 
+     target_entry = GQ_BROWSER_NODE_DN(
 	  gq_tree_get_node_data (GTK_CTREE(ctreeroot),
-				      GTK_CTREE_NODE(ctree_node));
+				      GTK_CTREE_NODE(ctree_node)));
 
-     if (! IS_DN_ENTRY(target_entry)) return;
-     target_server = server_from_node(GTK_CTREE(ctreeroot), 
+     if (! GQ_IS_BROWSER_NODE_DN(target_entry)) return;
+     target_server = server_from_node(GTK_CTREE(ctreeroot),
 				      GTK_CTREE_NODE(ctree_node));
 
      /* For now we allow recursive moves, as we _believe_ it
 	works even with a limited number of answers from
 	servers... */
 
-     answer = 
+     answer =
 	  question_popup(_("Do you really want to move this entry?"),
 			 _("Do you really want to move this entry recursively?\n"
 			   "Note that you should only do this if you have\n"
@@ -678,11 +635,11 @@ static void drag_data_received(GtkWidget *ctreeroot,
 			   "(the same holds for objects below it) as the\n"
 			   "original object(s) WILL BE REMOVED!\n\n"
 			   "USE AT YOUR OWN RISK!"));
-     
+
      if (answer) {
-	  selhash = drag_selection_data_unpack(data->data, 
+	  selhash = drag_selection_data_unpack((gchar*)data->data,
 					       data->length);
-	  
+
 	  if (selhash) {
 	       do_move_after_reception(ctreeroot, selhash, ctree_node,
 				       target_entry, target_server, 
@@ -715,13 +672,13 @@ static void drag_data_received(GtkWidget *ctreeroot,
 
 static void drag_data_delete(GtkWidget *widget,
 			     GdkDragContext *drag_context,
-			     struct tab *tab)
+			     GqTab *tab)
 {
      GQTreeWidgetNode *ctree_node;
      char *s = NULL, *dn = NULL;
-     struct ldapserver *server;
+     GqServer *server;
 
-     GHashTable *selhash = gtk_object_get_data(GTK_OBJECT(widget), 
+     GHashTable *selhash = gtk_object_get_data(GTK_OBJECT(widget),
 					       "drag-and-selection-data");
 
      s = g_hash_table_lookup(selhash, "nickname");
@@ -729,7 +686,7 @@ static void drag_data_delete(GtkWidget *widget,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_data_delete ctx=%08lx suggested=%d action=%d server=%s dn=%s\n", 
+	  printf("drag_data_delete ctx=%08lx suggested=%d action=%d server=%s dn=%s\n",
 		 (unsigned long) drag_context,
 		 drag_context->suggested_action,
 		 drag_context->action,
@@ -738,17 +695,17 @@ static void drag_data_delete(GtkWidget *widget,
      }
 #endif
 
-     server = server_by_name(s);
+     server = gq_server_list_get_by_name(gq_server_list_get(), s);
      if (!server) {
 	  return;
      }
      ctree_node = tree_node_from_server_dn(GQ_TREE_WIDGET(widget),
 					   server, dn);
-     
+
      if (ctree_node) {
 	  GQTreeWidgetNode *parent = gq_tree_get_parent_node (GQ_TREE_WIDGET (widget),
 							      ctree_node);
-  	  if (parent) {
+	  if (parent) {
 	       /* register that we have to refresh the parent node */
 
 	       dnd_refresh_list =
@@ -756,7 +713,7 @@ static void drag_data_delete(GtkWidget *widget,
 				  new_dnd_refresh_node(GTK_CTREE(widget),
 						       parent, 0)
 				  );
-/*    	       refresh_subtree_with_options(GTK_CTREE(widget), parent, 0); */
+/*	       refresh_subtree_with_options(GTK_CTREE(widget), parent, 0); */
 	  }
      }
 #ifdef DEBUG
@@ -764,39 +721,26 @@ static void drag_data_delete(GtkWidget *widget,
 	  printf("drag_data_delete done\n");
      }
 #endif
-     gtk_object_remove_data(GTK_OBJECT(widget), 
+     gtk_object_remove_data(GTK_OBJECT(widget),
 			    "drag-and-selection-data");
 }
 
-
-#if 0
-gboolean compare_drag_func(GQTreeWidget *ctreeroot,
-			   GQTreeWidgetNode *source_node,
-			   GQTreeWidgetNode *new_parent,
-			   GQTreeWidgetNode *new_sibling)
-{
-    printf("compare_drag_func\n");
-    return TRUE;
-}
-#endif
-
-
-void copy_entry(GtkWidget *widget, struct tab *tab)
+void copy_entry(GtkWidget *widget, GqTab *tab)
 {
      GQTreeWidget *ctree;
      int have_sel;
-     struct ldapserver *server;
-     dn_browse_entry *entry;
+     GqServer *server;
+     GqBrowserNodeDn *entry;
      GQTreeWidgetNode *ctree_node;
      GHashTable *seldata;
      char cbuf[20];
 
-     ctree = BROWSETAB(tab)->ctreeroot;
+     ctree = GQ_TAB_BROWSE(tab)->ctreeroot;
 
-     ctree_node = BROWSETAB(tab)->tree_row_popped_up;
-     entry = (dn_browse_entry *) gq_tree_get_node_data (ctree, ctree_node);
+     ctree_node = GQ_TAB_BROWSE(tab)->tree_row_popped_up;
+     entry = GQ_BROWSER_NODE_DN(gq_tree_get_node_data (ctree, ctree_node));
      if (entry == NULL) return;
-     if (!IS_DN_ENTRY(entry)) return;
+     if (!GQ_IS_BROWSER_NODE_DN(entry)) return;
 
      server = server_from_node(ctree, ctree_node);
 
@@ -804,7 +748,7 @@ void copy_entry(GtkWidget *widget, struct tab *tab)
      g_hash_table_insert(seldata, g_strdup("dn"), g_strdup(entry->dn));
      g_hash_table_insert(seldata, g_strdup("nickname"), g_strdup(server->name));
      g_hash_table_insert(seldata, g_strdup("server"), g_strdup(server->ldaphost));
-     snprintf(cbuf, sizeof(cbuf), "%d", server->ldapport);
+     g_snprintf(cbuf, sizeof(cbuf), "%d", server->ldapport);
      g_hash_table_insert(seldata, g_strdup("port"), g_strdup(cbuf));
 
      gtk_object_set_data_full(GTK_OBJECT(ctree), 
@@ -825,18 +769,18 @@ void copy_entry(GtkWidget *widget, struct tab *tab)
 }
 
 
-void copy_entry_all(GtkWidget *widget, struct tab *tab)
+void copy_entry_all(GtkWidget *widget, GqTab *tab)
 {
      GHashTable *selhash = NULL;
      GtkWidget *ctreeroot;
 
      copy_entry(widget, tab);
 
-     ctreeroot = GTK_WIDGET(BROWSETAB(tab)->ctreeroot);
+     ctreeroot = GTK_WIDGET(GQ_TAB_BROWSE(tab)->ctreeroot);
 
      selhash = gtk_object_get_data(GTK_OBJECT(ctreeroot), 
 				   "drag-and-selection-data");
-     
+
      g_hash_table_insert(selhash, g_strdup("recursively"), g_strdup("TRUE"));
 
 #ifdef DEBUG
@@ -852,8 +796,9 @@ static void get_selection_string(GtkWidget *ctree,
 				 GtkSelectionData *data,
 				 GHashTable *selhash)
 {
-     struct ldapserver *server = 
-	  server_by_name(g_hash_table_lookup(selhash, "nickname"));
+     GqServer *server =
+	  gq_server_list_get_by_name(gq_server_list_get(),
+				     g_hash_table_lookup(selhash, "nickname"));
      char *dn = g_hash_table_lookup(selhash, "dn");
      LDAPControl c, *ctrls[2] = { NULL, NULL } ;
      int scope;
@@ -867,9 +812,9 @@ static void get_selection_string(GtkWidget *ctree,
      char *attrs[] = {
 	  LDAP_ALL_USER_ATTRIBUTES,
 	  "ref",
-	  NULL 
+	  NULL
      };
-	       
+
      int ctx = error_new_context(_("Getting selection string"), ctree);
 
 #ifdef DEBUG
@@ -915,32 +860,32 @@ static void get_selection_string(GtkWidget *ctree,
 	  rc = ldap_search_s(ld, dn, scope, "(objectClass=*)", attrs, 0, &res);
      }
 
-     if (rc == LDAP_SUCCESS) { 
+     if (rc == LDAP_SUCCESS) {
 	  out = g_string_new("");
 
 	  for( e = ldap_first_entry(ld, res) ; e != NULL ;
 	       e = ldap_next_entry(ld, e) ) {
 	       ldif_entry_out(out, ld, e, ctx);
 	  }
-	  
+
 	  gtk_selection_data_set(data,
 				 GDK_SELECTION_TYPE_STRING,
 				 8, /* character data */
-				 out->str,
+				 (guchar*)out->str,
 				 out->len
 				 );
 	  ok = TRUE;
      } else if (rc == LDAP_SERVER_DOWN) {
 	  server->server_down++;
-	  error_push(ctx, 
+	  error_push(ctx,
 		     _("Error searching below '%1$s': %2$s"),
 		     dn, ldap_err2string(rc));
-	  push_ldap_addl_error(ld, ctx); 
+	  push_ldap_addl_error(ld, ctx);
      } else {
-	  error_push(ctx, 
+	  error_push(ctx,
 		     _("Error searching below '%1$s': %2$s"),
 		     dn, ldap_err2string(rc));
-	  push_ldap_addl_error(ld, ctx); 
+	  push_ldap_addl_error(ld, ctx);
      }
 
      if (!ok) {
@@ -954,7 +899,7 @@ static void get_selection_string(GtkWidget *ctree,
 
  fail:
      if (out) g_string_free(out, TRUE);
-     if (res) ldap_msgfree(res); 
+     if (res) ldap_msgfree(res);
      if (ld) close_connection(server, FALSE);
 
  done:
@@ -968,10 +913,10 @@ static void get_selection_gq(GtkWidget *ctree,
 			     GHashTable *selhash)
 {
      GByteArray *buf = g_byte_array_new();
-     char nul = 0;
+     guchar nul = '\0';
 
      g_hash_table_foreach(selhash, (GHFunc) drag_selection_data_prepare, buf);
-     g_byte_array_append(buf, &nul, 1); 
+     g_byte_array_append(buf, &nul, 1);
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
@@ -1000,7 +945,7 @@ static void get_selection(GtkWidget *ctree,
 			  GtkSelectionData *data,
 			  guint info,
 			  guint time,
-			  struct tab *tab)  /* XXX don't need to pass tab here */
+			  GqTab *tab)  /* XXX don't need to pass tab here */
 {
      GHashTable *selhash = gtk_object_get_data(GTK_OBJECT(ctree), 
 					       "drag-and-selection-data");
@@ -1016,12 +961,12 @@ static void get_selection(GtkWidget *ctree,
 }
 
 
-void paste_entry(GtkWidget *widget, struct tab *tab)
+void paste_entry(GtkWidget *widget, GqTab *tab)
 {
      GtkWidget *ctree;
      int ctx;
 
-     ctree = GTK_WIDGET(BROWSETAB(tab)->ctreeroot);
+     ctree = GTK_WIDGET(GQ_TAB_BROWSE(tab)->ctreeroot);
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
@@ -1044,15 +989,15 @@ void paste_entry(GtkWidget *widget, struct tab *tab)
 static void selection_received(GtkWidget *ctree,
 			       GtkSelectionData *data,
 			       guint time,
-			       struct tab *tab) 
+			       GqTab *tab) 
 {
      GQTreeWidgetNode *ctree_node;
-     dn_browse_entry *target_entry;
-     struct ldapserver *target_server;
+     GqBrowserNodeDn *target_entry;
+     GqServer *target_server;
      GQTreeWidget *ctreeroot;
      GHashTable *selhash = NULL;
 
-     ctreeroot = BROWSETAB(tab)->ctreeroot;
+     ctreeroot = GQ_TAB_BROWSE(tab)->ctreeroot;
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
@@ -1061,20 +1006,20 @@ static void selection_received(GtkWidget *ctree,
      }
 #endif
 
-     ctree_node = BROWSETAB(tab)->tree_row_popped_up;
-     target_entry = 
-	  (dn_browse_entry *) gq_tree_get_node_data (ctreeroot,
-						     ctree_node);
+     ctree_node = GQ_TAB_BROWSE(tab)->tree_row_popped_up;
+     target_entry =
+	  GQ_BROWSER_NODE_DN(gq_tree_get_node_data (ctreeroot,
+						     ctree_node));
      if (target_entry == NULL) return;
-     if (!IS_DN_ENTRY(target_entry)) return;
-     
+     if (!GQ_IS_BROWSER_NODE_DN(target_entry)) return;
+
      target_server = server_from_node(ctreeroot, ctree_node);
 
      /* For now we allow recursive moves, as we _believe_ it
 	works even with a limited number of answers from
 	servers... */
-     
-     selhash = drag_selection_data_unpack(data->data, 
+
+     selhash = drag_selection_data_unpack((gchar*)data->data,
 					  data->length);
 
      if (selhash) {
@@ -1089,7 +1034,7 @@ static void selection_received(GtkWidget *ctree,
 
 	  if (flags & MOVE_RECURSIVELY) {
 	       msg = _("Do you really want to paste this entry recursively?\n"
-		       "Note that you might not be able to really copy everything\n" 
+		       "Note that you might not be able to really copy everything\n"
 		       "in case you do not have access to ALL attributes of the\n"
 		       "object(s) to paste (the same holds for objects below it).\n"
 		       "USE AT YOUR OWN RISK!");
@@ -1114,7 +1059,7 @@ static void selection_received(GtkWidget *ctree,
 }
 
 
-void browse_dnd_setup(GtkWidget *ctreeroot, struct tab *tab) 
+void browse_dnd_setup(GtkWidget *ctreeroot, GqTab *tab) 
 {
      GtkTargetEntry *target_entry;
 
@@ -1128,14 +1073,14 @@ void browse_dnd_setup(GtkWidget *ctreeroot, struct tab *tab)
 			      gdk_atom_intern("gq-browse-ctree", FALSE),
 			      1);
 
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), 
+     g_signal_connect(ctreeroot, 
 			"selection-get",
-			(GtkSignalFunc) get_selection,
-			(gpointer) tab);
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), 
+			G_CALLBACK(get_selection),
+			tab);
+     g_signal_connect(ctreeroot, 
 			"selection-received",
-			(GtkSignalFunc) selection_received,
-			(gpointer) tab);
+			G_CALLBACK(selection_received),
+			tab);
 
      target_entry = (GtkTargetEntry*) g_malloc(sizeof(GtkTargetEntry));
      target_entry->target = "gq-browse-ctree";
@@ -1155,39 +1100,34 @@ void browse_dnd_setup(GtkWidget *ctreeroot, struct tab *tab)
      gtk_clist_set_button_actions(GTK_CLIST(ctreeroot), 1, 
 				  GTK_BUTTON_SELECTS | GTK_BUTTON_DRAGS | GTK_BUTTON_EXPANDS);
 
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-data-received",
-			GTK_SIGNAL_FUNC(drag_data_received),
-			(gpointer) NULL);
+     g_signal_connect(ctreeroot, "drag-data-received",
+			G_CALLBACK(drag_data_received),
+			NULL);
 
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-data-get",
-			 GTK_SIGNAL_FUNC(drag_data_get),
-			(gpointer) tab);
+     g_signal_connect(ctreeroot, "drag-data-get",
+			 G_CALLBACK(drag_data_get),
+			tab);
      
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-data-delete",
-			 GTK_SIGNAL_FUNC(drag_data_delete),
-			 (gpointer) tab);
+     g_signal_connect(ctreeroot, "drag-data-delete",
+			 G_CALLBACK(drag_data_delete),
+			 tab);
 
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-motion",
-			GTK_SIGNAL_FUNC(drag_motion),
-			(gpointer) NULL);
+     g_signal_connect(ctreeroot, "drag-motion",
+			G_CALLBACK(drag_motion),
+			NULL);
 
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-begin",
-			GTK_SIGNAL_FUNC(drag_begin),
-			(gpointer) tab);
+     g_signal_connect(ctreeroot, "drag-begin",
+			G_CALLBACK(drag_begin),
+			tab);
 
 
      /* HACK alert */
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-end",
-			GTK_SIGNAL_FUNC(drag_end),
-			(gpointer) tab);
+     g_signal_connect(ctreeroot, "drag-end",
+			G_CALLBACK(drag_end),
+			tab);
 
-#if 0
-     gtk_signal_connect(GTK_OBJECT(ctreeroot), "drag-drop",
-			GTK_SIGNAL_FUNC(drag_drop),
-			(gpointer) target_entry);
-#endif
 /*       gtk_ctree_set_drag_compare_func(GTK_CTREE(ctreeroot), */
-/*  				     compare_drag_func); */
+/*				     compare_drag_func); */
 
      g_free(target_entry);
      target_entry = NULL;
@@ -1196,7 +1136,7 @@ void browse_dnd_setup(GtkWidget *ctreeroot, struct tab *tab)
 
 #endif /* BROWSER_DND */
 
-/* 
+/*
    Local Variables:
    c-basic-offset: 5
    End:
