@@ -24,7 +24,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: browse-dnd.h,v 1.5 2003/10/17 05:47:49 stamfest Exp $ */
+/* $Id: browse-dnd.h 952 2006-08-26 11:17:35Z herzi $ */
 
 #ifndef GQ_BROWSE_DND_H_INCLUDED
 #define GQ_BROWSE_DND_H_INCLUDED
@@ -32,7 +32,7 @@
 #include <gtk/gtk.h>
 #include "common.h"
 
-#include "mainwin.h"  /* struct tab */
+#include "gq-tab.h"  /* GqTab */
 
 /* Flags used in new_dnd_refresh on top of the options for
    refresh_subtree_with_options. Thus these option values share the
@@ -40,10 +40,10 @@
 
 #define SELECT_AT_END		256
 
-void browse_dnd_setup(GtkWidget *ctreeroot, struct tab *tab);
-void copy_entry(GtkWidget *widget, struct tab *tab);
-void copy_entry_all(GtkWidget *widget, struct tab *tab);
-void paste_entry(GtkWidget *widget, struct tab *tab);
+void browse_dnd_setup(GtkWidget *ctreeroot, GqTab *tab);
+void copy_entry(GtkWidget *widget, GqTab *tab);
+void copy_entry_all(GtkWidget *widget, GqTab *tab);
+void paste_entry(GtkWidget *widget, GqTab *tab);
 
 #endif
 
