@@ -1,6 +1,7 @@
 /*
     GQ -- a GTK-based LDAP client
-    Copyright (C) 1998-2001 Bert Vermeulen
+    Copyright (C) 1998-2003 Bert Vermeulen
+    Copyright (C) 2002-2003 Peter Stamfest
 
     This program is released under the Gnu General Public License with
     the additional exemption that compiling, linking, and/or using
@@ -21,41 +22,59 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: debug.c,v 1.4 2002/06/09 15:24:20 stamfest Exp $ */
+/* $Id: debug.c 947 2006-08-24 23:10:20Z herzi $ */
 
 #include <stdio.h>
+
+#ifdef HAVE_CONFIG_H
+# include  <config.h>
+#endif /* HAVE_CONFIG_H */
+
+#ifdef HAVE_BACKTRACE
+#   include <execinfo.h>
+#   include <stdlib.h>
+#endif /* HAVE_BACKTRACE */
+
 #include "debug.h"
 #include "configfile.h"
 #include "common.h"
 
-int num_mallocs = 0;
-int max_mallocs = 0;
 
 
-void *gq_malloc(size_t size, char *msg)
+static int num_mallocs = 0;
+static int max_mallocs = 0;
+
+#ifdef DEBUG
+int debug = 0;
+#endif
+
+void *gq_malloc(size_t size, const char *msg)
 {
-
      void *newmem;
-
+     
      newmem = malloc(size);
-     printf("+ gq_malloc: %s (%d bytes @ 0x%x)\n", msg, size,
-	    (unsigned int) newmem);
-
-     num_mallocs++;
-     max_mallocs++;
-
+#ifdef DEBUG
+     if (debug & GQ_DEBUG_MALLOC) {
+	  printf("+ gq_malloc: %s (%d bytes @ 0x%x)\n", msg, size,
+		 (unsigned int) newmem);
+	  
+	  num_mallocs++;
+	  max_mallocs++;
+     }
+#endif
      return(newmem);
 }
 
 
-void gq_free(void *mem, char *msg)
+void gq_free(void *mem, const char *msg)
 {
-
-     printf("- gq_free: %s (0x%x)\n", msg, (unsigned int) mem);
-     num_mallocs--;
-
+#ifdef DEBUG
+     if (debug & GQ_DEBUG_MALLOC) {
+	  printf("- gq_free: %s (0x%x)\n", msg, (unsigned int) mem);
+	  num_mallocs--;
+     }
+#endif
      free(mem);
-
 }
 
 
@@ -73,61 +92,7 @@ void report_num_mallocs(void)
 }
 
 
-void dump_ldapservers(void)
-{
-     struct ldapserver *server;
-     int i;
-
-     i = 1;
-     server = config.ldapservers;
-     while(server) {
-	  printf("server %d: %s\n", i, server->name);
-	  server = server->next;
-	  i++;
-     }
-
-}
-
-
-
-
-#if 0
-
-void hexdump_plain(unsigned char *p, int n)
-{
-     int i;
-
-     for (i = 0 ; i < n ; i++, p++) {
-	  printf("%c", isprint(*p) ? *p : '.');
-     }
-}
-
-void hexdump(char *msg, unsigned char *p, int n)
-{
-     int i;
-     unsigned char *d = p;
-
-     printf("%s %08lx\n", msg, (long) p);
-
-     for (i = 0 ; i < n ; i++, p++) {
-	  printf("%02x ", *p);
-	  if (i % 16 == 15) {
-	       printf("   ");
-	       hexdump_plain(d, 16);
-	       d = p; d++;
-	       printf("\n");
-	  }
-     }
-     if (i % 16 != 0) {
-	  hexdump_plain(d, 16);
-	  d = p; d++;
-	  printf("\n");
-     }
-}
-
-#endif
-
-#ifdef MEMDEBUG
+#ifdef DEBUG
 #ifdef HAVE_MALLINFO
 static int memstat_timeout_id = -1;
 static int memstat_activity = 0;
@@ -153,4 +118,70 @@ void init_memstat_timer() {
     }
 }
 #endif /* HAVE_MALLINFO */
-#endif /* MEMDEBUG */
+
+#ifdef HAVE_BACKTRACE
+
+/* taken from the documentation of the GNU libc */
+void sprint_trace(GString *str)
+{
+     void *array[200];
+     size_t size;
+     char **strings;
+     size_t i;
+     
+     size    = backtrace(array, (sizeof(array) / sizeof(array[0])) );
+     strings = backtrace_symbols(array, size);
+     
+     if (strings) {
+	  for (i = 0; i < size; i++) {
+	       g_string_append(str, strings[i]);
+	       g_string_append(str, "\n");
+	  }
+	  free(strings);
+     }
+}
+
+/* taken from the documentation of the GNU libc */
+void print_trace(void)
+{
+     void *array[200];
+     size_t size;
+     char **strings;
+     size_t i;
+     
+     size    = backtrace(array, (sizeof(array) / sizeof(array[0])) );
+     strings = backtrace_symbols(array, size);
+     
+     if (strings) {
+	  fprintf(stderr, "----------------------------------------\n");
+	  
+	  for (i = 0; i < size; i++) {
+	       fprintf(stderr, "%s\n", strings[i]);
+	  }
+	  free(strings);
+     }
+}
+#else /* BACKTRACE */
+
+void sprint_trace(GString *str)
+{
+}
+
+void print_trace(void) 
+{
+}
+
+#endif /* BACKTRACE */
+
+#endif /* DEBUG */
+
+
+
+
+
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
