@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_binary.c,v 1.18 2006/04/05 00:53:55 herzi Exp $ */
+/* $Id: dt_binary.c 952 2006-08-26 11:17:35Z herzi $ */
 
 #include <string.h>
 #include <sys/types.h>
@@ -33,24 +33,25 @@
 #include <ctype.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #define GTK_ENABLE_BROKEN  /* for the text widget - should be replaced - FIXME */
 #include <gtk/gtk.h>
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 
-#include "gq.h"   /* GdkFont *big_fixed */
 #include "common.h"
+#include "gq-tab-browse.h"
 #include "util.h"
 #include "errorchain.h"
 #include "formfill.h"
 #include "input.h"
 #include "tinput.h"
-#include "browse.h"
 #include "encode.h"
 #include "ldif.h" /* for b64_decode */
 #include "dt_binary.h"
-#include "i18n.h"
 
 #define DT_BINARY_ENTRY		1
 #define DT_BINARY_TEXT		2
@@ -63,35 +64,8 @@
 
 #define DT_BINARY_ENCODING_MASK	(DT_BINARY_HEX|DT_BINARY_BASE64|DT_BINARY_PLAIN)
 
-static dt_binary_handler dt_binary_handler_vtab = {
-     {
-	  "Binary", 
-	  TRUE,
-	  FALSE,
-	  dt_binary_get_widget, /* reused from dt_generic_binary */
-	  dt_binary_get_data,
-	  dt_generic_binary_set_data, /* reused from dt_generic_binary */
-	  bervalLDAPMod
-     },
-     dt_b64_encode, /* encode */
-     dt_b64_decode, /* decode */
 
-     dt_binary_get_data_widget, /* get_data_widget */
-     dt_binary_store_data, /* store_data */
-     dt_binary_delete_data, /* delete_data */
-
-     dt_generic_binary_show_entries, /* show_entries */
-};
-
-#if GTK_MAJOR < 2
-static GtkStyle *fixed_style = NULL;
-#else
 static PangoFontDescription *fixed_font = NULL;
-#endif
-
-display_type_handler *dt_binary_get_handler() {
-     return (display_type_handler *) &dt_binary_handler_vtab;
-}
 
 static GByteArray *dt_binary_get_data_internal(struct formfill *form,
 					       GtkWidget *data_widget);
@@ -117,12 +91,12 @@ struct encoding_data {
 };
 
 static void dt_binary_switch_to_encoding(struct formfill *form,
-					 GtkWidget *hbox, 
-					 GtkWidget *data_widget, 
+					 GtkWidget *hbox,
+					 GtkWidget *data_widget,
 					 int newtype);
 
 static void dt_binary_set_encoding_in_menu(struct formfill *form,
-					   GtkWidget *menu_widget, 
+					   GtkWidget *menu_widget,
 					   int flags)
 {
      GList *children;
@@ -132,7 +106,7 @@ static void dt_binary_set_encoding_in_menu(struct formfill *form,
 	   children ;  children = children->next ) {
 	  enc =  gtk_object_get_data(GTK_OBJECT(children->data), "encoding");
 	  if (enc && (enc->enc & flags)) {
-	       GtkWidget *label = 
+	       GtkWidget *label =
 		    gtk_object_get_data(GTK_OBJECT(enc->hbox), "label");
 
 	       gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(children->data), TRUE);
@@ -144,48 +118,48 @@ static void dt_binary_set_encoding_in_menu(struct formfill *form,
 
 
 
-static void dt_binary_encoding_activated(GtkWidget *widget, 
+static void dt_binary_encoding_activated(GtkWidget *widget,
 					 struct encoding_data *enc)
 {
-     GtkWidget *label = 
+     GtkWidget *label =
 	  gtk_object_get_data(GTK_OBJECT(enc->hbox), "label");
 
      gtk_label_set_text(GTK_LABEL(label), enc->code);
 
      dt_binary_switch_to_encoding(enc->form,
-				  enc->hbox, 
+				  enc->hbox,
 				  enc->data_widget, enc->enc);
 }
 
 GtkWidget *dt_binary_get_widget(int error_context,
-				struct formfill *form, 
+				struct formfill *form,
 				GByteArray *data,
-				GtkSignalFunc *activatefunc,
-				gpointer funcdata) 
+				GCallback *activatefunc,
+				gpointer funcdata)
 {
-     GtkWidget *w = dt_generic_binary_get_widget(error_context, form, data, 
+     GtkWidget *w = dt_generic_binary_get_widget(error_context, form, data,
 						 activatefunc, funcdata);
 
 /*       GtkWidget *vbox = dt_generic_binary_retrieve_vbox_widget(w); */
      GtkWidget *menu = dt_generic_binary_retrieve_menu_widget(w);
      GtkWidget *data_widget = dt_generic_binary_retrieve_data_widget(w);
-     
+
      GSList *group = NULL;
      GtkWidget *item, *s, *label;
      int i;
      struct encoding_data *enc;
-     
+
      s = gtk_hseparator_new();
      gtk_widget_show(s);
 
      item = gtk_menu_item_new();
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(item), s);
-     
+
      gtk_menu_append(GTK_MENU(menu), item);
-    
+
      for(i = 0; encodings[i].token ; i++) {
-	  item = gtk_radio_menu_item_new_with_label(group, 
+	  item = gtk_radio_menu_item_new_with_label(group,
 						    encodings[i].keyword);
 
 	  group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
@@ -199,10 +173,10 @@ GtkWidget *dt_binary_get_widget(int error_context,
 	  enc->code = (char*) encodings[i].data;
 
 	  gtk_widget_show(item);
-	  gtk_signal_connect(GTK_OBJECT(item), "activate",
-			     GTK_SIGNAL_FUNC(dt_binary_encoding_activated),
+	  g_signal_connect(item, "activate",
+			     G_CALLBACK(dt_binary_encoding_activated),
 			     enc);
-	  
+
 	  gtk_menu_append(GTK_MENU(menu), item);
 
 	  /* store callback info with the object as well - this allows
@@ -212,7 +186,7 @@ GtkWidget *dt_binary_get_widget(int error_context,
 
      s = gtk_alignment_new(0,0,0,0);
      gtk_widget_show(s);
-          
+
      label = gtk_label_new("P");
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(s), label);
@@ -224,9 +198,9 @@ GtkWidget *dt_binary_get_widget(int error_context,
 }
 
 
-GtkWidget *dt_binary_get_data_widget(struct formfill *form, 
-				     GtkSignalFunc *activatefunc,
-				     gpointer funcdata) 
+GtkWidget *dt_binary_get_data_widget(struct formfill *form,
+				     GCallback *activatefunc,
+				     gpointer funcdata)
 {
      GtkWidget *data_widget;
      GtkWidget *inputbox;
@@ -250,14 +224,14 @@ GtkWidget *dt_binary_get_data_widget(struct formfill *form,
      return data_widget;
 }
 
-static void dt_binary_store_data_internal(struct formfill *form, 
+static void dt_binary_store_data_internal(struct formfill *form,
 					  GtkWidget *hbox,
 					  GtkWidget *data_widget,
 					  const GByteArray *data)
 {
      int *flags = gtk_object_get_data(GTK_OBJECT(data_widget), "flags");
      GtkWidget *widget = GTK_BIN(data_widget)->child;
-     
+
      if(data) {
 	 GByteArray *encoded;
 
@@ -280,7 +254,7 @@ static void dt_binary_store_data_internal(struct formfill *form,
 		   *flags = (*flags & ~DT_BINARY_ENCODING_MASK) | DT_BINARY_BASE64;
 		   /* adjust menu! */
 
-		   dt_binary_set_encoding_in_menu(form, 
+		   dt_binary_set_encoding_in_menu(form,
 						  dt_generic_binary_retrieve_menu_widget(hbox),
 						  *flags);
 	      } else {
@@ -290,14 +264,24 @@ static void dt_binary_store_data_internal(struct formfill *form,
 	      encoded = dt_b64_encode((gchar*)data->data, data->len);
 	 }
 	 if (*flags & DT_BINARY_TEXT) {
+	      GdkFont *font = gdk_font_load("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*,fixed");
 	      GtkText *text = GTK_TEXT(widget);
-	      
+
+	      if(!font) {
+		      PangoFontDescription* desc = pango_font_description_from_string("Fixed");
+		      font = gdk_font_from_description(desc);
+		      pango_font_description_free(desc);
+	      }
+
+	      g_return_if_fail(font);
+
 	      gtk_text_freeze(text);
 	      gtk_text_set_point(text, 0);
 	      gtk_text_forward_delete(text, gtk_text_get_length(text));
-	      gtk_text_insert(text, big_fixed, NULL, NULL,
+	      gtk_text_insert(text, font, NULL, NULL,
 			      (gchar*)encoded->data, encoded->len);
 	      gtk_text_thaw(text);
+	      gdk_font_unref(font);
 	 } else if (*flags & DT_BINARY_ENTRY) {
 	      gtk_entry_set_text(GTK_ENTRY(widget), (gchar*)encoded->data);
 	 }
@@ -308,9 +292,9 @@ static void dt_binary_store_data_internal(struct formfill *form,
 }
 
 static void dt_binary_switch_to_type(struct formfill *form,
-				     GtkWidget *hbox, 
-				     GtkWidget *data_widget, 
-				     int newtype) 
+				     GtkWidget *hbox,
+				     GtkWidget *data_widget,
+				     int newtype)
 {
      GtkWidget *widget = NULL;
      GtkWidget *oldwidget = GTK_BIN(data_widget)->child;
@@ -326,26 +310,18 @@ static void dt_binary_switch_to_type(struct formfill *form,
 	       gtk_widget_show(widget);
 	       gtk_text_set_editable(GTK_TEXT(widget), TRUE);
 
-#if GTK_MAJOR >= 2
 	       if (!fixed_font) {
 		    fixed_font = pango_font_description_from_string("fixed");
 	       }
 	       gtk_widget_modify_font(widget, fixed_font);
-#else
-	       if (!fixed_style) {
-		    fixed_style = gtk_style_new();
-		    fixed_style->font = gdk_font_load("fixed");
-	       }
-	       gtk_widget_set_style(widget, fixed_style);
-#endif
-	  } 
+	  }
 	  if (newtype & DT_BINARY_ENTRY) {
 	       widget = gtk_entry_new();
 	       gtk_widget_show(widget);
 	       gtk_entry_set_editable(GTK_ENTRY(widget), TRUE);
 	  }
 	  *flags = (*flags & ~DT_BINARY_WIDGET_MASK) | newtype;
-	  
+
 	  if (oldwidget) {
 	       gtk_container_remove(GTK_CONTAINER(data_widget), oldwidget);
 	  }
@@ -358,16 +334,16 @@ static void dt_binary_switch_to_type(struct formfill *form,
 }
 
 static void dt_binary_switch_to_encoding(struct formfill *form,
-					 GtkWidget *hbox, 
-					 GtkWidget *data_widget, 
-					 int newtype) 
+					 GtkWidget *hbox,
+					 GtkWidget *data_widget,
+					 int newtype)
 {
      int *flags = gtk_object_get_data(GTK_OBJECT(data_widget), "flags");
 
      if (newtype & DT_BINARY_BASE64)  newtype = DT_BINARY_BASE64;
      if (newtype & DT_BINARY_HEX) newtype = DT_BINARY_HEX;
      if (newtype & DT_BINARY_PLAIN) newtype = DT_BINARY_PLAIN;
-     
+
      if (newtype != (*flags & DT_BINARY_ENCODING_MASK)) {
 	  GByteArray *data = dt_binary_get_data_internal(form, data_widget);
 	  *flags = (*flags & ~DT_BINARY_ENCODING_MASK) | newtype;
@@ -378,7 +354,7 @@ static void dt_binary_switch_to_encoding(struct formfill *form,
      }
 }
 
-void dt_binary_store_data(struct formfill *form, 
+void dt_binary_store_data(struct formfill *form,
 			  GtkWidget *hbox,
 			  GtkWidget *data_widget,
 			  const GByteArray *data)
@@ -405,7 +381,7 @@ static GByteArray *dt_binary_get_data_internal(struct formfill *form,
 
      content = gtk_editable_get_chars(GTK_EDITABLE(inputbox), 0, -1);
      if (!content) return NULL;
-     
+
      l = strlen(content);
      if (l == 0) {
 	  g_free(content);
@@ -421,11 +397,8 @@ static GByteArray *dt_binary_get_data_internal(struct formfill *form,
 	  copy = dt_b64_decode(content, strlen(content));
      }
 
-/*       copy = DT_BINARY(form->dt_handler)->decode(content,  */
-/*  						strlen(content)); */
-
      g_free(content);
-     
+
      return copy;
 }
 
@@ -444,7 +417,7 @@ static void dt_binary_delete_data_internal(struct formfill *form,
 
      if (*flags & DT_BINARY_TEXT) {
 	  GtkText *text = GTK_TEXT(widget);
-	  
+
 	  gtk_text_freeze(text);
 	  gtk_text_set_point(text, 0);
 	  gtk_text_forward_delete(text, gtk_text_get_length(text));
@@ -456,7 +429,7 @@ static void dt_binary_delete_data_internal(struct formfill *form,
 }
 
 void dt_binary_delete_data(struct formfill *form,
-			   GtkWidget *hbox, 
+			   GtkWidget *hbox,
 			   GtkWidget *data_widget)
 {
 /*       int *flags = gtk_object_get_data(GTK_OBJECT(data_widget), "flags"); */
@@ -538,7 +511,7 @@ GByteArray *dt_hex_decode(const char *val, int len)
 	  return(NULL);
      }
 
-     
+
      for (i = 0, c = val ; i < len ; i++, c++) {
 	  if (*c == ':') {
 	       precolon = 0;
@@ -566,15 +539,36 @@ GByteArray *dt_hex_decode(const char *val, int len)
      b = 0;
      g_byte_array_append(gb, &b, 1);
      /* trust GByteArray to keep the NUL byte.... */
-     
+
      g_byte_array_set_size(gb, gb->len - 1);
-     
 
      return gb;
 }
 
-/* 
-   Local Variables:
-   c-basic-offset: 5
-   End:
- */
+/* GType */
+G_DEFINE_TYPE(GQDisplayBinary, gq_display_binary, GQ_TYPE_DISPLAY_BINARY_GENERIC);
+
+static void
+gq_display_binary_init(GQDisplayBinary* self) {}
+
+static void
+gq_display_binary_class_init(GQDisplayBinaryClass* self_class) {
+	GQTypeDisplayClass* gtd_class = GQ_TYPE_DISPLAY_CLASS(self_class);
+	GQDisplayBinaryGenericClass* gdbg_class = GQ_DISPLAY_BINARY_GENERIC_CLASS(self_class);
+
+	gtd_class->name = Q_("displaytype|Binary");
+	gtd_class->selectable = TRUE;
+	gtd_class->show_in_search_result = FALSE;
+	gtd_class->get_widget = dt_binary_get_widget; // reused from dt_generic_binary
+	gtd_class->get_data   = dt_binary_get_data;
+	gtd_class->set_data   = dt_generic_binary_set_data; // reused from dt_generic_binary
+	gtd_class->buildLDAPMod = bervalLDAPMod;
+
+	gdbg_class->encode = dt_b64_encode;
+	gdbg_class->decode = dt_b64_decode;
+	gdbg_class->get_data_widget = dt_binary_get_data_widget;
+	gdbg_class->store_data = dt_binary_store_data;
+	gdbg_class->delete_data = dt_binary_delete_data;
+	gdbg_class->show_entries = dt_generic_binary_show_entries;
+}
+
