@@ -24,9 +24,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_cert.c,v 1.10 2006/04/07 20:33:41 herzi Exp $ */
+/* $Id: dt_cert.c 937 2006-05-25 13:51:47Z herzi $ */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 #ifdef HAVE_LIBCRYPTO
 
 #include <string.h>
@@ -38,6 +40,7 @@
 #include <ctype.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #define GTK_ENABLE_BROKEN  /* for the text widget - should be replaced - FIXME */
 #include <gtk/gtk.h>
@@ -54,9 +57,8 @@
 #include "util.h"
 #include "formfill.h"
 #include "dt_cert.h"
-#include "i18n.h"
 
-static void dt_cert_fill_clist(struct formfill *form, 
+static void dt_cert_fill_clist(struct formfill *form,
 			       GtkWidget *hbox,
 			       GtkWidget *data_widget,
 			       GByteArray *internal,
@@ -68,37 +70,7 @@ static void dt_cert_fill_details(struct formfill *form,
 				 GByteArray *internal,
 				 GtkWidget *clist);
 
-static dt_cert_handler dt_cert_handler_vtab = {
-     {
-	  {
-	       "Certificate", 
-	       TRUE,
-	       FALSE,
-
-	       dt_clist_get_widget,
-	       dt_clist_get_data,
-	       dt_generic_binary_set_data, /* reused from dt_generic_binary */
-	       bervalLDAPMod
-	  },
-	  NULL, /* encode */
-	  NULL, /* decode */
-	  
-	  dt_clist_get_data_widget,	/* get_data_widget */
-	  dt_clist_store_data,		/* store_data */
-	  dt_clist_delete_data,		/* store_data */
-	  
-	  dt_clist_show_entries,	/* show_entries */
-     },
-     dt_cert_fill_clist,		/* fill_clist */
-     dt_cert_fill_details,		/* fill_details */
-};
-
-display_type_handler *dt_cert_get_handler()
-{
-     return (display_type_handler *) &dt_cert_handler_vtab;
-}
-
-static void dt_cert_fill_clist(struct formfill *form, 
+static void dt_cert_fill_clist(struct formfill *form,
 			       GtkWidget *hbox,
 			       GtkWidget *data_widget,
 			       GByteArray *internal,
@@ -114,8 +86,8 @@ static void dt_cert_fill_clist(struct formfill *form,
 	       databio = BIO_new(BIO_s_mem());
 
 	       rc = BIO_write(databio, internal->data, internal->len);
-	       
-	       x = d2i_X509_bio(databio, NULL); 
+
+	       x = d2i_X509_bio(databio, NULL);
 	       if (!x) {
 		    /* try PEM */
 		    BIO_reset(databio);
@@ -271,17 +243,43 @@ static void dt_cert_fill_details(struct formfill *form,
      data = BIO_new(BIO_s_mem());
      X509_print(data, x);
      BIO_get_mem_ptr(data, &bm);
-     
-     gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, 
+
+     gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
 		     bm->data, bm->length);
 
      BIO_free(data);
 }
 
+/* GType */
+G_DEFINE_TYPE(GQDisplayCert, gq_display_cert, GQ_TYPE_DISPLAY_CLIST);
+
+static void
+gq_display_cert_init(GQDisplayCert* self) {}
+
+static void
+gq_display_cert_class_init(GQDisplayCertClass* self_class) {
+	GQTypeDisplayClass * gtd_class = GQ_TYPE_DISPLAY_CLASS(self_class);
+	GQDisplayBinaryGenericClass* gdbg_class = GQ_DISPLAY_BINARY_GENERIC_CLASS(self_class);
+	GQDisplayCListClass* gdc_class = GQ_DISPLAY_CLIST_CLASS(self_class);
+
+	gtd_class->name = Q_("displaytype|Certificate");
+	gtd_class->selectable = TRUE;
+	gtd_class->show_in_search_result = FALSE;
+	gtd_class->get_widget = dt_clist_get_widget;
+	gtd_class->get_data = dt_clist_get_data;
+	gtd_class->set_data = dt_generic_binary_set_data; // reused from dt_generic_binary
+	gtd_class->buildLDAPMod = bervalLDAPMod;
+
+	gdbg_class->encode = NULL;
+	gdbg_class->decode = NULL;
+	gdbg_class->get_data_widget = dt_clist_get_data_widget;
+	gdbg_class->store_data = dt_clist_store_data;
+	gdbg_class->delete_data = dt_clist_delete_data;
+	gdbg_class->show_entries = dt_clist_show_entries;
+
+	gdc_class->fill_clist = dt_cert_fill_clist;
+	gdc_class->fill_details = dt_cert_fill_details;
+}
+
 #endif /* HAVE_LIBCRYPTO */
 
-/* 
-   Local Variables:
-   c-basic-offset: 5
-   End:
- */
