@@ -24,23 +24,33 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_clist.h,v 1.3 2003/11/02 07:20:43 stamfest Exp $ */
+/* $Id: dt_clist.h 937 2006-05-25 13:51:47Z herzi $ */
 
 #ifndef DT_CLIST_H_INCLUDED
 #define DT_CLIST_H_INCLUDED
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 #ifdef HAVE_LIBCRYPTO
 
 #include "syntax.h"
 #include "dt_generic_binary.h"
 
+typedef GQDisplayBinaryGeneric      GQDisplayCList;
+typedef struct _GQDisplayCListClass GQDisplayCListClass;
 
 #define DT_CLIST_EMPTY_HEIGHT 25
 #define DT_CLIST_FULL_HEIGHT 100
 
-typedef struct _dt_clist_handler {
-     dt_generic_binary_handler dt_generic;
+#define GQ_TYPE_DISPLAY_CLIST         (gq_display_clist_get_type())
+#define GQ_DISPLAY_CLIST_CLASS(c)     (G_TYPE_CHECK_CLASS_CAST((c), GQ_TYPE_DISPLAY_CLIST, GQDisplayCListClass))
+#define DT_CLIST(objpointer)          GQ_DISPLAY_CLIST_CLASS(objpointer)
+
+GType gq_display_clist_get_type(void);
+
+struct _GQDisplayCListClass {
+     GQDisplayBinaryGenericClass dt_generic;
 
      void (*fill_clist)(struct formfill *form,
 			GtkWidget *hbox, GtkWidget *data_widget,
@@ -50,21 +60,17 @@ typedef struct _dt_clist_handler {
 			  GtkWidget *hbox, GtkWidget *data_widget,
 			  GByteArray *internal,
 			  GtkWidget *clist);
-} dt_clist_handler;
-
-#define DT_CLIST(objpointer) ((dt_clist_handler*)(objpointer))
-
-display_type_handler *dt_clist_get_handler();
+};
 
 /* Methods, only to be used by subclasses */
 GtkWidget *dt_clist_get_widget(int error_context,
-			       struct formfill *form, 
+			       struct formfill *form,
 			       GByteArray *data,
-			       GtkSignalFunc *activatefunc,
+			       GCallback *activatefunc,
 			       gpointer funcdata);
 
-GtkWidget *dt_clist_get_data_widget(struct formfill *form, 
-				    GtkSignalFunc *activatefunc,
+GtkWidget *dt_clist_get_data_widget(struct formfill *form,
+				    GCallback *activatefunc,
 				    gpointer funcdata);
 
 GByteArray *dt_clist_get_data(struct formfill *form,
