@@ -24,9 +24,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_crl.c,v 1.7 2006/04/07 20:33:41 herzi Exp $ */
+/* $Id: dt_crl.c 937 2006-05-25 13:51:47Z herzi $ */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 #ifdef HAVE_LIBCRYPTO
 
 #include <string.h>
@@ -38,6 +40,7 @@
 #include <ctype.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #define GTK_ENABLE_BROKEN  /* for the text widget - should be replaced - FIXME */
 #include <gtk/gtk.h>
@@ -53,9 +56,8 @@
 #include "util.h"
 #include "formfill.h"
 #include "dt_crl.h"
-#include "i18n.h"
 
-static void dt_crl_fill_clist(struct formfill *form, 
+static void dt_crl_fill_clist(struct formfill *form,
 			      GtkWidget *hbox,
 			      GtkWidget *data_widget,
 			      GByteArray *internal,
@@ -67,37 +69,7 @@ static void dt_crl_fill_details(struct formfill *form,
 				GByteArray *internal,
 				GtkWidget *clist);
 
-static dt_crl_handler dt_crl_handler_vtab = {
-     {
-	  {
-	       "Certificate Revocation List",
-	       TRUE,
-	       FALSE, 
-
-	       dt_clist_get_widget,
-	       dt_clist_get_data,
-	       dt_generic_binary_set_data, /* reused from dt_generic_binary */
-	       bervalLDAPMod
-	  },
-	  NULL, /* encode */
-	  NULL, /* decode */
-	  
-	  dt_clist_get_data_widget,	/* get_data_widget */
-	  dt_clist_store_data,		/* store_data */
-	  dt_clist_delete_data,		/* store_data */
-	  
-	  dt_clist_show_entries,	/* show_entries */
-     },
-     dt_crl_fill_clist,			/* fill_clist */
-     dt_crl_fill_details,		/* fill_details */
-};
-
-display_type_handler *dt_crl_get_handler()
-{
-     return (display_type_handler *) &dt_crl_handler_vtab;
-}
-
-static void dt_crl_fill_clist(struct formfill *form, 
+static void dt_crl_fill_clist(struct formfill *form,
 			       GtkWidget *hbox,
 			       GtkWidget *data_widget,
 			       GByteArray *internal,
@@ -114,8 +86,8 @@ static void dt_crl_fill_clist(struct formfill *form,
 	       databio = BIO_new(BIO_s_mem());
 
 	       rc = BIO_write(databio, internal->data, internal->len);
-	       
-	       x = d2i_X509_CRL_bio(databio, NULL); 
+
+	       x = d2i_X509_CRL_bio(databio, NULL);
 	       if (!x) {
 		    /* try PEM */
 		    BIO_reset(databio);
@@ -133,15 +105,15 @@ static void dt_crl_fill_clist(struct formfill *form,
 		    BIO *bufbio = BIO_new(BIO_s_mem());
 		    BUF_MEM *bm;
 		    GByteArray *gb;
-		    
+
 		    /* convert data into DER form! */
 		    i2d_X509_CRL_bio(bufbio, x);
 		    BIO_get_mem_ptr(bufbio, &bm);
-		    
+
 		    gb = g_byte_array_new();
 		    g_byte_array_append(gb, (guchar*)bm->data, bm->length);
-		    
-		    gtk_object_set_data_full(GTK_OBJECT(data_widget), "data", 
+
+		    gtk_object_set_data_full(GTK_OBJECT(data_widget), "data",
 					     gb,
 					     (GtkDestroyNotify) free_internal_data);
 
@@ -185,11 +157,11 @@ static void dt_crl_fill_clist(struct formfill *form,
 
 		    /* list of revoked certs */
 
-		    rev = X509_CRL_get_REVOKED(x); 
+		    rev = X509_CRL_get_REVOKED(x);
 		    /* NOTE: we do _not_ have to free rev, it is part
 		       of the CRL itself */
 		    cols[0] = _("Number of revoked certificates");
-		    
+
 		    BIO_reset(bufbio);
 		    BIO_printf(bufbio, "%ld", (glong)sk_X509_REVOKED_num(rev));
 		    BIO_get_mem_ptr(bufbio, &bm);
@@ -204,7 +176,7 @@ static void dt_crl_fill_clist(struct formfill *form,
 		    }
 		    */
 		    gtk_clist_append(GTK_CLIST(clist), cols);
-		    
+
 		    /* Version:
 		     * Version 0x00 actually means Version 1 */
 		    BIO_reset(bufbio);
@@ -223,7 +195,7 @@ static void dt_crl_fill_clist(struct formfill *form,
 	       } else {
 		    BIO *err;
 		    err = BIO_new(BIO_s_mem());
-  		    ERR_print_errors(err);
+		    ERR_print_errors(err);
 
 		    /* FIXME: report error */
 
@@ -232,7 +204,7 @@ static void dt_crl_fill_clist(struct formfill *form,
 
 	       BIO_free(databio);
 	  }
-  	  if (hbox) gtk_widget_set_usize(GTK_WIDGET(hbox), -1, 60);
+	  if (hbox) gtk_widget_set_usize(GTK_WIDGET(hbox), -1, 60);
      }
 }
 
@@ -251,17 +223,43 @@ static void dt_crl_fill_details(struct formfill *form,
      data = BIO_new(BIO_s_mem());
      X509_CRL_print(data, x);
      BIO_get_mem_ptr(data, &bm);
-     
-     gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, 
+
+     gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
 		     bm->data, bm->length);
 
      BIO_free(data);
 }
 
+/* GType */
+G_DEFINE_TYPE(GQDisplayCRL, gq_display_crl, GQ_TYPE_DISPLAY_CLIST);
+
+static void
+gq_display_crl_init(GQDisplayCRL* self) {}
+
+static void
+gq_display_crl_class_init(GQDisplayCRLClass* self_class) {
+	GQTypeDisplayClass* gtd_class = GQ_TYPE_DISPLAY_CLASS(self_class);
+	GQDisplayBinaryGenericClass* gdbg_class = GQ_DISPLAY_BINARY_GENERIC_CLASS(self_class);
+	GQDisplayCListClass* gdc_class = GQ_DISPLAY_CLIST_CLASS(self_class);
+
+	gtd_class->name = Q_("displaytype|Certificate Revocation List");
+	gtd_class->selectable = TRUE;
+	gtd_class->show_in_search_result = FALSE;
+	gtd_class->get_widget = dt_clist_get_widget;
+	gtd_class->get_data = dt_clist_get_data;
+	gtd_class->set_data = dt_generic_binary_set_data; // reused from dt_generic_binary
+	gtd_class->buildLDAPMod = bervalLDAPMod;
+
+	gdbg_class->encode = NULL;
+	gdbg_class->decode = NULL;
+	gdbg_class->get_data_widget = dt_clist_get_data_widget;
+	gdbg_class->store_data = dt_clist_store_data;
+	gdbg_class->delete_data = dt_clist_delete_data;
+	gdbg_class->show_entries = dt_clist_show_entries;
+
+	gdc_class->fill_clist = dt_crl_fill_clist;
+	gdc_class->fill_details = dt_crl_fill_details;
+}
+
 #endif /* HAVE_LIBCRYPTO */
 
-/* 
-   Local Variables:
-   c-basic-offset: 5
-   End:
- */
