@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_generic_binary.c,v 1.14 2003/11/03 21:07:57 stamfest Exp $ */
+/* $Id: dt_generic_binary.c 952 2006-08-26 11:17:35Z herzi $ */
 
 #include <string.h>
 #include <sys/types.h>
@@ -30,9 +30,12 @@
 #include <fcntl.h>
 #include <unistd.h>
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdk.h>
 
 #define GTK_ENABLE_BROKEN  /* for the text widget - should be replaced */
@@ -41,15 +44,15 @@
 #include "dt_generic_binary.h"
 
 #include "common.h"
-#include "util.h"
 #include "errorchain.h"
+#include "gq-tab-browse.h"
+#include "syntax.h"
+#include "util.h"
 #include "formfill.h"
 #include "input.h"
 #include "tinput.h"
-#include "browse.h"
 #include "encode.h"
 #include "ldif.h" /* for b64_decode */
-#include "i18n.h"
 
 static void dt_generic_binary_import_button_clicked(GtkButton* button,
 					    gpointer func_data);
@@ -61,38 +64,7 @@ static void dt_generic_binary_delete_button_clicked(GtkButton* button,
 GByteArray *dt_b64_encode(const char *val, int len);
 GByteArray *dt_b64_decode(const char *val, int len);
 
-
-static dt_generic_binary_handler dt_generic_binary_handler_vtab = {
-     {
-	  "Generic Binary", FALSE,
-	  FALSE,
-	  dt_generic_binary_get_widget,
-	  NULL,
-	  NULL,
-	  bervalLDAPMod
-     },
-     NULL, /* encode */
-     NULL, /* decode */
-
-     NULL, /* get_data_widget */
-     NULL, /* store_data */
-     NULL, /* delete_data */
-
-     dt_generic_binary_show_entries, /* show_entries */
-};
-
-display_type_handler *dt_generic_binary_get_handler()
-{
-     return (display_type_handler *) &dt_generic_binary_handler_vtab;
-}
-
-
-/*  static void destroy_byte_array(gpointer gb) { */
-/*       printf("DESTROYING\n"); */
-/*       g_byte_array_free((GByteArray*) gb, TRUE); */
-/*  } */
-
-void dt_generic_binary_show_entries(struct formfill *form, 
+void dt_generic_binary_show_entries(struct formfill *form,
 				    GtkWidget *hbox, gboolean what)
 {
      GtkWidget *b;
@@ -102,7 +74,7 @@ void dt_generic_binary_show_entries(struct formfill *form,
      if (b) {
 	  gtk_widget_set_sensitive(b, what);
      }
-     
+
      b = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(hbox), "delete"));
      if (b) {
 	  gtk_widget_set_sensitive(b, what);
@@ -123,7 +95,7 @@ static void dt_generic_binary_create_menu(GtkWidget *hbox, GtkWidget *vbox,
 
      menu = gtk_menu_bar_new();
      gtk_widget_show(menu);
-     gtk_box_pack_start(GTK_BOX(vbox), menu, FALSE, FALSE, 0); 
+     gtk_box_pack_start(GTK_BOX(vbox), menu, FALSE, FALSE, 0);
 
      menu_item = gtk_menu_item_new();
      gtk_widget_show(menu_item);
@@ -144,51 +116,52 @@ static void dt_generic_binary_create_menu(GtkWidget *hbox, GtkWidget *vbox,
 
      menu_item = gtk_menu_item_new_with_label(_("Import..."));
      gtk_widget_show(menu_item);
-    
-     gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-			GTK_SIGNAL_FUNC(dt_generic_binary_import_button_clicked),
+
+     g_signal_connect(menu_item, "activate",
+			G_CALLBACK(dt_generic_binary_import_button_clicked),
 			hbox);
-    
+
      gtk_menu_append(GTK_MENU(submenu), menu_item);
 
      /* ------------------ Export -------------------- */
-    
+
      menu_item = gtk_menu_item_new_with_label(_("Export..."));
      gtk_widget_show(menu_item);
      gtk_widget_set_sensitive(menu_item, FALSE);
-	  
-     gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-			GTK_SIGNAL_FUNC(dt_generic_binary_export_button_clicked),
+
+     g_signal_connect(menu_item, "activate",
+			G_CALLBACK(dt_generic_binary_export_button_clicked),
 			hbox);
-     
+
      gtk_menu_append(GTK_MENU(submenu), menu_item);
      gtk_object_set_data(GTK_OBJECT(hbox), "export", GTK_WIDGET(menu_item));
-    
+
      /* ------------------ Delete -------------------- */
      menu_item = gtk_menu_item_new_with_label(_("Delete"));
      gtk_widget_show(menu_item);
      gtk_widget_set_sensitive(menu_item, FALSE);
 
-     gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-			GTK_SIGNAL_FUNC(dt_generic_binary_delete_button_clicked),
+     g_signal_connect(menu_item, "activate",
+			G_CALLBACK(dt_generic_binary_delete_button_clicked),
 			hbox);
 
      gtk_menu_append(GTK_MENU(submenu), menu_item);
      gtk_object_set_data(GTK_OBJECT(hbox), "delete", GTK_WIDGET(menu_item));
 }
 
-GtkWidget *dt_generic_binary_get_widget(int error_context, 
-					struct formfill *form, 
+GtkWidget *dt_generic_binary_get_widget(int error_context,
+					struct formfill *form,
 					GByteArray *data,
-					GtkSignalFunc *activatefunc,
-					gpointer funcdata) 
+					GCallback *activatefunc,
+					gpointer funcdata)
 {
+     GQTypeDisplayClass* klass = g_type_class_ref(form->dt_handler);
      GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
      GtkWidget *vbox;
 
-     GtkWidget *data_widget = 
-	 DT_GENERIC_BINARY(form->dt_handler)->get_data_widget(form,
-							      activatefunc, 
+     GtkWidget *data_widget =
+	 DT_GENERIC_BINARY(klass)->get_data_widget(form,
+							      activatefunc,
 							      funcdata);
 
      gtk_box_pack_start(GTK_BOX(hbox), data_widget, TRUE, TRUE, 0);
@@ -204,21 +177,24 @@ GtkWidget *dt_generic_binary_get_widget(int error_context,
 
      dt_generic_binary_create_menu(hbox, vbox, data);
 
-     DT_GENERIC_BINARY(form->dt_handler)->store_data(form, hbox, 
+     DT_GENERIC_BINARY(klass)->store_data(form, hbox,
 						     data_widget, data);
 
-     if (data && data->len) 
-	  DT_GENERIC_BINARY(form->dt_handler)->show_entries(form, hbox, TRUE);
+     if (data && data->len)
+	  DT_GENERIC_BINARY(klass)->show_entries(form, hbox, TRUE);
 
+     g_type_class_unref(klass);
      return hbox;
 }
 
-void dt_generic_binary_set_data(struct formfill *form, GByteArray *data, 
+void dt_generic_binary_set_data(struct formfill *form, GByteArray *data,
 				GtkWidget *hbox)
 {
-     DT_GENERIC_BINARY(form->dt_handler)->store_data(form, hbox, 
+	GQTypeDisplayClass* klass = g_type_class_ref(form->dt_handler);
+	DT_GENERIC_BINARY(klass)->store_data(form, hbox,
 						     dt_generic_binary_retrieve_data_widget(hbox),
 						     data);
+	g_type_class_unref(klass);
 }
 
 /* The file selection widget and the string to store the chosen filename */
@@ -227,13 +203,14 @@ void dt_generic_binary_set_data(struct formfill *form, GByteArray *data,
 static GtkWidget *file_selector; /* FIXME - make non-global ASAP!!!!!!*/
 
 
-static void dt_generic_binary_import_got_filename(GtkFileSelection *selector, 
+static void dt_generic_binary_import_got_filename(GtkFileSelection *selector,
 					  gpointer user_data)
 {
+     GQTypeDisplayClass* klass;
      GtkWidget *hbox = user_data;
      GtkWidget *data_widget;
      struct formfill *form;
-     const char *selected_filename = 
+     const char *selected_filename =
 	  gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
 
      GByteArray *ndata = NULL;
@@ -246,12 +223,12 @@ static void dt_generic_binary_import_got_filename(GtkFileSelection *selector,
 	  int rlen = 0;
 
 	  ndata = g_byte_array_new();
-	  
+
 	  if (fstat(file, &buf) == 0) {
 	       if (g_byte_array_set_size(ndata, buf.st_size) != NULL) {
 		    int n;
 
-		    while ((n = read(file, ndata->data + rlen, 
+		    while ((n = read(file, ndata->data + rlen,
 				    buf.st_size - rlen)) > 0) {
 			 rlen += n;
 			 /* check for overflow (concurrent update) */
@@ -269,7 +246,7 @@ static void dt_generic_binary_import_got_filename(GtkFileSelection *selector,
 	       /* Error dialog - FIXME */
 	       g_byte_array_free(ndata, TRUE);
 	       ndata = NULL;
-	  }	       
+	  }
 
 	  close(file);
      } else {
@@ -280,19 +257,21 @@ static void dt_generic_binary_import_got_filename(GtkFileSelection *selector,
 						    "formfill");
      data_widget = dt_generic_binary_retrieve_data_widget(hbox);
 
+     klass = g_type_class_ref(form->dt_handler);
      if (ndata && ndata->len > 0) {
-	  DT_GENERIC_BINARY(form->dt_handler)->store_data(form, hbox, 
-							  data_widget, 
+	  DT_GENERIC_BINARY(klass)->store_data(form, hbox,
+							  data_widget,
 							  ndata);
 	  g_byte_array_free(ndata, TRUE);
 
-	  DT_GENERIC_BINARY(form->dt_handler)->show_entries(form, hbox, TRUE);
+	  DT_GENERIC_BINARY(klass)->show_entries(form, hbox, TRUE);
      } else {
-	  DT_GENERIC_BINARY(form->dt_handler)->delete_data(form, hbox,
+	  DT_GENERIC_BINARY(klass)->delete_data(form, hbox,
 							   data_widget);
-	  DT_GENERIC_BINARY(form->dt_handler)->show_entries(form, hbox, FALSE);
+	  DT_GENERIC_BINARY(klass)->show_entries(form, hbox, FALSE);
      }
 
+     g_type_class_unref(klass);
 }
 
 static void dt_generic_binary_import_button_clicked(GtkButton* button,
@@ -301,51 +280,54 @@ static void dt_generic_binary_import_button_clicked(GtkButton* button,
      GtkWidget *hbox = func_data;
 
 /*       struct formfill *form =  */
-/*  	  gtk_object_get_data(GTK_OBJECT(hbox), "formfill"); */
+/*	  gtk_object_get_data(GTK_OBJECT(hbox), "formfill"); */
 
      /* Create the selector */
-     
+
      file_selector =
 	  gtk_file_selection_new(_("Please select a file containing data for this attribute"));
 
-     gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));   
+     gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
 
-     gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
-			"clicked", GTK_SIGNAL_FUNC (dt_generic_binary_import_got_filename), 
+     g_signal_connect(GTK_FILE_SELECTION(file_selector)->ok_button,
+			"clicked", G_CALLBACK(dt_generic_binary_import_got_filename),
 			hbox);
-     
+
      /* Ensure that the dialog box is destroyed when the user clicks a button. */
-     
-     gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-			       "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) file_selector);
-     
-     gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
-			       "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) file_selector);
-     
+
+     g_signal_connect_swapped(GTK_FILE_SELECTION(file_selector)->ok_button,
+			       "clicked", G_CALLBACK(gtk_widget_destroy),
+			       file_selector);
+
+     g_signal_connect_swapped(GTK_FILE_SELECTION(file_selector)->cancel_button,
+			       "clicked", G_CALLBACK(gtk_widget_destroy),
+			       file_selector);
+
      /* Display that dialog */
-     
+
      gtk_widget_show(file_selector);
      gtk_window_set_modal(GTK_WINDOW(file_selector), TRUE);
 }
 
 
-static void dt_generic_binary_export_got_filename(GtkFileSelection *selector, 
+static void dt_generic_binary_export_got_filename(GtkFileSelection *selector,
 						  gpointer user_data)
 {
+     GQTypeDisplayClass* klass;
      GtkWidget *hbox = user_data;
-     const char *selected_filename = 
+     const char *selected_filename =
 	  gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
 
      /* write file from pixbuf data */
      GByteArray *data;
      struct formfill *form;
      int file;
-     
+
      form = (struct formfill *) gtk_object_get_data(GTK_OBJECT(hbox),
 						    "formfill");
-     data = DISPLAY_TYPE_HANDLER(form->dt_handler)->get_data(form, hbox);
+     klass = g_type_class_ref(form->dt_handler);
+     data = DISPLAY_TYPE_HANDLER(klass)->get_data(form, hbox);
+     g_type_class_unref(klass);
 
      if (data) {
 	  file = open(selected_filename, O_WRONLY | O_TRUNC | O_CREAT, 0666);
@@ -363,7 +345,7 @@ static void dt_generic_binary_export_got_filename(GtkFileSelection *selector,
 	       }
 	       close(file);
 	  }
-	  
+
 	  g_byte_array_free(data, TRUE);
      }
 }
@@ -374,27 +356,27 @@ static void dt_generic_binary_export_button_clicked(GtkButton* button,
      GtkWidget *hbox = func_data;
 
      /* Create the selector */
-     
+
      file_selector =
 	  gtk_file_selection_new(_("Please select a file to export data to for this attribute"));
      gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
-    
-     gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
-			"clicked", GTK_SIGNAL_FUNC (dt_generic_binary_export_got_filename), 
+
+     g_signal_connect(GTK_FILE_SELECTION(file_selector)->ok_button,
+			"clicked", G_CALLBACK(dt_generic_binary_export_got_filename),
 			hbox);
-     
+
      /* Ensure that the dialog box is destroyed when the user clicks a button. */
-     
-     gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-			       "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) file_selector);
-     
-     gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
-			       "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) file_selector);
-     
+
+     g_signal_connect_swapped(GTK_FILE_SELECTION(file_selector)->ok_button,
+			       "clicked", G_CALLBACK(gtk_widget_destroy),
+			       file_selector);
+
+     g_signal_connect_swapped(GTK_FILE_SELECTION(file_selector)->cancel_button,
+			       "clicked", G_CALLBACK(gtk_widget_destroy),
+			       file_selector);
+
      /* Display that dialog */
-     
+
      gtk_widget_show(file_selector);
      gtk_window_set_modal(GTK_WINDOW(file_selector), TRUE);
 
@@ -403,14 +385,16 @@ static void dt_generic_binary_export_button_clicked(GtkButton* button,
 static void dt_generic_binary_delete_button_clicked(GtkButton* button,
 						    GtkWidget *hbox)
 {
+     GQTypeDisplayClass* klass;
      GtkWidget *data_widget = dt_generic_binary_retrieve_data_widget(hbox);
-     
+
      struct formfill *form;
      form = (struct formfill *) gtk_object_get_data(GTK_OBJECT(hbox),
 						    "formfill");
-
-     DT_GENERIC_BINARY(form->dt_handler)->delete_data(form, hbox, data_widget);
-     DT_GENERIC_BINARY(form->dt_handler)->show_entries(form, hbox, FALSE);
+     klass = g_type_class_ref(form->dt_handler);
+     DT_GENERIC_BINARY(klass)->delete_data(form, hbox, data_widget);
+     DT_GENERIC_BINARY(klass)->show_entries(form, hbox, FALSE);
+     g_type_class_unref(klass);
 }
 
 /* Utility functions */
@@ -435,8 +419,29 @@ struct formfill *dt_generic_binary_retrieve_formfill(GtkWidget *hbox)
      return gtk_object_get_data(GTK_OBJECT(hbox), "form");
 }
 
-/* 
-   Local Variables:
-   c-basic-offset: 5
-   End:
- */
+/* GType */
+G_DEFINE_ABSTRACT_TYPE(GQDisplayBinaryGeneric, gq_display_binary_generic, GQ_TYPE_TYPE_DISPLAY);
+
+static void
+gq_display_binary_generic_init(GQDisplayBinaryGeneric* self) {}
+
+static void
+gq_display_binary_generic_class_init(GQDisplayBinaryGenericClass* self_class) {
+	GQTypeDisplayClass* gtd_class = GQ_TYPE_DISPLAY_CLASS(self_class);
+
+	gtd_class->name = Q_("displaytype|Generic Binary");
+	gtd_class->selectable = FALSE;
+	gtd_class->show_in_search_result = TRUE;
+	gtd_class->get_widget = dt_generic_binary_get_widget;
+	gtd_class->get_data = NULL;
+	gtd_class->set_data = NULL;
+	gtd_class->buildLDAPMod = bervalLDAPMod;
+
+	self_class->encode = NULL;
+	self_class->decode = NULL;
+	self_class->get_data_widget = NULL;
+	self_class->store_data = NULL;
+	self_class->delete_data = NULL;
+	self_class->show_entries = dt_generic_binary_show_entries;
+}
+
