@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: encode.h,v 1.11 2003/10/19 11:48:02 stamfest Exp $ */
+/* $Id: encode.h 945 2006-08-24 13:45:22Z herzi $ */
 
 #ifndef GQ_ENCODE_H_INCLUDED
 #define GQ_ENCODE_H_INCLUDED
@@ -30,12 +30,14 @@
 #include <glib.h>
 #include <stdlib.h>		/* size_t */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include  <config.h>
+#endif /* HAVE_CONFIG_H */
 #include "formfill.h"
 
 #if ENABLE_NLS
 #  if HAVE_LANGINFO_CODESET
-#    include<langinfo.h>
+#    include <langinfo.h>
 #  endif /* HAVE_LANGINFO_CODESET */
 #endif /* ENABLE_NLS */
 
@@ -50,15 +52,7 @@
 
 extern const char *gq_codeset;
 
-#ifdef HAVE_LIBCRYPTO
-
-void encode_password_crypt(struct formfill *form);
-void encode_password_md5(struct formfill *form);
-void encode_password_sha1(struct formfill *form);
-
-#endif /* HAVE_LIBCRYPTO */
-
-const gchar *decode_string(gchar *native_string, const gchar *ldap_string, 
+const gchar *decode_string(gchar *native_string, const gchar *ldap_string,
 			   size_t len);
 const gchar *encode_string(gchar *ldap_string, const gchar *native_string,
 			   size_t len);
@@ -67,7 +61,7 @@ gchar *encoded_string(const gchar *string);
 
 #endif
 
-/* 
+/*
    Local Variables:
    c-basic-offset: 5
    End:
