@@ -1,6 +1,7 @@
 /*
     GQ -- a GTK-based LDAP client
-    Copyright (C) 1998-2001 Bert Vermeulen
+    Copyright (C) 1998-2003 Bert Vermeulen
+    Copyright (C) 2002-2003 Peter Stamfest
 
     This program is released under the Gnu General Public License with
     the additional exemption that compiling, linking, and/or using
@@ -21,36 +22,39 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: formfill.h,v 1.16 2002/06/11 21:15:25 stamfest Exp $ */
+/* $Id: formfill.h 958 2006-09-02 20:27:07Z herzi $ */
 
 #ifndef GQ_FORMFILL_H_INLCUDED
 #define GQ_FORMFILL_H_INLCUDED
 
 #include <ldap_schema.h>
 
-#include "config.h"		/* pull in HAVE_* defines */
-#include "common.h"		
+#ifdef HAVE_CONFIG_H
+# include  <config.h>		/* pull in HAVE_* defines */
+#endif /* HAVE_CONFIG_H */
+#include "common.h"
 
-#define DISPLAYTYPE_DN          1
-#define DISPLAYTYPE_ENTRY	2
-#define DISPLAYTYPE_TEXT	3
-#define DISPLAYTYPE_PASSWORD	4
-#define DISPLAYTYPE_BINARY	5
-
-#ifdef HAVE_PIXBUF
-#  define DISPLAYTYPE_JPEG	6
-#else  /* HAVE_PIXBUF */
-#  define DISPLAYTYPE_JPEG	DISPLAYTYPE_BINARY
-#endif  /* HAVE_PIXBUF */
-
-#define DISPLAYTYPE_OC		7
+typedef enum {
+	// FIXME: add DISPLAYTYPE_INVALID = 0,
+	DISPLAYTYPE_DN		= 1,
+	DISPLAYTYPE_ENTRY	= 2,
+	DISPLAYTYPE_TEXT	= 3,
+	DISPLAYTYPE_PASSWORD	= 4,
+	DISPLAYTYPE_BINARY	= 5,
+	DISPLAYTYPE_JPEG	= 6,
+	DISPLAYTYPE_OC		= 7,
 #ifdef HAVE_LIBCRYPTO
-#  define DISPLAYTYPE_CERT	8
-#  define DISPLAYTYPE_CRL	9
-#else 
-#  define DISPLAYTYPE_CERT	DISPLAYTYPE_BINARY
-#  define DISPLAYTYPE_CRL	DISPLAYTYPE_BINARY
+	DISPLAYTYPE_CERT	= 8,
+	DISPLAYTYPE_CRL		= 9,
+#else
+	DISPLAYTYPE_CERT	= DISPLAYTYPE_BINARY,
+	DISPLAYTYPE_CRL		= DISPLAYTYPE_BINARY,
 #endif
+	DISPLAYTYPE_TIME	= 10,
+	DISPLAYTYPE_INT		= 11,
+	DISPLAYTYPE_NUMSTR	= 12,
+	DISPLAYTYPE_DATE	= 13
+} GQDisplayType;
 
 #define FLAG_NOT_IN_SCHEMA      0x01
 #define FLAG_MUST_IN_SCHEMA	0x02
@@ -59,52 +63,55 @@
 #define FLAG_DEL_ME		0x04
 /* Used to suppress the "more" button for single valued attributes */
 #define FLAG_SINGLE_VALUE	0x08
+/* Used to temporarily mark attributes added for extensibleObject entries */
+#define FLAG_EXTENSIBLE_OBJECT_ATTR	0x10
 
-/* actually, we do not really need the following FLAG_* values, we
-   only use them as tokens in dt_password.c right now */
-#define FLAG_ENCODE_CRYPT	0x10
-#define FLAG_ENCODE_MD5		0x20
-#define FLAG_ENCODE_SHA		0x40
-#define FLAG_ENCODE_SMD5	0x30
-#define FLAG_ENCODE_SSHA	0x50
-
-#define ENCODING_MASK           ( FLAG_ENCODE_CRYPT | FLAG_ENCODE_MD5 | FLAG_ENCODE_SHA )
+/* Used to disable widgets for attributes marked as no_user_mod */
+#define FLAG_NO_USER_MOD	0x80
 
 /* forward decls to avoid circular inclusion problems */
 struct _display_type_handler;
 struct syntax_handler;
 
 struct formfill {
-     char attrname[MAX_ATTR_LEN];
+     gchar* attrname;
+     GqServer *server;
      int num_inputfields;
-     int displaytype;
-     struct _display_type_handler *dt_handler;
+     GQDisplayType displaytype;
+     GType dt_handler; // GQTypeDisplayClass derivate
      int flags;
      GList *values;
      struct syntax_handler *syntax;
-     
+
+     GtkWidget *event_box;
      GtkWidget *label;
      GtkWidget *vbox;
      GtkWidget *morebutton;
      GList *widgetList;
 };
 
+void init_internalAttrs();
+gboolean isInternalAttr(const char *attr);
 
 struct formfill *new_formfill(void);
 void free_formlist(GList *formlist);
 void free_formfill(struct formfill *form);
 void free_formfill_values(struct formfill *form);
 GList *formlist_append(GList *formlist, struct formfill *form);
-GList *formlist_from_entry(struct ldapserver *server, char *dn, int ocvalues_only);
+GList *formlist_from_entry(int error_context,
+			   GqServer *server, 
+			   const char *dn, int ocvalues_only);
 GList *dup_formlist(GList *formlist);
-void dump_hash(GHashTable *hash);
 void dump_formlist(GList *formlist);
 struct formfill *lookup_attribute(GList *formlist, char *attr);
-struct formfill *lookup_attribute_using_schema(GList *formlist, char *attr,
+struct formfill *lookup_attribute_using_schema(GList *formlist, 
+					       const char *attr,
 					       struct server_schema *schema,
 					       LDAPAttributeType **attrtype);
-int find_displaytype(struct ldapserver *server, struct formfill *form);
-void set_displaytype(struct ldapserver *server, struct formfill *form);
+int find_displaytype(int error_context, GqServer *server, 
+		     struct formfill *form);
+void set_displaytype(int error_context, GqServer *server, 
+		     struct formfill *form);
 
 char *attr_strip(const char *attr);
 
