@@ -22,12 +22,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include  <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #ifdef DEBUG
 #  include "debug.h"
@@ -36,9 +39,7 @@
 #  endif /* HAVE_MCHECK */
 #endif /* DEBUG */
 
-
-#include "gq.h"
-#include "i18n.h"
+#include "gq-server-list.h"
 #include "mainwin.h"
 #include "configfile.h"
 #include "syntax.h"
@@ -46,38 +47,23 @@
 #include "debug.h"
 #include "state.h"
 
-GdkFont *big_fixed;
 static void sigpipehandler(int sig);
 
-#ifndef HAVE_SETENV
-// newer SunOS versions don't have setenv
-// FIXME: herzi: glib-2.4 contains g_setenv()
-static void
-setenv(gchar const* key, gchar const* val, gboolean overwrite) {
-	gchar* kvp = NULL;
-	g_return_if_fail(overwrite); // we only implement this mode
-	kvp = g_strdup_printf("%s=%s", key, val);
-	putenv(kvp);
-	g_free(kvp);
-}
-#endif /* !HAVE_SETENV */
+static const GOptionEntry entries[] = {
+#ifdef DEBUG
+	{ "debug", 'd', 0, G_OPTION_ARG_INT, &debug, N_("Enable debugging"), N_("LEVEL") },
+#endif
+	{ NULL, 0, 0, 0, NULL, NULL, NULL }
+};
 
-int main(int argc, char *argv[])
-{
-     int c;
+int
+main(int argc, char **argv) {
+	GOptionContext* ctxt = NULL;
      void *p;
 
 /*      return main_test(argc, argv); */
 
 
-#ifdef DEBUG
-#  ifdef HAVE_MCHECK
-     if (debug & GQ_DEBUG_MEM) {
-	  mcheck(NULL);
-     }
-#  endif /* HAVE_MCHECK */
-#endif /* DEBUG */
-
 #if ENABLE_NLS
      setlocale(LC_ALL, "");
      textdomain(PACKAGE);
@@ -89,7 +75,7 @@ int main(int argc, char *argv[])
 	       /* PS: rule out ANSI* explicitly - I hope this is
 		  generically enough */
 	       if (strncmp(codeset, "ANSI", 4) != 0) {
-		    /* nl_langinfo uses a static buffer - thus duplicate the 
+		    /* nl_langinfo uses a static buffer - thus duplicate the
 		       codeset to use before it gets overwritten */
 		    gq_codeset = g_strdup(codeset);
 	       }
@@ -98,38 +84,25 @@ int main(int argc, char *argv[])
 #  endif /* HAVE_LANGINFO_CODESET */
 #endif
 
-     bindtextdomain(PACKAGE, LOCALEDIR);
-
 #if ENABLE_NLS
-#  if GTK_MAJOR >= 2
-     bind_textdomain_codeset(PACKAGE, "UTF-8");
-#  else 
-     bind_textdomain_codeset(PACKAGE, gq_codeset);
-#  endif 
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset(PACKAGE, "UTF-8");
 #endif
 
-     gtk_set_locale();
-     gtk_init(&argc, &argv);
+	ctxt = g_option_context_new(_("- an LDAP client"));
+	g_option_context_add_main_entries(ctxt, entries, PACKAGE);
+	g_option_context_add_group(ctxt, gtk_get_option_group(TRUE));
+	g_option_context_parse(ctxt, &argc, &argv, NULL);
+	g_option_context_free(ctxt);
+	ctxt = NULL;
 
-     /* parse command line... */
-     while ((c = getopt(argc, argv, "d:")) != EOF) {
-	  switch (c) {
-	  case 'd': {
 #ifdef DEBUG
-	       char *ep;
-	       debug = strtoul(optarg, &ep, 0);
-#else
-	       fprintf(stderr, _("gq was build without debugging support\n"));
-	       return 1;
-#endif
-	       break;
-	  }
-	  default: {
-	       fprintf(stderr, _("Unknown option %c\n"), c);
-	       return 2;
-	  }
-	  }
-     }
+# ifdef HAVE_MCHECK
+	if (debug & GQ_DEBUG_MEM) {
+		mcheck(NULL);
+	}
+# endif /* HAVE_MCHECK */
+#endif /* DEBUG */
 
      init_syntaxes();
      init_internalAttrs();
@@ -138,10 +111,9 @@ int main(int argc, char *argv[])
      init_state();
 
      if (config->do_not_use_ldap_conf) {
-	  setenv("LDAPNOINIT", "1", 1);
+	  g_setenv("LDAPNOINIT", "1", TRUE);
      }
 
-     setup_fonts();
      create_mainwin(&mainwin);
 
 #ifdef DEBUG
@@ -162,7 +134,7 @@ int main(int argc, char *argv[])
 #ifdef DEBUG
      if (debug & GQ_DEBUG_SIGNALS) {
 	  fprintf(stderr, "SIGPIPE handler installation: %s\n",
-		  p == SIG_ERR ? "failed" : "ok"); 
+		  p == SIG_ERR ? "failed" : "ok");
      }
 #endif /* DEBUG */
 
@@ -184,33 +156,26 @@ int main(int argc, char *argv[])
      return(0);
 }
 
-static void sigpipehandler(int sig) 
+static void
+increase_down_per_server(GQServerList* list, GqServer* server, gpointer user_data) {
+	server->server_down++;
+}
+
+static void sigpipehandler(int sig)
 {
-     struct ldapserver *server;
-     GList *I;
-     
 #ifdef DEBUG
      if (debug & GQ_DEBUG_SIGNALS) {
-	  fprintf(stderr, "Catched SIGPIPE\n"); 
+	  fprintf(stderr, "Caught SIGPIPE\n");
      }
 #endif /* DEBUG */
-     
-     for (I = config->servers ; I ; I = g_list_next(I)) {
-	  server = (struct ldapserver *) I->data;
-	  server->server_down++;
-     }
-     /* reinstall signal handler [make sure we work on BSD] */ 
+
+     gq_server_list_foreach(gq_server_list_get(), increase_down_per_server, NULL);
+
+     /* reinstall signal handler [make sure we work on BSD] */
      signal(SIGPIPE, sigpipehandler);
 }
 
-void setup_fonts(void)
-{
-
-     big_fixed = gdk_font_load(FONT_BIG_FIXED);
-
-}
-
-/* 
+/*
    Local Variables:
    c-basic-offset: 5
    End:
