@@ -22,24 +22,26 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: ldapops.c,v 1.23 2003/11/07 12:02:12 biot Exp $ */
+/* $Id: ldapops.c 955 2006-08-31 19:15:21Z herzi $ */
 
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <ldap.h>
-#include "config.h"
+#include <glib/gi18n.h>
+#ifdef HAVE_CONFIG_H
+# include  <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #include "ldapops.h"
 #include "util.h"
 #include "errorchain.h"
-#include "i18n.h"
 
 static char * move_entry_internal(LDAPMessage *e,
 				  char *source_dn,
-				  struct ldapserver *source_server, LDAP *sld,
+				  GqServer *source_server, LDAP *sld,
 				  char *target_dn,
-				  struct ldapserver *target_server, LDAP *tld,
+				  GqServer *target_server, LDAP *tld,
 				  int flags,
 				  int error_context,
 				  MoveProgressFunc progress);
@@ -48,8 +50,8 @@ static char * move_entry_internal(LDAPMessage *e,
 /* Copy an entry to a new position below target_entry. This works
    cross-server if the right options get used */
 
-char * move_entry(char *source_dn, struct ldapserver *source_server,
-		  char *target_dn, struct ldapserver *target_server,
+char * move_entry(char *source_dn, GqServer *source_server,
+		  char *target_dn, GqServer *target_server,
 		  int flags,
 		  MoveProgressFunc progress,
 		  int err_ctx)
@@ -142,9 +144,9 @@ char * move_entry(char *source_dn, struct ldapserver *source_server,
 
 static char * move_entry_internal(LDAPMessage *e,
 				  char *source_dn,
-				  struct ldapserver *source_server, LDAP *sld,
+				  GqServer *source_server, LDAP *sld,
 				  char *target_dn,
-				  struct ldapserver *target_server, LDAP *tld,
+				  GqServer *target_server, LDAP *tld,
 				  int flags,
 				  int error_context,
 				  MoveProgressFunc progress)
@@ -178,7 +180,7 @@ static char * move_entry_internal(LDAPMessage *e,
      newlen = strlen(target_dn) + strlen(sdn[0]) + 10;
      newdn = g_malloc(newlen);
     
-     snprintf(newdn, newlen, "%s,%s", sdn[0], target_dn);
+     g_snprintf(newdn, newlen, "%s,%s", sdn[0], target_dn);
 
 #if defined(HAVE_LDAP_RENAME)
      /* first try to do a rename operation */
