@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: ldapops.h,v 1.3 2003/11/02 07:20:44 stamfest Exp $ */
+/* $Id: ldapops.h 955 2006-08-31 19:15:21Z herzi $ */
 
 #ifndef GQ_LDAPOPS_H_INCLUDED
 #define GQ_LDAPOPS_H_INCLUDED
@@ -38,8 +38,8 @@
 typedef void (*MoveProgressFunc)(const char *from_dn, const char *to_dn,
 				 const char *new_dn);
 
-char * move_entry(char *source_dn, struct ldapserver *source_server,
-		  char *target_dn, struct ldapserver *target_server,
+char * move_entry(char *source_dn, GqServer *source_server,
+		  char *target_dn, GqServer *target_server,
 		  int flags,
 		  MoveProgressFunc progress,
 		  int err_ctx);
