@@ -22,90 +22,91 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: mainwin.c,v 1.58 2006/04/11 11:34:42 herzi Exp $ */
+/* $Id: mainwin.c 975 2006-09-07 18:44:41Z herzi $ */
 
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #include "common.h"
-#include "search.h"
+#include "gq-server-list.h"
+#include "gq-tab-browse.h"
+#ifdef HAVE_LDAP_STR2OBJECTCLASS
+#    include "gq-tab-schema.h"
+#endif
+#include "gq-tab-search.h"
+
 #include "mainwin.h"
 #include "configfile.h"
-#include "browse.h"
 #include "prefs.h"
 #include "util.h"
 #include "template.h"
 #include "filter.h"
 #include "COPYING.h"
-#include "i18n.h"
 #include "debug.h"
 #include "input.h"
 #include "state.h"
 #include "errorchain.h"
 #include "progress.h"
 
-#include "../icons/logo.xpm"
-
-#ifdef HAVE_LDAP_STR2OBJECTCLASS
-#    include "schemabrowse.h"
-#endif
-
 struct mainwin_data mainwin;
 
-static void create_license_window(void);
-static GtkWidget *create_about_window(void);
+static void create_about_window(GtkWindow* parent);
 
 static void close_current_tab(struct mainwin_data *win);
 static void mainwin_destroyed(GtkWidget *widget, struct mainwin_data *win);
 static void switchpage_refocus(GtkNotebook *notebook, GtkNotebookPage *page,
 			       int pagenum, struct mainwin_data *win);
 
-struct tab *get_last_of_mode(int mode)
+GqTab *get_last_of_mode(int mode)
 {
      if(!mainwin.lastofmode)
 	  return NULL;
-     
+
      return g_hash_table_lookup(mainwin.lastofmode, GINT_TO_POINTER(mode));
 }
 
 
-void go_to_page(struct tab *tab)
+void go_to_page(GqTab *tab)
 {
-     gtk_notebook_set_page(GTK_NOTEBOOK(mainwin.mainbook), 
+     gtk_notebook_set_page(GTK_NOTEBOOK(mainwin.mainbook),
 			   gtk_notebook_page_num(GTK_NOTEBOOK(mainwin.mainbook),
 						 tab->content));
 }
 
 
-void enter_last_of_mode(struct tab *tab)
+void enter_last_of_mode(GqTab *tab)
 {
 
      if(!mainwin.lastofmode)
 	  mainwin.lastofmode = g_hash_table_new(g_direct_hash, g_direct_equal);
-     
-     g_hash_table_insert(mainwin.lastofmode, (gpointer) tab->type, tab);
+
+     g_hash_table_insert(mainwin.lastofmode, GINT_TO_POINTER(tab->type), tab);
 
 }
 
+static void
+append_name(GQServerList* list, GqServer* server, gpointer data) {
+	GList** serverlist = data;
+	*serverlist = g_list_append(*serverlist, server->name);
+}
 
 void fill_serverlist_combo(GtkWidget *combo)
 {
-     GList *serverlist, *I;
-     struct ldapserver *server;
+     GList *serverlist = NULL;
+     GqServer *server;
 
      if(combo == NULL)
 	  return;
 
-     serverlist = NULL;
-     for (I = config->servers ; I ; I = I->next) {
-	  server = (struct ldapserver *) I->data;
-	  serverlist = g_list_append(serverlist, server->name);
-     }
+     gq_server_list_foreach(gq_server_list_get(), append_name, &serverlist);
 
      if(!serverlist)
 	  /* all servers were deleted -- pass an empty string to the combo */
@@ -114,7 +115,6 @@ void fill_serverlist_combo(GtkWidget *combo)
      gtk_combo_set_popdown_strings(GTK_COMBO(combo), serverlist);
 
      g_list_free(serverlist);
-
 }
 
 static gboolean mainwin_restore_snapshot(struct mainwin_data *win)
@@ -130,15 +130,15 @@ static gboolean mainwin_restore_snapshot(struct mainwin_data *win)
      update_progress(pw, _("Restoring tabs"));
 
      for (i = 0 ; ; i++) {
-	  snprintf(tmp, sizeof(tmp), "mainwin.tabs.%d", i);
+	  g_snprintf(tmp, sizeof(tmp), "mainwin.tabs.%d", i);
 	  if (!exists_entity(tmp)) break;
 	  type = state_value_get_int(tmp, "type", -1);
 	  if (type > 0) {
-	       struct tab *tab = new_modetab(win, type);
+	       GqTab *tab = new_modetab(win, type);
 	       int error_ctx = error_new_context("", pw->win);
 
-	       if (tab->vtab && tab->vtab->restore_snapshot) {
-		    tab->vtab->restore_snapshot(error_ctx, tmp, tab, pw);
+	       if (GQ_TAB_GET_CLASS(tab)->restore_snapshot) {
+		    GQ_TAB_GET_CLASS(tab)->restore_snapshot(error_ctx, tmp, tab, pw);
 	       }
 
 	       error_flush(error_ctx);
@@ -150,38 +150,36 @@ static gboolean mainwin_restore_snapshot(struct mainwin_data *win)
      if (i > 0) {
 	  type = state_value_get_int("mainwin.tabs", "active", -1);
 	  gtk_notebook_set_page(GTK_NOTEBOOK(win->mainbook), type);
-     } 
+     }
 
      update_progress(pw, _("Restoring tabs"));
      free_progress(pw);
 
-     
-
      return i > 0;
 }
 
 static void mainwin_save_snapshot(struct mainwin_data *win)
 {
-     struct tab *tab = NULL; 
+     GqTab *tab = NULL;
      int i;
      char tmp[32];
 
      rm_value("mainwin.tabs");
 
      if (!config->restore_tabs) return;
-     
+
      for( i = 0 ; (tab = mainwin_get_tab_nth(win, i)) != NULL ; i++) {
-	  snprintf(tmp, sizeof(tmp), "mainwin.tabs.%d", i);
+	  g_snprintf(tmp, sizeof(tmp), "mainwin.tabs.%d", i);
 	  state_value_set_int(tmp, "type", tab->type);
-	  if (tab->vtab && tab->vtab->save_snapshot) {
-	       int error_ctx = 
+	  if (GQ_TAB_GET_CLASS(tab)->save_snapshot) {
+	       int error_ctx =
 		    error_new_context(_("Saving main window snapshot"), NULL);
-	       tab->vtab->save_snapshot(error_ctx, tmp, tab);
+	       GQ_TAB_GET_CLASS(tab)->save_snapshot(error_ctx, tmp, tab);
 	       error_flush(error_ctx);
 	  }
      }
 
-     state_value_set_int("mainwin.tabs", "active", 
+     state_value_set_int("mainwin.tabs", "active",
 			 gtk_notebook_get_current_page(GTK_NOTEBOOK(win->mainbook)));
 }
 
@@ -204,16 +202,12 @@ static gboolean ctrl_b_hack(GtkWidget *widget, GdkEventKey *event, gpointer obj)
 {
      if(event && event->type == GDK_KEY_PRESS &&
 	event->state & GDK_CONTROL_MASK && event->keyval == GDK_b) {
-	  gtk_signal_emit_by_name(GTK_OBJECT(obj), "activate");
-	  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+	  g_signal_emit_by_name(obj, "activate");
+	  g_signal_stop_emission_by_name(widget, "key_press_event");
 	  return(TRUE);
      }
 
-#if GTK_MAJOR >= 2
      return(FALSE);
-#else
-     return(TRUE);  /* may be a bug to return TRUE unconditionally in gtk 1.2 */
-#endif
 }
 
 
@@ -221,16 +215,12 @@ static gboolean ctrl_w_hack(GtkWidget *widget, GdkEventKey *event, gpointer obj)
 {
      if(event && event->type == GDK_KEY_PRESS &&
 	event->state & GDK_CONTROL_MASK && event->keyval == GDK_w) {
-	  gtk_signal_emit_by_name(GTK_OBJECT(obj), "activate");
-	  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+	  g_signal_emit_by_name(obj, "activate");
+	  g_signal_stop_emission_by_name(widget, "key_press_event");
 	  return(TRUE);
      }
 
-#if GTK_MAJOR >= 2
      return(FALSE);
-#else
-     return(TRUE);  /* may be a bug to return TRUE unconditionally in gtk 1.2 */
-#endif
 }
 
 static void new_modetab_search(struct mainwin_data *win) 
@@ -261,7 +251,6 @@ void clear_message_history()
      }
 
      if (mainwin.ml_text) {
-#if GTK_MAJOR >= 2
 	  GtkTextIter start;
 	  GtkTextIter end;
 	  
@@ -269,9 +258,6 @@ void clear_message_history()
 	  gtk_text_buffer_get_end_iter(mainwin.ml_buffer, &end);
 
 	  gtk_text_buffer_delete(mainwin.ml_buffer, &start, &end);
-#else
-	  gtk_editable_delete_text(GTK_EDITABLE(mainwin.ml_text), 0, -1);
-#endif
      }
 }
 
@@ -282,7 +268,6 @@ void message_log_append(const char *buf)
      log_list_len++;
 
      if (mainwin.ml_text) {
-#if GTK_MAJOR >= 2
 	  GtkTextIter iter;
 	  gtk_text_buffer_get_end_iter(mainwin.ml_buffer, &iter);
 	  gtk_text_buffer_insert(mainwin.ml_buffer, &iter,
@@ -297,13 +282,6 @@ void message_log_append(const char *buf)
 				       0.0,
 				       FALSE,
 				       0.0, 0.0);
-#else
-	  int pos = gtk_text_get_length(GTK_TEXT(mainwin.ml_text));
-	  gtk_editable_insert_text(GTK_EDITABLE(mainwin.ml_text), buf, 
-				   strlen(buf), &pos);
-	  gtk_editable_insert_text(GTK_EDITABLE(mainwin.ml_text), "\n", 
-				   1, &pos);
-#endif
      }
      while (log_list_len > MESSAGE_LOG_MAX) {
 	  g_free(log_list->data);
@@ -317,9 +295,7 @@ static void message_log_destroyed(GtkWidget *window,
 {
      win->ml_window = NULL;
      win->ml_text   = NULL;
-#if GTK_MAJOR >= 2
      win->ml_buffer = NULL;
-#endif
 }
 
 static void clear_clicked(void)
@@ -330,43 +306,30 @@ static void clear_clicked(void)
 static void message_log(struct mainwin_data *win) 
 {
      GtkWidget *window, *vbox0, *scrwin, *text, *bbox, *button;
-#if GTK_MAJOR >= 2
      GtkTextBuffer *buffer;
      GtkTextIter iter;
-#else
-     int pos = 0;
-#endif
      GList *I;
 
-     assert(win);
+     g_assert(win);
 
      if (win->ml_window) {
 	  gdk_beep(); /* Is this OK, philosophically? */
-#if GTK_MAJOR >= 2
 	  gtk_window_present(GTK_WINDOW(win->ml_window));
-#else
-	  gtk_window_activate_focus(GTK_WINDOW(win->ml_window));
-#endif
 	  return;
      }
 
-#if GTK_MAJOR < 2
-     window = stateful_gtk_window_new(GTK_WINDOW_DIALOG,
-				      "statusbar-log", 500, 350); 
-#else
      window = stateful_gtk_window_new(GTK_WINDOW_TOPLEVEL,
-				      "statusbar-log", 500, 350); 
-#endif
+				      "statusbar-log", 500, 350);
      win->ml_window = window;
 
      gtk_widget_realize(window);
 
-     gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			GTK_SIGNAL_FUNC(message_log_destroyed), win);
+     g_signal_connect(window, "destroy",
+			G_CALLBACK(message_log_destroyed), win);
 
-     gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			GTK_SIGNAL_FUNC(close_on_esc),
-			(gpointer) window);
+     g_signal_connect(window, "key_press_event",
+			G_CALLBACK(close_on_esc),
+			window);
 
 /*      current_search_options_window = window; */
      gtk_window_set_title(GTK_WINDOW(window), _("Message Log"));
@@ -384,23 +347,16 @@ static void message_log(struct mainwin_data *win)
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
 				    GTK_POLICY_AUTOMATIC,
 				    GTK_POLICY_AUTOMATIC);
-#if GTK_MAJOR >= 2
      gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwin),
 					 GTK_SHADOW_IN);
-#endif
      gtk_box_pack_start(GTK_BOX(vbox0), scrwin, TRUE, TRUE, 0);
 
-#if GTK_MAJOR >= 2
      text = gtk_text_view_new();
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
      win->ml_buffer = buffer;
 
      gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
      gtk_text_buffer_get_end_iter(buffer, &iter);
-#else
-     text = gtk_text_new(NULL, NULL);
-     gtk_text_set_editable(GTK_TEXT(text), FALSE);
-#endif
 
      win->ml_text = text;
 
@@ -408,18 +364,11 @@ static void message_log(struct mainwin_data *win)
      gtk_container_add(GTK_CONTAINER(scrwin), text); 
 
      for (I = log_list ; I ; I = g_list_next(I) ) {
-#if GTK_MAJOR >= 2
 	  gtk_text_buffer_insert(buffer, &iter,
 				 I->data, strlen(I->data));
 	  gtk_text_buffer_insert(buffer, &iter, "\n", 1);
-#else
-	  gtk_editable_insert_text(GTK_EDITABLE(text), I->data, 
-				   strlen(I->data), &pos);
-	  gtk_editable_insert_text(GTK_EDITABLE(text), "\n", 1, &pos);
-#endif
      }
 
-#if GTK_MAJOR >= 2
      gtk_text_buffer_get_end_iter(buffer, &iter);
      gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(text),
 				  gtk_text_buffer_create_mark(buffer, NULL,
@@ -427,33 +376,24 @@ static void message_log(struct mainwin_data *win)
 				  0.0,
 				  FALSE,
 				  0.0, 0.0);
-#endif
 
      bbox = gtk_hbutton_box_new();
      gtk_widget_show(bbox);
 
      gtk_box_pack_end(GTK_BOX(vbox0), bbox, FALSE, FALSE, 3);
 
-#if GTK_MAJOR >= 2
      button = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
-#else
-     button = gq_button_new_with_label(_("_Clear"));
-#endif
      gtk_widget_show(button);
-     gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(clear_clicked),
-			(gpointer) win);
+     g_signal_connect(button, "clicked",
+			G_CALLBACK(clear_clicked),
+			win);
      gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, TRUE, 10);
 
-#if GTK_MAJOR >= 2
      button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-#else
-     button = gq_button_new_with_label(_("_Close"));
-#endif
      gtk_widget_show(button);
-     gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			       (gpointer) window);
+     g_signal_connect_swapped(button, "clicked",
+			       G_CALLBACK(gtk_widget_destroy),
+			       window);
      gtk_box_pack_end(GTK_BOX(bbox), button, FALSE, TRUE, 10);
 
      GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
@@ -469,14 +409,14 @@ static void remove_tab(GtkContainer *notebook,
 		       struct mainwin_data *win)
 {
      int thismode;
-     struct tab *tab = NULL; 
+     GqTab *tab = NULL;
      int i;
 
      tab = gtk_object_get_data(GTK_OBJECT(content), "tab");
      if (tab) {
 	  thismode = tab->type;
-	  g_hash_table_insert(win->lastofmode, (gpointer) thismode, NULL);
-	  
+	  g_hash_table_insert(win->lastofmode, GINT_TO_POINTER(thismode), NULL);
+
 	  /* try to find another tab with the same mode so we can put that
 	     one into lastofmode... */
 	  for( i = 0 ; (tab = mainwin_get_tab_nth(win, i)) != NULL ; i++) {
@@ -505,12 +445,12 @@ void mainwin_update_filter_menu(struct mainwin_data *win)
 	  for ( I = g_list_first(menuitems) ; I ; I = g_list_next(I) ) {
 	       GtkWidget *item = GTK_WIDGET(I->data);
 	       gpointer data = gtk_object_get_data(GTK_OBJECT(item), "filter");
-	       
+
 	       if (data) {
 		    gtk_widget_destroy(item);
 	       }
 	  }
-	  
+
 	  g_list_free(menuitems);
      }
 
@@ -522,9 +462,9 @@ void mainwin_update_filter_menu(struct mainwin_data *win)
 	  menuitem = gtk_menu_item_new_with_label(filter->name);
 
 	  gtk_object_set_data(GTK_OBJECT(menuitem), "filter", filter);
-	  gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
-				    GTK_SIGNAL_FUNC(filter_selected),
-				    (GtkObject *) filter);
+	  g_signal_connect_swapped(menuitem, "activate",
+				    G_CALLBACK(filter_selected),
+				    filter);
 
 	  gtk_container_add(GTK_CONTAINER(win->filtermenu), menuitem);
 	  gtk_widget_show(menuitem);
@@ -537,21 +477,19 @@ void create_mainwin(struct mainwin_data *win)
      GtkWidget *outer_vbox, *main_vbox, *menubar, *menuitem, *submenu;
      GtkWidget *File, *menuFile, *New, *Close, *ShowM, *Quit;
      GtkWidget *Search, *Browse, *Schema;
-     GtkWidget *menuHelp, *Help, *License, *About;
+     GtkWidget *menuHelp, *Help, *About;
      GtkWidget *Filters, *menuFilters;
      GtkWidget *handlebox;
      GtkAccelGroup *accel_group;
 
-     assert(win != NULL);
-/*      mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL); */
-/*      gtk_window_set_default_size(GTK_WINDOW(mainwin), 770, 478); */
+     g_assert(win != NULL);
 
      win->mainwin = stateful_gtk_window_new(GTK_WINDOW_TOPLEVEL,
-					    "mainwin", 770, 478); 
+					    "mainwin", 770, 478);
 
      gtk_container_border_width(GTK_CONTAINER(win->mainwin), 0);
-     gtk_signal_connect(GTK_OBJECT(win->mainwin), "destroy",
-			GTK_SIGNAL_FUNC(mainwin_destroyed),
+     g_signal_connect(win->mainwin, "destroy",
+			G_CALLBACK(mainwin_destroyed),
 			win);
      gtk_window_set_title(GTK_WINDOW(win->mainwin), _("GQ"));
      gtk_window_set_policy(GTK_WINDOW(win->mainwin), FALSE, TRUE, FALSE);
@@ -565,12 +503,7 @@ void create_mainwin(struct mainwin_data *win)
 
      accel_group = gtk_accel_group_new();
 
-#if GTK_MAJOR < 2
-     gtk_accel_group_attach(accel_group, GTK_OBJECT(win->mainwin));
-#else 
      gtk_window_add_accel_group(GTK_WINDOW(win->mainwin), accel_group);
-#endif
-
 
      handlebox = gtk_handle_box_new();
      gtk_widget_show(handlebox);
@@ -599,9 +532,9 @@ void create_mainwin(struct mainwin_data *win)
      Search = gq_menu_item_new_with_label(_("_Search"));
      gtk_widget_show(Search);
      gtk_menu_append(GTK_MENU(submenu), Search);
-     gtk_signal_connect_object(GTK_OBJECT(Search), "activate",
-			       GTK_SIGNAL_FUNC(new_modetab_search),
-			       (gpointer) win);
+     g_signal_connect_swapped(Search, "activate",
+			       G_CALLBACK(new_modetab_search),
+			       win);
      gtk_widget_add_accelerator(Search, "activate", accel_group, 'S',
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
@@ -609,16 +542,16 @@ void create_mainwin(struct mainwin_data *win)
      Browse = gq_menu_item_new_with_label(_("_Browse"));
      gtk_widget_show(Browse);
      gtk_menu_append(GTK_MENU(submenu), Browse);
-     gtk_signal_connect_object(GTK_OBJECT(Browse), "activate",
-			       GTK_SIGNAL_FUNC(new_modetab_browse),
-			       (gpointer) win);
+     g_signal_connect_swapped(Browse, "activate",
+			       G_CALLBACK(new_modetab_browse),
+			       win);
      gtk_widget_add_accelerator(Browse, "activate", accel_group, 'B',
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
      /* ctrl-b is used by text widgets, so the searchterm textbox that
 	always has focus in search mode blocks the above accelerator...*/
-     gtk_signal_connect(GTK_OBJECT(win->mainwin), "key_press_event",
-			GTK_SIGNAL_FUNC(ctrl_b_hack),
-			(gpointer) Browse);
+     g_signal_connect(win->mainwin, "key_press_event",
+			G_CALLBACK(ctrl_b_hack),
+			Browse);
 
      /* File | New | Schema */
      Schema = gq_menu_item_new_with_label(_("S_chema"));
@@ -627,9 +560,9 @@ void create_mainwin(struct mainwin_data *win)
      gtk_widget_add_accelerator(Schema, "activate", accel_group, 'Z',
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
-     gtk_signal_connect_object(GTK_OBJECT(Schema), "activate",
-			       GTK_SIGNAL_FUNC(new_modetab_schema),
-			       (gpointer) win);
+     g_signal_connect_swapped(Schema, "activate",
+			       G_CALLBACK(new_modetab_schema),
+			       win);
 #else
      gtk_widget_set_sensitive(Schema, FALSE);
 #endif
@@ -638,9 +571,9 @@ void create_mainwin(struct mainwin_data *win)
      menuitem = gq_menu_item_new_with_label(_("_Preferences"));
      gtk_widget_show(menuitem);
      gtk_container_add(GTK_CONTAINER(menuFile), menuitem);
-     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
-			       GTK_SIGNAL_FUNC(create_prefs_window),
-			       (gpointer) win);
+     g_signal_connect_swapped(menuitem, "activate",
+			       G_CALLBACK(create_prefs_window),
+			       win);
      gtk_widget_add_accelerator(menuitem, "activate", accel_group, 'P',
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
@@ -648,32 +581,32 @@ void create_mainwin(struct mainwin_data *win)
      Close = gq_menu_item_new_with_label(_("_Close tab"));
      gtk_widget_show(Close);
      gtk_container_add(GTK_CONTAINER(menuFile), Close);
-     gtk_signal_connect_object(GTK_OBJECT(Close), "activate",
-			       GTK_SIGNAL_FUNC(close_current_tab), 
-			       (gpointer) win);
+     g_signal_connect_swapped(Close, "activate",
+			       G_CALLBACK(close_current_tab),
+			       win);
      gtk_widget_add_accelerator(Close, "activate", accel_group, 'W',
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
      /* :-( */
-     gtk_signal_connect(GTK_OBJECT(win->mainwin), "key_press_event",
-			GTK_SIGNAL_FUNC(ctrl_w_hack),
-			(gpointer) Close);
+     g_signal_connect(win->mainwin, "key_press_event",
+			G_CALLBACK(ctrl_w_hack),
+			Close);
 
      /* File | Show Message */
 
      ShowM = gq_menu_item_new_with_label(_("Show _Message Log"));
      gtk_widget_show(ShowM);
      gtk_container_add(GTK_CONTAINER(menuFile), ShowM);
-     gtk_signal_connect_object(GTK_OBJECT(ShowM), "activate",
-			       GTK_SIGNAL_FUNC(message_log), 
-			       (gpointer) win);
+     g_signal_connect_swapped(ShowM, "activate",
+			       G_CALLBACK(message_log),
+			       win);
 
      /* File | Quit */
      Quit = gq_menu_item_new_with_label(_("_Quit"));
      gtk_widget_show(Quit);
      gtk_container_add(GTK_CONTAINER(menuFile), Quit);
-     gtk_signal_connect_object(GTK_OBJECT(Quit), "activate",
-			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			       GTK_OBJECT(win->mainwin));
+     g_signal_connect_swapped(Quit, "activate",
+			       G_CALLBACK(gtk_widget_destroy),
+			       win->mainwin);
      gtk_widget_add_accelerator(Quit, "activate", accel_group, 'Q',
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
@@ -698,24 +631,24 @@ void create_mainwin(struct mainwin_data *win)
      menuitem = gq_menu_item_new_with_label(_("From _Search tab"));
      gtk_widget_show(menuitem);
      gtk_menu_append(GTK_MENU(submenu), menuitem);
-     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
-			       GTK_SIGNAL_FUNC(add_filter),
+     g_signal_connect_swapped(menuitem, "activate",
+			       G_CALLBACK(add_filter),
 			       NULL);
 
      /* Filters | New | Filter Editor */
      menuitem = gq_menu_item_new_with_label(_("Filter _editor"));
      gtk_widget_show(menuitem);
      gtk_menu_append(GTK_MENU(submenu), menuitem);
-     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
-			       GTK_SIGNAL_FUNC(add_new_filter_callback),
+     g_signal_connect_swapped(menuitem, "activate",
+			       G_CALLBACK(add_new_filter_callback),
 			       NULL);
 
      /* Filters | Edit Filters */
      menuitem = gq_menu_item_new_with_label(_("_Edit Filters"));
      gtk_widget_show(menuitem);
      gtk_container_add(GTK_CONTAINER(menuFilters), menuitem);
-     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
-			       GTK_SIGNAL_FUNC(show_filters),
+     g_signal_connect_swapped(menuitem, "activate",
+			       G_CALLBACK(show_filters),
 			       NULL);
 
      /* Filters separator */
@@ -729,24 +662,16 @@ void create_mainwin(struct mainwin_data *win)
      Help = gq_menu_item_new_with_label(_("_Help"));
      gtk_widget_show(Help);
      gtk_container_add(GTK_CONTAINER(menubar), Help);
-     gtk_menu_item_right_justify(GTK_MENU_ITEM(Help));
 
      menuHelp = gtk_menu_new();
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(Help), menuHelp);
 
-     /* Help | License */
-     License = gq_menu_item_new_with_label(_("_License"));
-     gtk_widget_show(License);
-     gtk_container_add(GTK_CONTAINER(menuHelp), License);
-     gtk_signal_connect(GTK_OBJECT(License), "activate",
-			GTK_SIGNAL_FUNC(create_license_window), NULL);
-
      /* Help | About */
      About = gq_menu_item_new_with_label(_("_About"));
      gtk_widget_show(About);
      gtk_container_add(GTK_CONTAINER(menuHelp), About);
-     gtk_signal_connect(GTK_OBJECT(About), "activate",
-			GTK_SIGNAL_FUNC(create_about_window), NULL);
+     g_signal_connect_swapped(About, "activate",
+			      G_CALLBACK(create_about_window), win->mainwin);
 
      main_vbox = gtk_vbox_new(FALSE, 2);
      gtk_container_border_width(GTK_CONTAINER(main_vbox), 4);
@@ -764,10 +689,10 @@ void create_mainwin(struct mainwin_data *win)
      gtk_box_pack_end(GTK_BOX(outer_vbox), win->statusbar, FALSE, FALSE, 0);
      gtk_widget_set_sensitive(win->statusbar, TRUE);
      
-     gtk_signal_connect(GTK_OBJECT(win->mainbook), "switch-page",
-			GTK_SIGNAL_FUNC(switchpage_refocus), win);
-     gtk_signal_connect(GTK_OBJECT(win->mainbook), "remove",
-			GTK_SIGNAL_FUNC(remove_tab), win);
+     g_signal_connect(win->mainbook, "switch-page",
+			G_CALLBACK(switchpage_refocus), win);
+     g_signal_connect(win->mainbook, "remove",
+			G_CALLBACK(remove_tab), win);
 
      gtk_widget_realize(win->mainwin);
 
@@ -780,7 +705,7 @@ void create_mainwin(struct mainwin_data *win)
      gtk_widget_show(win->mainwin);
 }
 
-struct tab *mainwin_get_tab_nth(struct mainwin_data *win, int n)
+GqTab *mainwin_get_tab_nth(struct mainwin_data *win, int n)
 {
      GtkWidget *content = 
 	  gtk_notebook_get_nth_page(GTK_NOTEBOOK(win->mainbook), n);
@@ -789,7 +714,7 @@ struct tab *mainwin_get_tab_nth(struct mainwin_data *win, int n)
      return gtk_object_get_data(GTK_OBJECT(content), "tab");
 }
 
-struct tab *mainwin_get_current_tab(GtkWidget *notebook)
+GqTab *mainwin_get_current_tab(GtkWidget *notebook)
 {
      int tabnum = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
      GtkWidget *content = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 
@@ -797,10 +722,10 @@ struct tab *mainwin_get_current_tab(GtkWidget *notebook)
      return gtk_object_get_data(GTK_OBJECT(content), "tab");
 }
 
-struct tab *new_modetab(struct mainwin_data *win, int mode)
+GqTab *new_modetab(struct mainwin_data *win, int mode)
 {
      GtkWidget *label, *focusbox;
-     struct tab *tab;
+     GqTab *tab;
      int focus;
 
      /* hack, hack */
@@ -852,7 +777,7 @@ static void switchpage_refocus(GtkNotebook *notebook, GtkNotebookPage *page,
 			       int pagenum, struct mainwin_data *win)
 {
      GtkWidget *focusbox;
-     struct tab *tab;
+     GqTab *tab;
 
      tab = mainwin_get_tab_nth(win, pagenum);
      if(!tab)
@@ -860,7 +785,7 @@ static void switchpage_refocus(GtkNotebook *notebook, GtkNotebookPage *page,
 
      /* retrieve mode, store this pane as the last one used for this mode */
      enter_last_of_mode(tab);
-     
+
      focusbox = tab->focus;
      if(focusbox) {
 	  gtk_widget_grab_focus(focusbox);
@@ -872,8 +797,8 @@ static void switchpage_refocus(GtkNotebook *notebook, GtkNotebookPage *page,
 void cleanup_all_tabs(struct mainwin_data *win)
 {
      /* don't waste time refocusing on disappearing tabs */
-     gtk_signal_disconnect_by_func(GTK_OBJECT(win->mainbook),
-				   GTK_SIGNAL_FUNC(switchpage_refocus), win);
+     g_signal_handlers_disconnect_by_func(win->mainbook,
+				   G_CALLBACK(switchpage_refocus), win);
 }
 
 
@@ -895,13 +820,13 @@ static void close_current_tab(struct mainwin_data *win)
 
 void update_serverlist(struct mainwin_data *win)
 {
-     struct tab *tab;
+     GqTab *tab;
      int i;
 
      for( i = 0 ; (tab = mainwin_get_tab_nth(win, i)) != NULL ; i++) {
 	  switch(tab->type) {
 	  case SEARCH_MODE:
-	       fill_serverlist_combo(SEARCHTAB(tab)->serverlist_combo);
+	       fill_serverlist_combo(GQ_TAB_SEARCH(tab)->serverlist_combo);
 	       break;
 	  case BROWSE_MODE:
 	       update_browse_serverlist(tab);
@@ -913,237 +838,38 @@ void update_serverlist(struct mainwin_data *win)
      }
 }
 
-static GtkWidget *license_window = NULL;
-
-static void close_license_window(GtkWidget *window)
-{
-     gtk_widget_destroy(window);
+static void
+create_about_window(GtkWindow* parent) {
+	static gchar const* authors[] = {
+		"Bert Vermeulen",
+		"Peter Stamfest",
+		"David Malcom",
+		"Sven Herzberg (current maintainer)",
+		NULL
+	};
+	GdkPixbuf* logo = gdk_pixbuf_new_from_file(PACKAGE_PREFIX "/share/pixmaps/gq/gq.xpm", NULL);
+	gtk_show_about_dialog(parent,
+			      // "artists", NULL,
+			      "authors", authors,
+			      "comments", _("The gentleman's LDAP client"),
+			      "copyright", _("Copyright (C) 1998-2003 Bert Vermeulen\n"
+					     "Copyright (C) 2002-2003 Peter Stamfest\n"
+					     "Copyright (C) 2006 Sven Herzberg"),
+			      // documenters
+			      "license", license,
+			      "logo", logo,
+			      "name", _("GQ LDAP Client"),
+			      "translator-credits", _("translator-credits"),
+			      "version", VERSION,
+			      "website", "http://www.gq-project.org/",
+			      "website-label", _("GQ Website"),
+			      NULL);
+	if(logo) {
+		g_object_unref(logo);
+	}
 }
 
-static void destroy_license_window(GtkWidget *window)
-{
-     assert(window == license_window);
-     license_window = NULL;
-}
-
-static void create_license_window(void)
-{
-#if GTK_MAJOR < 2
-     GtkWidget *hbox;
-     GtkStyle *license_style;
-#else
-     PangoFontDescription *font_desc;
-#endif
-     GtkWidget *window, *vbox1, *vbox2, *text, *hbox0, *w;
-     GtkWidget *ok_button;
-
-     if (license_window) {
-#if GTK_MAJOR >= 2
-	  gtk_window_present(GTK_WINDOW(license_window));
-#else
-	  gtk_window_activate_focus(GTK_WINDOW(license_window));
-#endif
-	  return;
-     }
-
-     license_window = window = gtk_dialog_new();
-     gtk_widget_set_usize(window, 540, 400);
-     gtk_window_set_title(GTK_WINDOW(window), _("License"));
-
-     vbox1 = GTK_DIALOG(window)->vbox;
-/*       gtk_container_border_width(GTK_CONTAINER(vbox1), 20); */
-     gtk_container_border_width(GTK_CONTAINER(vbox1), CONTAINER_BORDER_WIDTH);
-     gtk_widget_show(vbox1);
-
-#if GTK_MAJOR < 2
-     hbox = gtk_hbox_new(FALSE, 0);
-/*      gtk_container_border_width(GTK_CONTAINER(hbox), CONTAINER_BORDER_WIDTH); */
-     gtk_widget_show(hbox);
-     gtk_box_pack_start(GTK_BOX(vbox1), hbox, TRUE, TRUE, 0);
-
-     license_style = gtk_style_new();
-     license_style->font = gdk_font_load("fixed");
-
-     text = gtk_text_new(NULL, NULL);
-     gtk_widget_set_style(text, license_style);
-     gtk_style_unref(license_style);
-
-     gtk_text_set_point(GTK_TEXT(text), 0);
-     gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		     license, strlen(license));
-     gtk_widget_show(text);
-     gtk_text_set_editable(GTK_TEXT(text), FALSE);
-
-     gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);
-
-     w = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
-     gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-     gtk_widget_show(w);
-#else
-     text = gtk_text_view_new();
-
-     gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
-			      license, strlen(license));
-
-     /* Change default font throughout the widget */
-     font_desc = pango_font_description_from_string("Monospace");
-     gtk_widget_modify_font(text, font_desc);
-     pango_font_description_free(font_desc);
-
-     gtk_widget_show(text);
-
-     w = gtk_scrolled_window_new(NULL, NULL);
-     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w), 
-				    GTK_POLICY_AUTOMATIC,
-				    GTK_POLICY_AUTOMATIC);
-     gtk_container_add(GTK_CONTAINER(w), text); 
-     gtk_widget_show(w);
-
-     gtk_box_pack_start(GTK_BOX(vbox1), w, TRUE, TRUE, 0);
-#endif
-
-     vbox2 = GTK_DIALOG(window)->action_area;
-     gtk_widget_show(vbox2);
-
-     hbox0 = gtk_hbutton_box_new();
-     gtk_widget_show(hbox0);
-     gtk_box_pack_start(GTK_BOX(vbox2), hbox0, TRUE, FALSE, 0);
-
-#if GTK_MAJOR >= 2
-     ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
-#else
-     ok_button = gq_button_new_with_label(_("_OK"));
-#endif
-     gtk_widget_show(ok_button);
-     gtk_box_pack_start(GTK_BOX(hbox0), ok_button,
-			FALSE, FALSE, 0);
-     GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
-     gtk_widget_grab_default(ok_button);
-     gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			(GtkSignalFunc) destroy_license_window,
-			NULL);
-     gtk_signal_connect_object(GTK_OBJECT(ok_button), "clicked",
-			       (GtkSignalFunc) close_license_window,
-			       GTK_OBJECT(window));
-     gtk_signal_connect_object(GTK_OBJECT(window), "key_press_event",
-			       (GtkSignalFunc) close_on_esc,
-			       (gpointer) window);
-
-     gtk_widget_show(window);
-
-}
-
-static GtkWidget *about_window = NULL;
-
-static void destroy_about_window(GtkWidget *window)
-{
-     assert(window == about_window);
-     about_window = NULL;
-}
-
-static GtkWidget *create_about_window(void)
-{
-     GtkWidget *dialog_vbox2, *hbox0, *hbox;
-     GtkWidget *about_label;
-     GtkWidget *dialog_action_area2;
-     GtkWidget *about_ok_button;
-     GtkWidget *pixmap;
-     GdkPixmap *gq_icon;
-     GdkBitmap *gq_icon_mask;
-     GString *about_text = NULL;
-
-     if (about_window) {
-#if GTK_MAJOR >= 2
-	  gtk_window_present(GTK_WINDOW(about_window));
-#else
-	  gtk_window_activate_focus(GTK_WINDOW(about_window));
-#endif
-	  return about_window;
-     }
-
-     about_window = gtk_dialog_new();
-     gtk_widget_realize(GTK_WIDGET(about_window));
-/*       gtk_container_border_width(GTK_CONTAINER(about_window), 12); */
-
-     gtk_window_set_title(GTK_WINDOW(about_window), _("About GQ"));
-     gtk_window_set_policy(GTK_WINDOW(about_window), FALSE, FALSE, FALSE);
-
-     dialog_vbox2 = GTK_DIALOG(about_window)->vbox;
-     gtk_widget_show(dialog_vbox2);
-
-     hbox = gtk_hbox_new(FALSE, 0);
-     gtk_widget_show(hbox);
-     gtk_box_pack_start(GTK_BOX(dialog_vbox2), hbox, TRUE, TRUE, 10);
-
-     gq_icon = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(about_window)->window,
-					    &gq_icon_mask,
-					    &about_window->style->white,
-					    logo_xpm);
-     pixmap = gtk_pixmap_new(gq_icon, gq_icon_mask);
-     gtk_widget_show(pixmap);
-     gtk_box_pack_start(GTK_BOX(hbox), pixmap, TRUE, TRUE, 10);
-
-     about_text = g_string_sized_new(512);
-
-     g_string_sprintf(about_text,
-		      "GQ %s\n\n\n%s\n\n%s\n%s\n%s\n\n%s\n\n%s\n\n"
-		      "GLib version %d.%d.%d\n"
-		      "GTK version %d.%d.%d\n",
-		      VERSION, _("The gentleman's LDAP client"),
-		      _("Copyright (C) 1998-2003 Bert Vermeulen <bert@biot.com>"),
-                      _("Copyright (C) 2002-2003 Peter Stamfest <peter@stamfest.at>"),
-                      _("Copyright (C) 2006 Sven Herzberg"),
-		      _("For license terms please see the license window"),
-		      _("http://biot.com/gq/"),
-		      glib_major_version, glib_minor_version, glib_micro_version,
-		      gtk_major_version, gtk_minor_version, gtk_micro_version);
-
-     about_label = gtk_label_new(about_text->str);
-     g_string_free(about_text, TRUE);
-
-     gtk_label_set_justify(GTK_LABEL(about_label), GTK_JUSTIFY_LEFT);
-     gtk_widget_show(about_label);
-     gtk_box_pack_start(GTK_BOX(hbox), about_label, FALSE, FALSE, 4);
-
-     dialog_action_area2 = GTK_DIALOG(about_window)->action_area;
-     gtk_container_border_width(GTK_CONTAINER(dialog_action_area2), 
-				CONTAINER_BORDER_WIDTH);
-     gtk_widget_show(dialog_action_area2);
-
-     hbox0 = gtk_hbutton_box_new();
-     gtk_widget_show(hbox0);
-     gtk_box_pack_start(GTK_BOX(dialog_action_area2), hbox0,
-			TRUE, FALSE, 0);
-
-#if GTK_MAJOR >= 2
-     about_ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
-#else
-     about_ok_button = gq_button_new_with_label(_("_OK"));
-#endif
-     gtk_widget_show(about_ok_button);
-     gtk_box_pack_start(GTK_BOX(hbox0), about_ok_button,
-			TRUE, FALSE, 0);
-     GTK_WIDGET_SET_FLAGS(about_ok_button, GTK_CAN_DEFAULT);
-     gtk_widget_grab_default(about_ok_button);
-/*       gtk_container_border_width(GTK_CONTAINER(about_ok_button), 7); */
-     gtk_signal_connect_object(GTK_OBJECT(about_ok_button), "clicked",
-			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			       GTK_OBJECT(about_window));
-
-     gtk_signal_connect_object(GTK_OBJECT(about_window), "key_press_event",
-			       GTK_SIGNAL_FUNC(close_on_esc), (gpointer) about_window);
-
-     gtk_signal_connect(GTK_OBJECT(about_window), "destroy",
-			(GtkSignalFunc) destroy_about_window,
-			NULL);
-
-     gtk_widget_show(about_window);
-
-     return(about_window);
-}
-
-
-/* 
+/*
    Local Variables:
    c-basic-offset: 5
    End:
