@@ -22,15 +22,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: mainwin.h,v 1.26 2006/04/07 21:37:09 herzi Exp $ */
+/* $Id: mainwin.h 960 2006-09-02 20:42:46Z herzi $ */
 
 #ifndef GQ_MAINWIN_H_INCLUDED
 #define GQ_MAINWIN_H_INCLUDED
 
-
-#include "config.h"
-#include "common.h"
 #include <gtk/gtk.h>
+#include "common.h"
+#include "gq-tab.h"
 
 struct mainwin_data {
      GtkWidget *mainwin;
@@ -42,59 +41,31 @@ struct mainwin_data {
      /* the message log window */
      GtkWidget *ml_window;
      GtkWidget *ml_text;
-#if GTK_MAJOR >= 2
      GtkTextBuffer *ml_buffer;
-#endif
-};
-
-#define SEARCH_MODE       1
-#define BROWSE_MODE       2
-#define SCHEMA_MODE       3
-
-struct tab;
-struct pbar_win;
-
-struct tab_vtab {
-     void (*save_snapshot)(int error_context, char *state_name, struct tab *);
-     void (*restore_snapshot)(int error_context, 
-			      char *state_name, struct tab *, 
-			      struct pbar_win *progress);
-};
-
-struct tab {
-     int type;
-     struct mainwin_data *win;		/* what window does this tab
-					   belong to */
-     GtkWidget *focus;
-     GtkWidget *content;		/* what is the "content"
-					   widget of the tab */
-     void *modeinfo;
-
-     struct tab_vtab *vtab;
 };
 
 extern struct mainwin_data mainwin;
 
-struct tab *get_last_of_mode(int mode);
-void go_to_page(struct tab *tab);
-void enter_last_of_mode(struct tab *tab);
+GqTab *get_last_of_mode(int mode);
+void go_to_page(GqTab *tab);
+void enter_last_of_mode(GqTab *tab);
 
 void fill_serverlist_combo(GtkWidget *combo);
 void cleanup(struct mainwin_data *win);
 void create_mainwin(struct mainwin_data *);
 void mainwin_update_filter_menu(struct mainwin_data *win);
 
-struct tab *new_modetab(struct mainwin_data *, int mode);
+GqTab *new_modetab(struct mainwin_data *, int mode);
 void cleanup_all_tabs(struct mainwin_data *win);
-void cleanup_tab(struct tab *tab);
+void cleanup_tab(GqTab *tab);
 void update_serverlist(struct mainwin_data *win);
 
 /* gboolean ctrl_b_hack(GtkWidget *widget, GdkEventKey *event, gpointer obj); */
 /* gboolean ctrl_w_hack(GtkWidget *widget, GdkEventKey *event, gpointer obj); */
 
-/* return the struct tab for the n'th tab in the gq main window win */
-struct tab *mainwin_get_tab_nth(struct mainwin_data *win, int n);
-struct tab *mainwin_get_current_tab(GtkWidget *notebook);
+/* return the GqTab for the n'th tab in the gq main window win */
+GqTab *mainwin_get_tab_nth(struct mainwin_data *win, int n);
+GqTab *mainwin_get_current_tab(GtkWidget *notebook);
 
 void message_log_append(const char *buf);
 void clear_message_history();
