@@ -22,9 +22,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: schema.c,v 1.28 2003/11/07 12:02:12 biot Exp $ */
+/* $Id: schema.c 955 2006-08-31 19:15:21Z herzi $ */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
 
@@ -35,11 +37,13 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+
 #include "common.h"
 #include "configfile.h"
-#include "util.h"
+#include "gq-server-list.h"
 #include "schema.h"
-#include "i18n.h"
+#include "util.h"
 #include "debug.h"
 #include "errorchain.h"
 
@@ -49,7 +53,7 @@
  * examine server's root DSE for schema info, store directly
  * in ldapserver struct
  */
-struct server_schema *get_schema(int error_context, struct ldapserver *server)
+struct server_schema *get_schema(int error_context, GqServer *server)
 {
      struct server_schema *ss;
 
@@ -65,7 +69,7 @@ struct server_schema *get_schema(int error_context, struct ldapserver *server)
      if(ss == NULL) {
 	  /* server didn't publish a schema, try the last resort
 	     schemaserver */
-	  server = server_by_name(config->schemaserver);
+	  server = gq_server_list_get_by_name(gq_server_list_get(), config->schemaserver);
 	  if(server == NULL) {
 	       error_push(error_context,
 			  _("Cannot find last-resort schema server '%s'"),
@@ -96,7 +100,7 @@ struct server_schema *get_schema(int error_context, struct ldapserver *server)
 
 
 struct server_schema *get_server_schema(int error_context, 
-					struct ldapserver *server)
+					GqServer *server)
 {
      BerElement *berptr;
      LDAP *ld;
@@ -191,7 +195,7 @@ struct server_schema *get_server_schema(int error_context,
 			      &res);
      }
 
-     g_free_if(subschema);
+     g_free(subschema);
 
      if(msg != LDAP_SUCCESS) {
 	  if (msg == LDAP_SERVER_DOWN) {
@@ -377,7 +381,7 @@ LDAPObjectClass *find_oc_by_oc_name(struct server_schema *ss, char *ocname)
 }
 
 
-GList *attrlist_by_oclist(struct ldapserver *server, GList *oclist)
+GList *attrlist_by_oclist(GqServer *server, GList *oclist)
 {
      GList *attrlist;
      LDAPObjectClass *oc;
