@@ -1,6 +1,7 @@
 /*
     GQ -- a GTK-based LDAP client
-    Copyright (C) 1998-2001 Bert Vermeulen
+    Copyright (C) 1998-2003 Bert Vermeulen
+    Copyright (C) 2002-2003 Peter Stamfest
 
     This program is released under the Gnu General Public License with
     the additional exemption that compiling, linking, and/or using
@@ -30,17 +31,21 @@
 #include <ldap.h>
 #include <ldap_schema.h>
 
+#include "common.h"
+
 #define GQ_SCHEMA_PARSE_FLAG    0x03
 
-struct server_schema *get_schema(struct ldapserver *server);
-struct server_schema *get_server_schema(struct ldapserver *server);
+struct server_schema *get_schema(int error_context, GqServer *server);
+struct server_schema *get_server_schema(int error_context,
+					GqServer *server);
+
 int sort_oc(LDAPObjectClass *oc1, LDAPObjectClass *oc2);
 int sort_at(LDAPAttributeType *at1, LDAPAttributeType *at2);
 int sort_mr(LDAPMatchingRule *mr1, LDAPMatchingRule *mr2);
 int sort_s(LDAPSyntax *s1, LDAPSyntax *s2);
 
 LDAPObjectClass *find_oc_by_oc_name(struct server_schema *ss, char *ocname);
-GList *attrlist_by_oclist(struct ldapserver *server, GList *oclist);
+GList *attrlist_by_oclist(GqServer *server, GList *oclist);
 
 #endif
 
