@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: syntax.c,v 1.22 2006/04/05 00:53:55 herzi Exp $ */
+/* $Id: syntax.c 955 2006-08-31 19:15:21Z herzi $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -33,7 +33,9 @@
 #include <lber.h>
 #include <ldap.h>
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include  <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #include "common.h"
 #include "util.h"
@@ -53,6 +55,7 @@
 #include "dt_time.h"
 #include "dt_int.h"
 #include "dt_numstr.h"
+#include "dt_date.h"
 
 /* Syntaxes we recognize and may handle specially. This is a rather
    inefficient way to store oids (we search by string-comparison, but
@@ -61,70 +64,71 @@
 
 static int octetString_attr(const char *attr, const char *syntax_oid);
 static int oid_attr(const char *attr, const char *syntax_oid);
+static int int_attr(const char *attr, const char *syntax_oid);
 
 static struct syntax_handler syntaxes[] = {
      { "1.3.6.1.4.1.1466.115.121.1.1",
-       "ACI Item", 
+       "ACI Item",
        DISPLAYTYPE_BINARY,
-       NULL, 
+       NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.2",
-       "Access Point", 
+       "Access Point",
        DISPLAYTYPE_ENTRY,
        NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.3",
-       "Attribute Type Description", 
+       "Attribute Type Description",
        DISPLAYTYPE_ENTRY,
        NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.4",
-       "Audio", 
+       "Audio",
        DISPLAYTYPE_BINARY,
        NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.5",
-       "Binary", 
+       "Binary",
        DISPLAYTYPE_BINARY,
        NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.6",
-       "Bit String", 
+       "Bit String",
        DISPLAYTYPE_ENTRY,
        NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.7",
-       "Boolean", 
+       "Boolean",
        DISPLAYTYPE_ENTRY,
        NULL,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.8",
-       "Certificate", 
+       "Certificate",
        DISPLAYTYPE_CERT,
        NULL,
        1,
      },
      { "1.3.6.1.4.1.1466.115.121.1.9",
-       "Certificate List", 
+       "Certificate List",
        DISPLAYTYPE_CRL,
        NULL,
        1,
      },
      { "1.3.6.1.4.1.1466.115.121.1.10",
-       "Certificate Pair", 
+       "Certificate Pair",
        DISPLAYTYPE_BINARY,
        NULL,
        1,
      },
      { "1.3.6.1.4.1.1466.115.121.1.11",
-       "Country String", 
+       "Country String",
        DISPLAYTYPE_ENTRY,
        NULL,
        0,
@@ -222,7 +226,7 @@ static struct syntax_handler syntaxes[] = {
      { "1.3.6.1.4.1.1466.115.121.1.27",
        "INTEGER",
        DISPLAYTYPE_INT,
-       NULL,
+       int_attr,
        0,
      },
      { "1.3.6.1.4.1.1466.115.121.1.28",
@@ -421,11 +425,7 @@ static GHashTable *syntax_hash = NULL;
 
 static int octetString_attr(const char *attr, const char *syntax_oid) {
      if(!strcasecmp(attr, "userPassword")) {
-#ifdef HAVE_LIBCRYPTO
 	  return DISPLAYTYPE_PASSWORD;
-#else
-	  return DISPLAYTYPE_ENTRY;
-#endif
      }
 #ifdef HAVE_LIBCRYPTO
      if(!strcasecmp(attr, "nDSPKIPublicKeyCertificate")) {
@@ -444,6 +444,16 @@ static int oid_attr(const char *attr, const char *syntax_oid) {
      return DISPLAYTYPE_ENTRY;
 }
 
+static int int_attr(const char *attr, const char *syntax_oid) {
+     if(!strcasecmp(attr, "shadowLastChange")) {
+	  return DISPLAYTYPE_DATE;
+     }
+     if(!strcasecmp(attr, "shadowExpire")) {
+	  return DISPLAYTYPE_DATE;
+     }
+     return DISPLAYTYPE_INT;
+}
+
 /* mapping of display types to handlers */
 static GHashTable *dt_handlers = NULL;
 
@@ -454,76 +464,70 @@ static GHashTable *handlers_dt = NULL;
 
 static GList *selectable_dt_list = NULL;
 
-static void add_syntax(int type, display_type_handler *dt_handler)
-{
-     int *p = (int *) g_malloc(sizeof(int)); /* leak memory */
-     *p = type;
+static void
+add_syntax(int type, GType class_type) {
+	g_return_if_fail(g_type_is_a(class_type, GQ_TYPE_TYPE_DISPLAY));
 
-     if (dt_handlers == NULL) {
-	  dt_handlers = g_hash_table_new(g_int_hash, g_int_equal);
-     }
-     if (handlers_dt == NULL) {
-	  handlers_dt = g_hash_table_new(g_direct_hash, g_direct_equal);
-     }
+	if (dt_handlers == NULL) {
+		dt_handlers = g_hash_table_new(g_direct_hash, g_direct_equal);
+	}
+	if (handlers_dt == NULL) {
+		handlers_dt = g_hash_table_new(g_direct_hash, g_direct_equal);
+	}
 
-     g_hash_table_insert(dt_handlers, p, dt_handler);
-     g_hash_table_insert(handlers_dt, dt_handler, p);
+	g_hash_table_insert(dt_handlers, GINT_TO_POINTER(type), GINT_TO_POINTER(class_type));
+	g_hash_table_insert(handlers_dt, GINT_TO_POINTER(class_type), GINT_TO_POINTER(type));
 
-     if (dt_handler->selectable) {
-	  selectable_dt_list = g_list_append(selectable_dt_list, dt_handler);
-     }
+	if (!G_TYPE_IS_ABSTRACT(class_type)) {
+		selectable_dt_list = g_list_append(selectable_dt_list, GINT_TO_POINTER(class_type));
+	}
 }
 
-GList *get_selectable_displaytypes() 
+GList *get_selectable_displaytypes()
 {
      return selectable_dt_list;
 }
 
-display_type_handler *get_dt_handler(int type) {
-     display_type_handler *h = NULL;
-     if (dt_handlers) { 
-	  h = g_hash_table_lookup(dt_handlers, &type);
+GType
+get_dt_handler(int type) {
+     GType h = G_TYPE_INVALID;
+     if (dt_handlers) {
+	  h = GPOINTER_TO_INT(g_hash_table_lookup(dt_handlers, GINT_TO_POINTER(type)));
      }
      return h;
 }
 
-int get_dt_from_handler(display_type_handler *h) {
-     int *dt = NULL;
+int get_dt_from_handler(GType h) {
+     gpointer dt = NULL;
 
-     if (handlers_dt) { 
-	  dt = g_hash_table_lookup(handlers_dt, h);
+     if (handlers_dt) {
+	  dt = g_hash_table_lookup(handlers_dt, GINT_TO_POINTER(h));
      }
-     return dt ? *dt : -1;
+     return dt != NULL ? GPOINTER_TO_INT(dt) : -1;
 }
 
-/* initialize the displaytype -> dt_handler hash. 
-
-   NOTE: this leaks memory (but I do not care for it currently, gq
-   doesn't get embedded) */
+/* initialize the displaytype -> dt_handler hash */
 void init_syntaxes(void) {
-    add_syntax(DISPLAYTYPE_ENTRY,	dt_entry_get_handler());
-    add_syntax(DISPLAYTYPE_TEXT,	dt_text_get_handler());
-    add_syntax(DISPLAYTYPE_BINARY,	dt_binary_get_handler());
-#ifdef HAVE_LIBCRYPTO
-    add_syntax(DISPLAYTYPE_PASSWORD,	dt_password_get_handler());
-#endif
-#ifdef HAVE_PIXBUF
-    add_syntax(DISPLAYTYPE_JPEG,	dt_jpeg_get_handler());
-#endif
+    add_syntax(DISPLAYTYPE_ENTRY,	GQ_TYPE_DISPLAY_ENTRY);
+    add_syntax(DISPLAYTYPE_TEXT,	GQ_TYPE_DISPLAY_TEXT);
+    add_syntax(DISPLAYTYPE_BINARY,	GQ_TYPE_DISPLAY_BINARY);
+    add_syntax(DISPLAYTYPE_PASSWORD,	GQ_TYPE_DISPLAY_PASSWORD);
+    add_syntax(DISPLAYTYPE_JPEG,	GQ_TYPE_DISPLAY_JPEG);
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
-    add_syntax(DISPLAYTYPE_OC,		dt_oc_get_handler());
+    add_syntax(DISPLAYTYPE_OC,		GQ_TYPE_DISPLAY_OC);
 #endif
 #ifdef HAVE_LIBCRYPTO
-    add_syntax(DISPLAYTYPE_CERT,	dt_cert_get_handler());
-    add_syntax(DISPLAYTYPE_CRL,		dt_crl_get_handler());
+    add_syntax(DISPLAYTYPE_CERT,	GQ_TYPE_DISPLAY_CERT);
+    add_syntax(DISPLAYTYPE_CRL,		GQ_TYPE_DISPLAY_CRL);
 #endif
-    add_syntax(DISPLAYTYPE_TIME,	dt_time_get_handler());
-    add_syntax(DISPLAYTYPE_INT,		dt_int_get_handler());
-    add_syntax(DISPLAYTYPE_NUMSTR,	dt_numstr_get_handler());
+    add_syntax(DISPLAYTYPE_TIME,	GQ_TYPE_DISPLAY_TIME);
+    add_syntax(DISPLAYTYPE_INT,		GQ_TYPE_DISPLAY_INT);
+    add_syntax(DISPLAYTYPE_NUMSTR,	GQ_TYPE_DISPLAY_NUMSTR);
+    add_syntax(DISPLAYTYPE_DATE,	GQ_TYPE_DISPLAY_DATE);
 }
 
-struct syntax_handler *get_syntax_handler_of_attr(int error_context, 
-						  struct ldapserver *server, 
+struct syntax_handler *get_syntax_handler_of_attr(int error_context,
+						  GqServer *server,
 						  const char *attrname,
 						  const char *oidin)
 {
@@ -537,7 +541,7 @@ struct syntax_handler *get_syntax_handler_of_attr(int error_context,
 	 syntax_hash = g_hash_table_new(g_str_hash, g_str_equal);
 	 /* lookup syntax... */
 	 for(sh = syntaxes ; sh->syntax_oid ; sh++) {
-	     g_hash_table_insert(syntax_hash, (gpointer) sh->syntax_oid, sh);
+	     g_hash_table_insert(syntax_hash, GINT_TO_POINTER(sh->syntax_oid), sh);
 	 }
      }
 
@@ -545,8 +549,8 @@ struct syntax_handler *get_syntax_handler_of_attr(int error_context,
      return sh;
 }
 
-int get_display_type_of_attr(int error_context, 
-			     struct ldapserver *server, 
+int get_display_type_of_attr(int error_context,
+			     GqServer *server,
 			     const char *attrname)
 {
      int dt = -1;
@@ -555,7 +559,7 @@ int get_display_type_of_attr(int error_context,
 
      /* Those without a syntax (most notably cn and sn) most often are
         printable anyway .... */
-    
+
      if (!oid) return DISPLAYTYPE_ENTRY;
 
      sh = get_syntax_handler_of_attr(error_context, server, attrname, oid);
@@ -563,30 +567,30 @@ int get_display_type_of_attr(int error_context,
      if (sh) {
 	 if (sh->displayTypeFunc) {
 	     dt = sh->displayTypeFunc(attrname, oid);
-	 } 
+	 }
 	 if (dt == -1) {
 	     dt = sh->displaytype;
 	 }
-	 
+
 	 return dt;
      }
 
      return DISPLAYTYPE_BINARY;
 }
 
-int show_in_search(int error_context, 
-		   struct ldapserver *server, const char *attrname)
+int show_in_search(int error_context,
+		   GqServer *server, const char *attrname)
 {
-     display_type_handler *dh;
+     GType type;
      struct syntax_handler *sh =
 	  get_syntax_handler_of_attr(error_context, server, attrname, NULL);
-     
+
      if (!sh) return FALSE;
 
-     dh = get_dt_handler(sh->displaytype);
-     if (!dh) return FALSE;
+     type = GPOINTER_TO_INT(get_dt_handler(sh->displaytype));
+     if (!type) return FALSE;
 
-     return dh->show_in_search_result;
+     return G_TYPE_IS_ABSTRACT(type);
 }
 
 GByteArray *identity(const char *val, int len)
@@ -595,7 +599,7 @@ GByteArray *identity(const char *val, int len)
      g_byte_array_set_size(gb, len + 1);
      if (gb) {
 	  memcpy(gb->data, val, len);
-	  /* uuuhhh this is ugly.... AND dangerous... currently we trust 
+	  /* uuuhhh this is ugly.... AND dangerous... currently we trust
 	     g_byte_array_set_size to not really make [len] inaccessible */
 	  gb->data[len] = 0;
 	  g_byte_array_set_size(gb, len);
@@ -640,14 +644,14 @@ LDAPMod *bervalLDAPMod(struct formfill *form,
 
      mod->mod_bvalues = (struct berval **)
 	  calloc(g_list_length(values) + 1, sizeof(struct berval *));
-     
+
      if (mod->mod_bvalues == NULL) {
 	  free(mod);
 	  return NULL;
      }
-     
+
      cval = 0;
-    
+
 /*       printf("bervalLDAPMod op=%d attr=%s\n", op, mod->mod_type); */
 
      while(values) {
@@ -666,10 +670,10 @@ LDAPMod *bervalLDAPMod(struct formfill *form,
 	  }
 	  mod->mod_bvalues[cval]->bv_len = copy->len;
 	  mod->mod_bvalues[cval]->bv_val = (gchar*)copy->data;
-	
-/*  	  printf("bervalLDAPMod data=%s (%d)\n", copy->data, copy->len); */
+
+/*	  printf("bervalLDAPMod data=%s (%d)\n", copy->data, copy->len); */
 	  g_byte_array_free(copy, FALSE);
-	
+
 	  values = values->next;
 	  cval++;
      }
@@ -680,7 +684,7 @@ LDAPMod *bervalLDAPMod(struct formfill *form,
      return mod;
 }
 
-/* 
+/*
    Local Variables:
    c-basic-offset: 5
    End:
