@@ -22,13 +22,16 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: tinput.c,v 1.47 2006/04/09 20:34:32 herzi Exp $ */
+/* $Id: tinput.c 975 2006-09-07 18:44:41Z herzi $ */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include <lber.h>
@@ -47,14 +50,12 @@
 #include "tinput.h"
 #include "formfill.h"
 #include "input.h"
-#include "i18n.h"
-
 
 /*
  * fetch objectclasses in template from server and build GList of
  * formfill with it
  */
-GList *formfill_from_template(int error_context, struct ldapserver *server,
+GList *formfill_from_template(int error_context, GqServer *server,
 			      struct gq_template *template)
 {
      GList *formlist, *oclist;
@@ -95,7 +96,7 @@ GList *formfill_from_template(int error_context, struct ldapserver *server,
  * fetch objectclasses in entry from server and build GList of formfill with it
  */
 GList *formfill_from_entry_objectclass(int error_context, 
-				       struct ldapserver *server, 
+				       GqServer *server, 
 				       const char *dn)
 {
      GList *formlist, *oclist, *tmplist;
@@ -227,7 +228,7 @@ static GList *add_oc_and_superiors(GList *oc_list, struct server_schema *ss,
      return oc_list;
 }
 
-GList *add_attrs_by_oc(int error_context, struct ldapserver *server,
+GList *add_attrs_by_oc(int error_context, GqServer *server,
 		       GList *oclist)
 {
      GList *formlist, *ocs_to_add = NULL, *l;
@@ -245,12 +246,12 @@ GList *add_attrs_by_oc(int error_context, struct ldapserver *server,
 
      /* add the objectclass attribute in manually */
      form = new_formfill();
-     assert(form);
+     g_assert(form);
 
-     form->server = server;
-     ldapserver_ref(server);
+     form->server = g_object_ref(server);
 
-     strcpy(form->attrname, "objectClass");  /* Flawfinder: ignore */
+     g_free(form->attrname);
+     form->attrname = g_strdup("objectClass");
      form->flags |= FLAG_MUST_IN_SCHEMA;
      set_displaytype(error_context, server, form);
 
@@ -273,17 +274,17 @@ GList *add_attrs_by_oc(int error_context, struct ldapserver *server,
 
 	       /* required attributes */
 	       for(i=0; oc->oc_at_oids_must && oc->oc_at_oids_must[i]; i++) {
-/*  	            if (strcasecmp(oc->oc_at_oids_must[i], "objectClass")) { */
+/*	            if (strcasecmp(oc->oc_at_oids_must[i], "objectClass")) { */
 	            if (lookup_attribute_using_schema(formlist,
 						      oc->oc_at_oids_must[i],
 						      ss, &at) == NULL) {
 		         form = new_formfill();
-			 assert(form);
+			 g_assert(form);
 
-			 form->server = server;
-			 ldapserver_ref(server);
+			 form->server = g_object_ref(server);
 
-		         strncpy(form->attrname, oc->oc_at_oids_must[i], MAX_ATTR_LEN);
+			 g_free(form->attrname);
+			 form->attrname = g_strdup(oc->oc_at_oids_must[i]);
 			 form->flags |= FLAG_MUST_IN_SCHEMA;
 			 if (at && at->at_single_value) {
 			      form->flags |= FLAG_SINGLE_VALUE;
@@ -299,17 +300,17 @@ GList *add_attrs_by_oc(int error_context, struct ldapserver *server,
 	       /* allowed attributes */
 	       i = 0;
 	       while(oc->oc_at_oids_may && oc->oc_at_oids_may[i]) {
-/*  		    if (strcasecmp(oc->oc_at_oids_may[i], "objectClass")) { */
+/*		    if (strcasecmp(oc->oc_at_oids_may[i], "objectClass")) { */
 	            if (lookup_attribute_using_schema(formlist,
 						      oc->oc_at_oids_may[i],
 						      ss, &at) == NULL) {
 		         form = new_formfill();
-			 assert(form);
+			 g_assert(form);
 
-			 form->server = server;
-			 ldapserver_ref(server);
+			 form->server = g_object_ref(server);
 
-		         strncpy(form->attrname, oc->oc_at_oids_may[i], MAX_ATTR_LEN);
+			 g_free(form->attrname);
+			 form->attrname = g_strdup(oc->oc_at_oids_may[i]);
 			 set_displaytype(error_context, server, form);
 		         formlist = formlist_append(formlist, form);
 			 if (at && at->at_single_value) {
@@ -332,7 +333,7 @@ GList *add_attrs_by_oc(int error_context, struct ldapserver *server,
 /*
  * adds all attributes allowed in schema to formlist
  */
-GList *add_schema_attrs(int error_context, struct ldapserver *server,
+GList *add_schema_attrs(int error_context, GqServer *server,
 			GList *value_list)
 {
      GList *oclist, *schema_list, *tmplist, *tmpvallist, *addendum;
@@ -396,11 +397,9 @@ GList *add_schema_attrs(int error_context, struct ldapserver *server,
 	  if(newform == NULL) {
 	       /* attribute type not in schema */
 	       newform = new_formfill();
-	       newform->server = server;
-	       ldapserver_ref(server);
+	       newform->server = g_object_ref(server);
 
-	       strncpy(newform->attrname, form->attrname,
-		       sizeof(newform->attrname));
+	       newform->attrname = g_strdup(form->attrname);
 	       newform->num_inputfields = form->num_inputfields;
 	       newform->displaytype = form->displaytype;
 	       newform->dt_handler = form->dt_handler;
