@@ -22,9 +22,10 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: util.c,v 1.96 2006/04/27 15:25:26 herzi Exp $ */
+/* $Id: util.c 975 2006-09-07 18:44:41Z herzi $ */
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -34,7 +35,9 @@
 #include <pwd.h>
 #include <errno.h>
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #include <lber.h>
 #include <ldap.h>
@@ -53,32 +56,32 @@
 #include <sys/time.h>
 #endif
 
+#include <glade/glade.h>
+
 #include "common.h"
-#include "util.h"
 #include "configfile.h"
 #include "errorchain.h"
+#include "gq-keyring.h"
+#include "gq-server-list.h"
+#include "util.h"
 #include "template.h"
 #include "debug.h"
 #include "schema.h"
 #include "encode.h"
-#include "i18n.h"
 #include "mainwin.h"
 #include "input.h"
 #include "mainwin.h"		/* message_log_append */
-#include "utf8-compat.h"
-
-#include "../icons/warning.xpm"
 
 #define TRY_VERSION3 1
 
 LDAP *open_connection_ex(int open_context,
-			 struct ldapserver *server, int *ldap_errno);
+			 GqServer *server, int *ldap_errno);
 
 #ifdef HAVE_SASL
 static int util_ldap_sasl_interact(LDAP *ld, unsigned flags, void *defaults, void *in)
 {
      sasl_interact_t *interact = in;
-     struct ldapserver *def = defaults;
+     GqServer *def = defaults;
      
      for (; interact->id != SASL_CB_LIST_END; interact++) {
 	
@@ -89,8 +92,16 @@ static int util_ldap_sasl_interact(LDAP *ld, unsigned flags, void *defaults, voi
 			     break;
 	
 		     case SASL_CB_PASS:
-			     interact->result = def->bindpw;
-			     interact->len = strlen(def->bindpw);
+			      if (def->ask_pw) {
+				      if (def->enteredpw[0]) {
+					      interact->result = def->enteredpw;
+					      interact->len = strlen(def->enteredpw);
+				      }
+			      }
+			      else if (def->bindpw[0]) {
+				      interact->result = def->bindpw;
+				      interact->len = strlen(def->bindpw);
+			      }
 			     break;
 	     }
      }
@@ -99,7 +110,7 @@ static int util_ldap_sasl_interact(LDAP *ld, unsigned flags, void *defaults, voi
 #endif
 
 static int
-do_ldap_auth(LDAP *ld, struct ldapserver *server, int open_context) {
+do_ldap_auth(LDAP *ld, GqServer *server, int open_context) {
 	char *binddn = NULL;
 	char *bindpw = NULL;
 	int rc = LDAP_SUCCESS;
@@ -164,7 +175,7 @@ do_ldap_auth(LDAP *ld, struct ldapserver *server, int open_context) {
 	return rc;
 }
 
-static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
+static int do_ldap_connect(LDAP **ld_out, GqServer *server,
 			   int open_context, int flags)
 {
      LDAP *ld = NULL;
@@ -239,8 +250,8 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 #if defined(HAVE_TLS)
 	       {
 		    if (server->version != LDAP_VERSION3) {
-			 error_push(open_context, 
-				    _("Server '%s': Couldn't set protocol version to LDAPv3."), 
+			 error_push(open_context,
+				    _("Server '%s': Couldn't set protocol version to LDAPv3."),
 				    server->name);
 		    }
 	       }
@@ -248,7 +259,7 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 	       /* Let's turn on TLS */
 	       rc = ldap_start_tls_s(ld, NULL, NULL);
 	       if(rc != LDAP_SUCCESS) {
-		    error_push(open_context, 
+		    error_push(open_context,
 			       _("Couldn't enable TLS on the LDAP connection to '%1$s': %2$s"),
 			       server->name,
 			       ldap_err2string(rc));
@@ -258,7 +269,7 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 		    return rc;
 	       }
 #else
-	       error_push(open_context, 
+	       error_push(open_context,
 			  _("Cannot use TLS with server '%s'. TLS was not found to be supported by your LDAP libraries.\n"
 			    "See README.TLS for more information.\n"),
 			  server->name);
@@ -283,8 +294,7 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 
 	  if (rc != LDAP_SUCCESS) {
 	       /* Maybe we cannot use LDAPv3 ... try again */
-
-	       error_push(open_context, 
+	       error_push(open_context,
 			  _("Couldn't bind LDAP connection to '%1$s': %2$s"),
 			  server->name, ldap_err2string(rc));
 	       push_ldap_addl_error(ld, open_context);
@@ -302,45 +312,11 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 	  }
 
      }
-#if 0
-     /* 
 
-     MANAGEDSAIT is ALWAYS needed for browse mode and MAY BE needed for 
-     search mode.
-
-     This means that a global setting of this control is not possible,
-     it has to be specified at the point where the search gets
-     performed through ldap_search_ext.
-
-     */
-
-     if (ld && server->show_ref) {
-	  LDAPControl c;
-	  LDAPControl *ctrls[2];
-	  ctrls[0] = &c;
-	  ctrls[1] = NULL;
-	  
-	  c.ldctl_oid = LDAP_CONTROL_MANAGEDSAIT;
-	  c.ldctl_value.bv_val = NULL;
-	  c.ldctl_value.bv_len = 0;
-	  c.ldctl_iscritical = 1;
-	  
-	  rc = ldap_set_option(ld, LDAP_OPT_SERVER_CONTROLS, ctrls);
-	  
-	  if (rc != LDAP_OPT_SUCCESS ) {
-	       error_push(open_context,
-			  _("Could not set critical ManageDSAit control on connection to server '%s'."), server->name);
-	       push_ldap_addl_error(ld, open_context);
-
-	       ldap_unbind(ld);
-	       ld = NULL;
-	  }
-     }
-#endif
      /* experimental referral stuff ... */
      if (ld) {
 	  rc = ldap_set_option(ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF);
-	  
+
 	  if (rc != LDAP_OPT_SUCCESS ) {
 	       error_push(open_context,
 			  _("Could not disable following referrals on connection to server '%s'."), server->name);
@@ -349,7 +325,7 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 	       /* this is not a fatal error, so continue */
 	  }
      }
-     
+
      *ld_out = ld;
      return rc;
 }
@@ -368,69 +344,70 @@ static int do_ldap_connect(LDAP **ld_out, struct ldapserver *server,
 
     3) An anonymous bind gets attempted.
   */
-struct ldapserver *get_referral_server(int error_context, 
-				       struct ldapserver *parent, 
-				       const char *base_url)
+GqServer*
+get_referral_server(int error_context,
+		    GqServer *parent,
+		    const char *base_url)
 {
      LDAPURLDesc *desc = NULL;
-     struct ldapserver *newserver = NULL, *s;
+     GqServer *newserver = NULL, *s;
      int ld_err;
 
-     assert(parent);
+     g_assert(parent);
 
      if (ldap_url_parse(base_url, &desc) == 0) {
 	  GString *new_uri = g_string_sized_new(strlen(base_url));
-	  g_string_sprintf(new_uri, "%s://%s:%d/", 
+	  g_string_sprintf(new_uri, "%s://%s:%d/",
 			   desc->lud_scheme,
 			   desc->lud_host,
 			   desc->lud_port);
 
-	  newserver = new_ldapserver();
+	  newserver = gq_server_new();
 
 	  if (! config->never_leak_credentials) {
 	       copy_ldapserver(newserver, parent);
-	       
+
 	       g_free_and_dup(newserver->name,     new_uri->str);
 	       g_free_and_dup(newserver->ldaphost, new_uri->str);
 	       g_free_and_dup(newserver->basedn,   desc->lud_dn);
-	       
+
 	       /* some sensible settings for the "usual" case:
 		  Anonymous bind. Also show referrals */
 	       newserver->ask_pw   = 0;
 	       newserver->show_ref = 1;
 	       newserver->quiet    = 1;
-	       
-	       if (open_connection_ex(error_context, 
+
+	       if (open_connection_ex(error_context,
 				      newserver, &ld_err)) {
 		    close_connection(newserver, FALSE);
-		    
+
 		    statusbar_msg(_("Initialized temporary server-definition '%1$s' from existing server '%2$s'"), new_uri->str, parent->name);
-		    
+
 		    goto done;
 	       }
 	       if (ld_err == LDAP_SERVER_DOWN) {
 		    goto done;
 	       }
-	       
+
 	       reset_ldapserver(newserver);
 
 	       /* check: do we have this server around already??? */
 	       s = server_by_canon_name(new_uri->str, TRUE);
-	       
+
 	       if (s) {
 		    copy_ldapserver(newserver, s);
-		    
+
 		    g_free_and_dup(newserver->name,     new_uri->str);
 		    g_free_and_dup(newserver->ldaphost, new_uri->str);
 		    g_free_and_dup(newserver->basedn,   desc->lud_dn);
-		    
+
 		    /* some sensible settings for the "usual" case:
 		       Anonymous bind. Also show referrals */
 		    newserver->ask_pw   = 0;
 		    newserver->show_ref = 1;
 		    newserver->quiet    = 1;
-		    
-		    if (open_connection_ex(error_context, 
+
+		    if (open_connection_ex(error_context,
 					   newserver, &ld_err)) {
 			 close_connection(newserver, FALSE);
 			 statusbar_msg(_("Initialized temporary server-definition '%1$s' from existing server '%2$s'"), new_uri->str, s->name);
@@ -477,13 +454,109 @@ struct ldapserver *get_referral_server(int error_context,
      return newserver;
 }
 
+static gboolean
+delete_on_escape(GtkDialog* dialog, GdkEventKey* ev) {
+	if(ev->keyval == GDK_Escape) {
+		gtk_dialog_response(dialog, GTK_RESPONSE_DELETE_EVENT);
+		return TRUE;
+	}
+	return FALSE;
+}
 
+static gchar*
+get_password_from_dialog(GqServer* server) {
+	GladeXML    * xml = NULL;
+	GtkSizeGroup* group_left  = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+	GtkSizeGroup* group_right = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+	GtkWidget   * old_grab = gtk_grab_get_current();
+	gchar const * password = NULL;
+	gchar       * string = NULL;
+
+	xml = glade_xml_new(PACKAGE_PREFIX "/share/gq/gq.glade", "password_dialog", NULL);
+	if(!xml) {
+		GtkWidget* dlg = gtk_message_dialog_new(NULL, // FIXME: set modal parent
+							GTK_DIALOG_MODAL,
+							GTK_MESSAGE_ERROR,
+							GTK_BUTTONS_CLOSE,
+							"%s", _("The user interface definition could not be "
+								"loaded, please make sure that GQ is correctly "
+								"installed."));
+		gtk_dialog_run(GTK_DIALOG(dlg));
+		gtk_widget_hide(dlg);
+		gtk_widget_destroy(dlg);
+	}
+
+	// FIXME: the GtkCTree has got the pointer grabbed
+	if(old_grab) {
+		gtk_widget_hide(old_grab);
+		gtk_widget_show(old_grab);
+	}
+
+	g_signal_connect(glade_xml_get_widget(xml, "password_dialog"), "key-press-event",
+			 G_CALLBACK(delete_on_escape), NULL);
+
+	gtk_size_group_add_widget(group_left, glade_xml_get_widget(xml, "label_hostname"));
+	gtk_size_group_add_widget(group_left, glade_xml_get_widget(xml, "label_bind_dn"));
+	gtk_size_group_add_widget(group_left, glade_xml_get_widget(xml, "label_bind_mode"));
+	gtk_size_group_add_widget(group_left, glade_xml_get_widget(xml, "label_password"));
+	gtk_size_group_add_widget(group_right, glade_xml_get_widget(xml, "input_hostname"));
+	gtk_size_group_add_widget(group_right, glade_xml_get_widget(xml, "input_bind_dn"));
+	gtk_size_group_add_widget(group_right, glade_xml_get_widget(xml, "input_bind_mode"));
+	gtk_size_group_add_widget(group_right, glade_xml_get_widget(xml, "input_password"));
+
+	if(!gnome_keyring_is_available()) {
+		gtk_widget_hide(glade_xml_get_widget(xml, "checkbutton_save_password"));
+	}
+
+	if(server->name && server->ldaphost &&
+	   (!strcmp(server->name, server->ldaphost))) {
+		// Server Name and Hostname are equal, do not print twice
+		// TRANSLATORS: "hostname:port"
+		string = g_strdup_printf(_("%s:%d"), server->name, server->ldapport);
+	} else {
+		// TRANSLATORS: "servername (hostname:port)"
+		string = g_strdup_printf(_("%s (%s:%d)"), server->name, server->ldaphost, server->ldapport);
+	}
+	gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xml, "input_hostname")), string);
+	g_free(string);
+
+	gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xml, "input_bind_dn")), server->binddn);
+
+	// TRANSLATORS: "(Simple,Kerberos,SASL) Authentication"
+	string = g_strdup_printf(_("%s Authentication"), token_bindtype[server->bindtype].keyword);
+	// FIXME: get this translatable as a whole text
+	gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xml, "input_bind_mode")), string);
+	g_free(string);
+	string = NULL;
+
+	switch(gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "password_dialog")))) {
+	case GTK_RESPONSE_OK:
+		g_free_and_dup(server->bindpw, gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "input_password"))));
+
+		// FIXME: store password only from successful connections
+		if(gnome_keyring_is_available() &&
+		   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "checkbutton_save_password")))) {
+			gq_keyring_save_server_password(server);
+		}
+		break;
+	case GTK_RESPONSE_HELP:
+		// FIXME: add documentation and something about the login dialog
+	default:
+		break;
+	}
+	gtk_widget_destroy(GTK_WIDGET(glade_xml_get_widget(xml, "password_dialog")));
+	g_object_unref(xml);
+	g_object_unref(group_left);
+	g_object_unref(group_right);
+
+	return string;
+}
 
 /*
  * open connection to LDAP server, and store connection for caching
  */
-LDAP *open_connection_ex(int open_context,
-			 struct ldapserver *server, int *ldap_errno)
+LDAP*
+open_connection_ex(int open_context, GqServer *server, int *ldap_errno)
 {
      LDAP *ld;
      int rc;
@@ -501,52 +574,52 @@ LDAP *open_connection_ex(int open_context,
 	       return(server->connection);
 	  else {
 	       /* do not leak file descriptors in case we need to
-                  "rebind" */
+                * "rebind" */
 	       ldap_unbind(server->connection);
 	       server->connection = NULL;
 	  }
      }
 
      if (server->ask_pw &&
-	 server->binddn[0] != 0 && /* makes sense only if we bind as someone */
+	 server->binddn[0] && /* makes sense only if we bind as someone */
 	 /*	 server->bindpw[0] == 0 &&  */
 	 server->enteredpw[0] == 0) {
 	  char *ret = NULL;
-	  message = g_string_sized_new(128);
-	  g_string_sprintf(message,
-			   _("Password for server %s"), server->name);
-	  
+
 	  if (server->quiet) {
 	       return NULL;
 	  }
 
-	  if (query_popup(message->str, &ret, TRUE /* is_password */,
-			  NULL)) { /* FIXME: somehow get modal_for widget */
-	       g_string_free(message, TRUE);
+	  ret = gq_keyring_get_password(server);
+
+	  if(!ret) {
+		  ret = get_password_from_dialog(server);
+	  }
+
+	  if (ret) { /* FIXME: somehow get modal_for widget */
 	       if (ret) {
-		    g_free_if(server->enteredpw);
+		    g_free(server->enteredpw);
 		    server->enteredpw = ret;
 		    newpw = 1;
 	       } else {
 		    server->enteredpw[0] = 0;
 	       }
 	  } else {
-	       g_string_free(message, TRUE);
 	       return NULL;
 	  }
      }
-     
+
      if(server->ldapport == 389) {
 	  statusbar_msg(_("Connecting to %s"), server->ldaphost);
      } else {
-	  statusbar_msg(_("Connecting to %1$s port %2$d"), 
+	  statusbar_msg(_("Connecting to %1$s port %2$d"),
 			server->ldaphost, server->ldapport);
      }
 
      ld = NULL;
 /*      open_context = error_new_context(_("Error connecting to server")); */
 
-     rc = do_ldap_connect(&ld, server, open_context, 
+     rc = do_ldap_connect(&ld, server, open_context,
 			  TRY_VERSION3);
      if (rc == LDAP_PROTOCOL_ERROR) {
 	  /* this might be an indication that the server does not
@@ -554,7 +627,7 @@ LDAP *open_connection_ex(int open_context,
 	  rc = do_ldap_connect(&ld, server, open_context, 0);
 	  if (rc == LDAP_SUCCESS) error_clear(open_context);
      } else if (rc == LDAP_INVALID_CREDENTIALS) {
-	  /* Should the server configuration pop up? */	  
+	  /* Should the server configuration pop up? */
      }
 
      /* this is not quite OK: FIXME */
@@ -570,7 +643,7 @@ LDAP *open_connection_ex(int open_context,
      return(ld);
 }
 
-LDAP *open_connection(int open_context, struct ldapserver *server)
+LDAP *open_connection(int open_context, GqServer *server)
 {
      return open_connection_ex(open_context, server, NULL);
 }
@@ -580,43 +653,25 @@ LDAP *open_connection(int open_context, struct ldapserver *server)
  * if caching is enabled for the server. Set always to TRUE if you want to
  * close the connection regardless of caching.
  */
-void close_connection(struct ldapserver *server, int always)
+void
+close_connection(GqServer *server, int always)
 {
-     server->missing_closes--;
-/*      if (server->missing_closes < 0) abort(); */
-
-     if( server->connection &&
-	 ( (server->missing_closes == 0 && !server->cacheconn)
-	   || always) ) {
-
-#if 0
-	  /*
-	   * DO NOT clear the schema - we might be disconnecting
-	   * because we are not caching the connection.
-	   * 
-	   * This fixes a bug reported by Stephan Duehr <stephand@suse.de>
-	   *
-	   * BUT: FIXME: Think about how to deal with schema changes
-	   * due to server restarts (Popup menu on the schema top
-	   * nodes [ie. resurrect the refresh popup already there]?)
-	   * 
-	   */
- 
-	  clear_server_schema(server);
-#endif
-	  /* definitely close this connection */
-	  ldap_unbind(server->connection);
-	  server->connection = NULL;
-/*  	  server->missing_closes = 0; */
-     }
+	server->missing_closes--;
 
+	if(server->connection &&
+	   ((server->missing_closes == 0 && !server->cacheconn) || always))
+	{
+		/* definitely close this connection */
+		ldap_unbind(server->connection);
+		server->connection = NULL;
+	}
 }
 
 
 /*
  * clear cached server schema
  */
-void clear_server_schema(struct ldapserver *server)
+void clear_server_schema(GqServer *server)
 {
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
      GList *list;
@@ -679,7 +734,7 @@ void clear_server_schema(struct ldapserver *server)
  * delete entry
  */
 gboolean delete_entry_full(int delete_context,
-			   struct ldapserver *server, char *dn,
+			   GqServer *server, char *dn,
 			   gboolean recursive)
 {
      LDAP *ld;
@@ -783,13 +838,13 @@ gboolean delete_entry_full(int delete_context,
 
 
 gboolean delete_entry(int delete_context,
-		      struct ldapserver *server, char *dn) 
+		      GqServer *server, char *dn) 
 {
      return delete_entry_full(delete_context, server, dn, FALSE);
 }
 
 gboolean do_recursive_delete(int delete_context, 
-			     struct ldapserver *server, char* dn)
+			     GqServer *server, char* dn)
 {
      return delete_entry_full(delete_context, server, dn, TRUE);
 }
@@ -825,15 +880,11 @@ int close_on_esc(GtkWidget *widget, GdkEventKey *event, gpointer obj)
 {
      if(event && event->type == GDK_KEY_PRESS && event->keyval == GDK_Escape) {
 	  gtk_widget_destroy(GTK_WIDGET(obj));
-	  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+	  g_signal_stop_emission_by_name(widget, "key_press_event");
 	  return(TRUE);
      }
 
-#if GTK_MAJOR >= 2
      return(FALSE);
-#else
-     return(TRUE);  /* may be a bug to return TRUE unconditionally in gtk 1.2 */
-#endif
 }
 
 
@@ -847,15 +898,11 @@ int func_on_esc(GtkWidget *widget, GdkEventKey *event, GtkWidget *window)
      if(event && event->type == GDK_KEY_PRESS && event->keyval == GDK_Escape) {
 	  func = gtk_object_get_data(GTK_OBJECT(window), "close_func");
 	  func(widget);
-	  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+	  g_signal_stop_emission_by_name(widget, "key_press_event");
 	  return(TRUE);
      }
 
-#if GTK_MAJOR >= 2
      return(FALSE);
-#else
-     return(TRUE);  /* may be a bug to return TRUE unconditionally in gtk 1.2 */
-#endif
 }
 
 
@@ -934,7 +981,7 @@ void statusbar_msg(const char *fmt, ...)
 	       *buf = 0;
 	       
 	       va_start(ap, fmt);
-	       n = vsnprintf(buf, len, fmt, ap);
+	       n = g_vsnprintf(buf, len, fmt, ap);
 	       va_end(ap);
 	       
 	       if (n > len || n == -1) {
@@ -963,7 +1010,7 @@ void statusbar_msg(const char *fmt, ...)
 void statusbar_msg_clear()
 {
      if(!context)
-	  context = 
+	  context =
 	       gtk_statusbar_get_context_id(GTK_STATUSBAR(mainwin.statusbar),
 					    "mainwin");
      if(msgid)
@@ -971,97 +1018,41 @@ void statusbar_msg_clear()
 			       context, msgid);
 }
 
-#if 0  /* not needed anymore */
 /*
- * display message in main window's statusbar, and flushes the
- * GTK event queue, UTF-8 version
- *
+ * return pointer to GqServer matching the canonical name
  */
-void statusbar_msg_utf8(const char *message)
-{
-#if GTK_MAJOR >= 2
-     statusbar_msg(message);
-#else
-     int len = strlen(message);
-     char *buf = g_malloc(len + 10);
-     
-     decode_string(buf, message, len);
-
-     if(!context)
-	  context = gtk_statusbar_get_context_id(GTK_STATUSBAR(mainwin.statusbar),
-						 "mainwin");
-     if(msgid)
-	  gtk_statusbar_remove(GTK_STATUSBAR(mainwin.statusbar), context, msgid);
-
-     msgid = gtk_statusbar_push(GTK_STATUSBAR(mainwin.statusbar), context, buf);
-
-     /* make sure statusbar gets updated right away */
-     while(gtk_events_pending())
-	  gtk_main_iteration();
-
-     g_free(buf);
-#endif
-}
-#endif
-
-/*
- * return pointer to (struct ldapserver *) matching name
- */
-struct ldapserver *server_by_name(const char *name)
-{
-     struct ldapserver *server;
-     GList *I;
-     
-     if(name == NULL || name[0] == '\0')
-	  return(NULL);
-
-     for (I = config->servers ; I ; I = g_list_next(I)) {
-	  server = (struct ldapserver *) I->data;
-	  if(!strcmp(server->name, name))
-	       return(server);
-     }
-
-     return(NULL);
-}
-
-
-/*
- * return pointer to (struct ldapserver *) matching the canonical name
- */
-struct ldapserver *server_by_canon_name(const char *name, 
+GqServer *server_by_canon_name(const char *name,
 					gboolean include_transient)
 {
-     struct ldapserver *server;
+     GqServer *server = NULL;
      GList *I;
-     
+
      if(name == NULL || name[0] == '\0')
 	  return NULL;
 
-     for (I = config->servers ; I ; I = g_list_next(I)) {
-	  server = (struct ldapserver *) I->data;
-	  if(!strcmp(server->canon_name, name))
-	       return(server);
-     }
-
-     if (include_transient) {
+     server = gq_server_list_get_by_canon_name(gq_server_list_get(), name);
+     if(!server && include_transient) {
 	  for (I = transient_servers ; I ; I = g_list_next(I)) {
-	       server = (struct ldapserver *) I->data;
-	       if(!strcmp(server->canon_name, name))
-		    return(server);
+	       if(!strcmp(((GqServer*)I->data)->canon_name, name)) {
+		    server = I->data;
+		    break;
+	       }
 	  }
      }
-     return NULL;
+     return server;
 }
 
-gboolean is_transient_server(const struct ldapserver *server) {
-     return g_list_find(config->servers, (void*) server) == NULL;
+gboolean
+is_transient_server(const GqServer *server) {
+	g_return_val_if_fail(GQ_IS_SERVER(server), FALSE);
+	return !gq_server_list_contains(gq_server_list_get(), server);
 }
 
 /*
  * check if entry has a subtree
  */
-int is_leaf_entry(int error_context, 
-		  struct ldapserver *server, char *dn)
+int is_leaf_entry(int error_context,
+		  GqServer *server, char *dn)
 {
      LDAP *ld;
      LDAPMessage *res;
@@ -1186,8 +1177,6 @@ void warning_popup(GList *messages)
      GtkWidget *window, *vbox1, *vbox2, *vbox3, *label, *hbox0;
      GtkWidget *hbox1, *ok_button;
      GtkWidget *pixmap;
-     GdkPixmap *warning;
-     GdkBitmap *warning_mask;
 
      window = gtk_dialog_new();
      gtk_container_border_width(GTK_CONTAINER(window), CONTAINER_BORDER_WIDTH);
@@ -1199,11 +1188,7 @@ void warning_popup(GList *messages)
      hbox1 = gtk_hbox_new(FALSE, 0);
      gtk_widget_show(hbox1);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 10);
-     warning = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mainwin.mainwin)->window,
-					    &warning_mask,
-					    &mainwin.mainwin->style->white,
-					    warning_xpm);
-     pixmap = gtk_pixmap_new(warning, warning_mask);
+     pixmap = gtk_image_new_from_file(PACKAGE_PREFIX "/share/pixmaps/gq/warning.xpm");
      gtk_widget_show(pixmap);
      gtk_box_pack_start(GTK_BOX(hbox1), pixmap, TRUE, TRUE, 10);
 
@@ -1228,27 +1213,24 @@ void warning_popup(GList *messages)
      gtk_widget_show(hbox0);
      gtk_box_pack_start(GTK_BOX(vbox3), hbox0, TRUE, TRUE, 0);
 
-#if GTK_MAJOR >= 2
      ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
-#else
-     ok_button = gq_button_new_with_label(_("_OK"));
-#endif
-     gtk_signal_connect_object(GTK_OBJECT(ok_button), "clicked",
-			       (GtkSignalFunc) gtk_widget_destroy,
-			       (gpointer) window);
+
+     g_signal_connect_swapped(ok_button, "clicked",
+			       G_CALLBACK(gtk_widget_destroy),
+			       window);
      gtk_box_pack_start(GTK_BOX(hbox0), ok_button, FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
      gtk_widget_grab_default(ok_button);
      gtk_widget_show(ok_button);
 
      /* what does this mean? PS: 20030928 - FIXME */
-     gtk_signal_connect_object(GTK_OBJECT(window), "destroy",
-			       (GtkSignalFunc) gtk_widget_destroy,
-			       (gpointer) window);
+     g_signal_connect_swapped(window, "destroy",
+			       G_CALLBACK(gtk_widget_destroy),
+			       window);
 
-     gtk_signal_connect_object(GTK_OBJECT(window), "key_press_event",
-                               GTK_SIGNAL_FUNC(close_on_esc),
-                               (gpointer) window);
+     g_signal_connect_swapped(window, "key_press_event",
+                               G_CALLBACK(close_on_esc),
+                               window);
 
      gtk_widget_show(window);
 
@@ -1268,7 +1250,7 @@ void single_warning_popup(char *message)
 
 
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
-GList *find_at_by_mr_oid(struct ldapserver *server, const char *oid)
+GList *find_at_by_mr_oid(GqServer *server, const char *oid)
 {
      GList *list, *srvlist;
      LDAPAttributeType *at;
@@ -1315,7 +1297,7 @@ LDAPAttributeType *find_canonical_at_by_at(struct server_schema *schema,
      return NULL;
 }
 
-GList *find_at_by_s_oid(struct ldapserver *server, const char *oid)
+GList *find_at_by_s_oid(GqServer *server, const char *oid)
 {
      GList *list, *srvlist;
      LDAPAttributeType *at;
@@ -1338,7 +1320,7 @@ GList *find_at_by_s_oid(struct ldapserver *server, const char *oid)
 }
 
 
-GList *find_mr_by_s_oid(struct ldapserver *server, const char *oid)
+GList *find_mr_by_s_oid(GqServer *server, const char *oid)
 {
      GList *list, *srvlist;
      LDAPMatchingRule *mr;
@@ -1362,7 +1344,7 @@ GList *find_mr_by_s_oid(struct ldapserver *server, const char *oid)
 
 
 GList *find_oc_by_at(int error_context,
-		     struct ldapserver *server, const char *atname)
+		     GqServer *server, const char *atname)
 {
      GList *list, *oclist;
      LDAPObjectClass *oc;
@@ -1415,7 +1397,7 @@ GList *find_oc_by_at(int error_context,
      return(list);
 }
 
-const char *find_s_by_at_oid(int error_context, struct ldapserver *server,
+const char *find_s_by_at_oid(int error_context, GqServer *server,
 			     const char *oid)
 {
      GList *srvlist;
@@ -1462,7 +1444,7 @@ static struct oid2syntax_t oid2syntax[] = {
      { NULL, NULL },
 };
 
-const char *find_s_by_at_oid(struct ldapserver *server, const char *oid)
+const char *find_s_by_at_oid(GqServer *server, const char *oid)
 {
      struct oid2syntax_t *os;
      for (os = oid2syntax ; os->oid ; os++) {
@@ -1533,13 +1515,13 @@ static struct popup_comm *new_popup_comm()
 
 static void free_popup_comm(struct popup_comm *c)
 {
-     assert(c);
+     g_assert(c);
      g_free(c);
 }
 
 /* gtk2 checked (multiple destroy callbacks safety), confidence 0.7 */
 static void query_destroy(GtkWidget *window, struct popup_comm *comm) {
-     assert(comm);
+     g_assert(comm);
      if (!comm) return;
      if (!comm->ended)
 	  gtk_main_quit(); /* quits only nested main loop */
@@ -1572,7 +1554,7 @@ int query_popup(const char *title, gchar **inout_buf, gboolean is_password,
      int rc;
      GtkWidget *f = gtk_grab_get_current();
      struct popup_comm *comm = new_popup_comm();
-     
+
      /* This is a BAD hack - it solves a problem with the query popup
         dialog that locks up focus handling with all the
         window-managers I have been able to test this with. Actually,
@@ -1592,12 +1574,12 @@ int query_popup(const char *title, gchar **inout_buf, gboolean is_password,
      gtk_container_border_width(GTK_CONTAINER(window), CONTAINER_BORDER_WIDTH);
      gtk_window_set_title(GTK_WINDOW(window), title);
      gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
-     gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			(GtkSignalFunc) query_destroy,
-			(gpointer) comm);
-     gtk_signal_connect_object(GTK_OBJECT(window), "key_press_event",
-                               GTK_SIGNAL_FUNC(close_on_esc),
-                               (gpointer) window);
+     g_signal_connect(window, "destroy",
+			G_CALLBACK(query_destroy),
+			comm);
+     g_signal_connect_swapped(window, "key_press_event",
+                               G_CALLBACK(close_on_esc),
+                               window);
 
      if (modal_for) {
 	  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
@@ -1628,8 +1610,8 @@ int query_popup(const char *title, gchar **inout_buf, gboolean is_password,
      }
 
      gtk_widget_show(inputbox);
-     gtk_signal_connect(GTK_OBJECT(inputbox), "activate",
-			GTK_SIGNAL_FUNC(query_ok), comm);
+     g_signal_connect(inputbox, "activate",
+			G_CALLBACK(query_ok), comm);
      gtk_box_pack_end(GTK_BOX(vbox1), inputbox, TRUE, TRUE, 5);
 
      vbox2 = GTK_DIALOG(window)->action_area;
@@ -1640,28 +1622,20 @@ int query_popup(const char *title, gchar **inout_buf, gboolean is_password,
      gtk_widget_show(hbox0);
      gtk_box_pack_start(GTK_BOX(vbox2), hbox0, TRUE, TRUE, 0);
 
-#if GTK_MAJOR >= 2
      button = gtk_button_new_from_stock(GTK_STOCK_OK);
-#else
-     button = gq_button_new_with_label(_("_OK"));
-#endif
 
-     gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(query_ok), comm);
+     g_signal_connect(button, "clicked",
+			G_CALLBACK(query_ok), comm);
      gtk_box_pack_start(GTK_BOX(hbox0), button, FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
      GTK_WIDGET_SET_FLAGS(button, GTK_RECEIVES_DEFAULT);
      gtk_widget_grab_default(button);
      gtk_widget_show(button);
 
-#if GTK_MAJOR >= 2
      button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-#else
-     button = gq_button_new_with_label(_("_Cancel"));
-#endif
 
-     gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(query_cancel), 
+     g_signal_connect(button, "clicked",
+			G_CALLBACK(query_cancel), 
 			comm);
 
      gtk_box_pack_end(GTK_BOX(hbox0), button, FALSE, FALSE, 0);
@@ -1702,11 +1676,9 @@ int question_popup(const char *title, const char *question)
 {
      GtkWidget *window, *hbox0, *hbox1, *vbox1, *hbox2, *label, *button, *pixmap;
      int rc;
-     GdkPixmap *warning;
-     GdkBitmap *warning_mask;
      GtkWidget *f = gtk_grab_get_current();
      struct popup_comm *comm = new_popup_comm();
-     
+
      /* This is a BAD hack - it solves a problem with the query popup
         dialog that locks up focus handling with all the
         window-managers I have been able to test this with. Actually,
@@ -1722,12 +1694,12 @@ int question_popup(const char *title, const char *question)
      gtk_container_border_width(GTK_CONTAINER(window), CONTAINER_BORDER_WIDTH);
      gtk_window_set_title(GTK_WINDOW(window), title);
      gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
-     gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			(GtkSignalFunc) query_destroy,
-			(gpointer) comm);
-     gtk_signal_connect_object(GTK_OBJECT(window), "key_press_event",
-                               GTK_SIGNAL_FUNC(close_on_esc),
-                               (gpointer) window);
+     g_signal_connect(window, "destroy",
+			G_CALLBACK(query_destroy),
+			comm);
+     g_signal_connect_swapped(window, "key_press_event",
+                               G_CALLBACK(close_on_esc),
+                               window);
 
      gtk_widget_show(window);
 
@@ -1737,11 +1709,7 @@ int question_popup(const char *title, const char *question)
      hbox1 = gtk_hbox_new(FALSE, 0);
      gtk_widget_show(hbox1);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 10);
-     warning = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(window)->window,
-                                            &warning_mask,
-                                            &window->style->white,
-                                            warning_xpm);
-     pixmap = gtk_pixmap_new(warning, warning_mask);
+     pixmap = gtk_image_new_from_file(PACKAGE_PREFIX "/share/pixmaps/gq/warning.xpm");
      gtk_widget_show(pixmap);
      gtk_box_pack_start(GTK_BOX(hbox1), pixmap, TRUE, TRUE, 10);
 
@@ -1758,26 +1726,20 @@ int question_popup(const char *title, const char *question)
      gtk_widget_show(hbox0);
      gtk_box_pack_start(GTK_BOX(hbox2), hbox0, TRUE, TRUE, 0);
 
-#if GTK_MAJOR >= 2
      button = gtk_button_new_from_stock(GTK_STOCK_YES);
-#else
-     button = gq_button_new_with_label(_("_Yes"));
-#endif
-     gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(query_ok), comm);
+
+     g_signal_connect(button, "clicked",
+			G_CALLBACK(query_ok), comm);
      gtk_box_pack_start(GTK_BOX(hbox0), button, FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
      GTK_WIDGET_SET_FLAGS(button, GTK_RECEIVES_DEFAULT);
      gtk_widget_grab_default(button);
      gtk_widget_show(button);
 
-#if GTK_MAJOR >= 2
      button = gtk_button_new_from_stock(GTK_STOCK_NO);
-#else
-     button = gq_button_new_with_label(_("_No"));
-#endif
-     gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(query_cancel), 
+
+     g_signal_connect(button, "clicked",
+			G_CALLBACK(query_cancel), 
 			comm);
 
      gtk_box_pack_end(GTK_BOX(hbox0), button, FALSE, FALSE, 0);
@@ -1808,7 +1770,7 @@ int question_popup(const char *title, const char *question)
  * get all suffixes a server considers itself authorative for.
  */
 
-GList *get_suffixes(int error_context, struct ldapserver *server)
+GList *get_suffixes(int error_context, GqServer *server)
 {
      LDAP *ld;
      LDAPMessage *res, *e;
@@ -1969,61 +1931,20 @@ void gq_exploded_free(char **exploded_dn)
 #endif
 
 static GtkWidget *enable_uline(GtkWidget *label) {
-#if GTK_MAJOR >= 2
      gtk_label_set_use_underline(GTK_LABEL(label), TRUE);
-#else
-     gchar *text;
-     gchar *c, *d, *f;
-     gboolean underscore_seen = FALSE;
-
-     gtk_label_get(GTK_LABEL(label), &text);
-     f = d = g_malloc(strlen(text) + 1);
-
-
-     /* PS: AFAIK gtk1 does not support mnemonics out of the
-	box. Thus, for the time being, we will filter out the gtk2
-	style underline notation from labels used to denote the
-	mnemonic */
-     
-     for (c = text ; *c ; c++) {
-	  if (underscore_seen) {
-	       *d++ = *c;
-	       underscore_seen = FALSE;
-	       continue;
-	  } else if (*c == '_') {
-	       underscore_seen = TRUE;
-	       continue;
-	  } else {
-	       *d++ = *c;
-	  }
-     }
-     *d++ = 0;
-
-     gtk_label_set(GTK_LABEL(label), f);
-     g_free(f);
-/*      gtk_label_parse_uline(GTK_LABEL(label), text); */
-#endif
      return label;
 }
 
 static GtkWidget *bin_enable_uline(GtkWidget *w) {
      GtkWidget *l;
-#if GTK_MAJOR >= 2
      l = gtk_bin_get_child(GTK_BIN(w));
-#else
-     l = GTK_BIN(w)->child;
-#endif
      enable_uline(l);
      return w;
 }
 
 
 GtkWidget *gq_label_new(const char *text) {
-#if GTK_MAJOR >= 2
      return gtk_label_new_with_mnemonic(text);
-#else
-     return enable_uline(gtk_label_new(text));
-#endif
 }
 
 
@@ -2050,82 +1971,27 @@ GtkWidget *gq_toggle_button_new_with_label(const gchar *text) {
 }
 
 
-#ifndef HAVE_REORDERABLE_G_STRING_SPRINTF
-
-/* This is a work-around around the fact that g_string_sprintf is
-   unable to deal with positional parameters */
-
-void gq_g_string_sprintf(GString *string, const gchar *format, ...)
-{
-     int s = -1;
-#if GTK_MAJOR >= 2
-     int n = string->allocated_len;
-#else
-     int n = 0;
-#endif
-     va_list ap;
-
-     assert(string);
-/*      printf("gq_g_string_sprintf...\n"); */
-/*      while (s > string->len || s < 0) { */
-     for (;;) {
-	  va_start(ap, format);
-	  s = vsnprintf(string->str, n, format, ap);
-	  if (s == -1) {
-	       n = 2 * n;
-	       if (n < 32) n = 32;
-	  } else if (s > n) {
-	       n = s + 1;
-	  } else {
-	       va_end(ap);
-	       string->len = s; /* duck */
-	       break;
-	  }
-
-/* 	  printf("resizing %d to %d...\n", s, n); */
-	  
-#if GTK_MAJOR >= 2
-	  g_string_set_size(string, n + 1);
-#else
-	  {
-	       /* Uarghhh, what an ugly hack */
-	       char *tmp = g_malloc(n + 1);
-	       memset(tmp, 'x', n);
-	       tmp[n] = 0;
-	       g_string_assign(string, tmp);
-	       g_free(tmp);
-	  }
-#endif
-	  va_end(ap);
-     }
-     return;
-}
-
-#endif /* HAVE_REORDERABLE_G_STRING_SPRINTF */
-
-
 struct dn_on_server *new_dn_on_server(const char *d,
-				      struct ldapserver *s)
+				      GqServer *s)
 {
-     struct dn_on_server *dos = g_malloc0(sizeof(struct dn_on_server)); 
-     dos->server = s;
-     ldapserver_ref(s);
+     struct dn_on_server *dos = g_malloc0(sizeof(struct dn_on_server));
+     dos->server = g_object_ref(s);
 
      if (d) {
 	  dos->dn = g_strdup(d);
      } else {
 	  dos->dn = NULL;
      }
-	  
+
      return dos;
 }
 
 void free_dn_on_server(struct dn_on_server *s)
 {
      if (s) {
-	  g_free_if(s->dn);
-	  ldapserver_unref(s->server);
+	  g_free(s->dn);
 	  s->dn = NULL;
+	  g_object_unref(s->server);
 	  s->server = NULL;
 
 	  g_free(s);
